;; x-symbol-nedpg.el   
;;	Token language "Nedpg Symbols" for package x-symbol
;;
;;  ID:         x-symbol-nedpg.el,v 8.0 2004/04/17 23:40:00 da Exp
;;  Author:     C. Raffalli
;;              Updates by Markus Wenzel, Christoph Wedler, David Aspinall.
;;  Copyright   1998 Technische Universitaet Muenchen
;;  License     GPL (GNU GENERAL PUBLIC LICENSE)
;;
;;  Adapted from x-symbol-isabelle.el
;; NB: Part of Proof General distribution.
;;

;; CW: this sexpr can be deleted with X-Symbol 4.4.3
(eval-when-compile
  ;; Next lines should allow this file to work standalone 
  ;; without proof-x-symbol.el.  See comments further below too.
  (require 'cl)
  (ignore-errors (require 'x-symbol-vars)))

(defvar x-symbol-nedpg-required-fonts nil)
  
;;;===========================================================================
;;;  General language accesses, see `x-symbol-language-access-alist'
;;;===========================================================================

;; NB da: these next two are also set in proof-x-symbol.el, but
;; it would be handy to be able to use this file away from PG.  
;; FIXME: In future could fix things so just 
;; (require 'proof-x-symbol) would be enough here.
(defvar x-symbol-nedpg-name "Nedpg Symbol")
(defvar x-symbol-nedpg-modeline-name "nedpg")

(defcustom x-symbol-nedpg-header-groups-alist nil
   "*If non-nil, used in isasym specific grid/menu.
See `x-symbol-header-groups-alist'."
  :group 'x-symbol-nedpg
  :group 'x-symbol-input-init
  :type 'x-symbol-headers)

(defcustom x-symbol-nedpg-electric-ignore nil
;;  "[:'][A-Za-z]\\|<=\\|\\[\\[\\|\\]\\]\\|~="
  "*Additional Nedpg version of `x-symbol-electric-ignore'."
  :group 'x-symbol-nedpg
  :group 'x-symbol-input-control
  :type 'x-symbol-function-or-regexp)

(defvar x-symbol-nedpg-required-fonts nil
  "List of features providing fonts for language `nedpg'.")

(defvar x-symbol-nedpg-extra-menu-items nil
  "Extra menu entries for language `nedpg'.")


(defvar x-symbol-nedpg-token-grammar
  ;; CW: for X-Symbol-4.4.3:
  ;; '(x-symbol-make-grammar ...)
  (if (fboundp 'x-symbol-make-grammar) ;; x-symbol >=4.3 versions
      (x-symbol-make-grammar
       :encode-spec '(((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")) .
                    ((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")))
       :decode-spec nil
       :decode-regexp "\\([_'a-zA-Z0-9]+\\)\\|\\([]><=\\/~&+-*%!{}:-]+\\)"
       :token-list #'x-symbol-nedpg-default-token-list)))

(defvar x-symbol-nedpg-input-token-grammar
  '("\\([_'a-zA-Z0-9]+\\)\\|\\([]><=\\/~&+-*%!{}:-]+\\)"
     ((id . "[A-Za-z_0-9]") (op . "[<>!+-*/|&]"))
     (id . "[A-Za-z_0-9]") (op . "[<>!+-*/|&]"))
  "Grammar of input method Token for language `nedpg'.")

(defun x-symbol-nedpg-default-token-list (tokens)
  (mapcar 
   (lambda (x)
     (cons x 
	   (cond
	    ;; CW: where are the shapes `id' and `op' used?
	    ((string-match "\\`[A-Za-z_][A-Za-z_0-9]+\\'" x)
	     'id)
	    ((string-match "\\`[<>!+-*/|&]+\\'" x) 
	     'op))))
   tokens))

(defvar x-symbol-nedpg-user-table nil
  "User table defining Nedpg commands, used in `x-symbol-nedpg-table'.")

(defvar x-symbol-nedpg-generated-data nil
  "Internal.")


;;;===========================================================================
;;;  No image support
;;;===========================================================================

(defvar x-symbol-nedpg-master-directory  'ignore)
(defvar x-symbol-nedpg-image-searchpath '("./"))
(defvar x-symbol-nedpg-image-cached-dirs '("images/" "pictures/"))
(defvar x-symbol-nedpg-image-file-truename-alist nil)
(defvar x-symbol-nedpg-image-keywords nil)

;;;===========================================================================
;;;  Charsym Info
;;;===========================================================================

(defcustom x-symbol-nedpg-class-alist
  '((VALID "Nedpg Symbol" (x-symbol-info-face))
    (INVALID "no Nedpg Symbol" (red x-symbol-info-face)))
  "Alist for Nedpg's token classes displayed by info in echo area.
See `x-symbol-language-access-alist' for details."
  :group 'x-symbol-texi
  :group 'x-symbol-info-strings
  :set 'x-symbol-set-cache-variable
  :type 'x-symbol-class-info)


(defcustom x-symbol-nedpg-class-face-alist nil
  "Alist for Nedpg's color scheme in TeXinfo's grid and info.
See `x-symbol-language-access-alist' for details."
  :group 'x-symbol-nedpg
  :group 'x-symbol-input-init
  :group 'x-symbol-info-general
  :set 'x-symbol-set-cache-variable
  :type 'x-symbol-class-faces)


(defvar x-symbol-nedpg-font-lock-keywords nil)

(defvar x-symbol-nedpg-font-lock-allowed-faces t)

;;;===========================================================================
;;;  The tables
;;;===========================================================================

(defvar x-symbol-nedpg-case-insensitive nil)
(defvar x-symbol-nedpg-token-shape nil)
(defvar x-symbol-nedpg-input-token-ignore nil)

;; FIXME: next one not needed in X-Symbol 4, kept for back compat.
;;(defvar x-symbol-nedpg-exec-specs 
;;  '(nil ("\\`\\\\<[A-Za-z][A-Za-z0-9_']*>\\'" .
;;	 "\\\\<[A-Za-z][A-Za-z0-9_']*>")))

(defvar x-symbol-nedpg-token-list 'identity)

(defvar x-symbol-nedpg-xsymb0-table      ; symbols 
  '((greaterequal ">=")
    (lessequal "<=")
    (notequal "!=")
;;    (dotequal "=")
    (element "in")
    (notelement "notin")
    (propersubset "<<")
    (intersection "inter")
    (union "union")
    (backslash3 "minus")
    (universal1 "\\\\/")
    (existential1 "/\\\\")
    (logicalor "+")
    (logicaland "&")
    (arrowboth "<=>")
    (arrowright "->")
    (arrowdblright "=>")
    (notsign "~")
    (lambda "\\")
))

(defun x-symbol-nedpg-prepare-table (table)
  "Prepar the table."
    (mapcar (lambda (entry)
              (list (car entry) nil
		    (cadr entry)))
            table))

(defvar x-symbol-nedpg-table
  (x-symbol-nedpg-prepare-table
    (append x-symbol-nedpg-user-table x-symbol-nedpg-xsymb0-table)))

(provide 'x-symbol-nedpg)

;;;===========================================================================
;;;  Internal
;;;===========================================================================
;; CW: all are not needed in X-Symbol >= v4.3

(defvar x-symbol-nedpg-menu-alist nil
  "Internal.  Alist used for Isasym specific menu.")
(defvar x-symbol-nedpg-grid-alist nil
  "Internal.  Alist used for Isasym specific grid.")

(defvar x-symbol-nedpg-decode-atree nil
  "Internal.  Atree used by `x-symbol-token-input'.")
(defvar x-symbol-nedpg-decode-alist nil
  "Internal.  Alist used for decoding of Isasym macros.")
(defvar x-symbol-nedpg-encode-alist nil
  "Internal.  Alist used for encoding to Isasym macros.")

;; FIXME: next two not needed for newer X-Symbol versions.
(defvar x-symbol-nedpg-nomule-decode-exec nil
  "Internal.  File name of Isasym decode executable.")
(defvar x-symbol-nedpg-nomule-encode-exec nil
  "Internal.  File name of Isasym encode executable.")

(provide 'x-symbol-nedpg)

