(* Data.sml  -- the abstract syntax datatype *)
exception Error of string


type tc  = string;
type vc  = string;
type var = int;

datatype ty = TC of tc | TA of ty * ty 

(* Main Datastructures *)             
datatype sourceexpr = 
    AppS of sourceexpr * sourceexpr
  | VarS of string
  | LamS of (string * ty ) * sourceexpr
  | CstS of string                         
  | AndS of sourceexpr * sourceexpr
  | OrS  of sourceexpr * sourceexpr
  | ImpS of sourceexpr * sourceexpr
  | NotS of sourceexpr
  | EqS  of sourceexpr * sourceexpr
  | ExS  of (string * ty ) * sourceexpr
  | AllS of (string * ty ) * sourceexpr
 (* Es folgen weitere Typen, die zum Parsen benoetigt werden *) 



datatype pter =                    (* interne Repraesentation der Formeln *)
    A of pter * pter               (* Variablenname wird durchgeschleust  *)
  | V of var
  | GAMMA of ( ty * int )          (* Fuer Gamma Variablen mit Typ und Nr. *)
  | L of (ty * var) * pter
  | C of vc
  | AND of pter * pter
  | OR  of pter * pter
  | IMP of pter * pter
  | NOT of pter
  | EQ of pter * pter
  | EX  of (ty * var) * pter
  | ALL of (ty * var) * pter  


datatype pter' =                   (* uebergangsrepresentation *)
    A' of pter' * pter'  
  | V' of var
  | L' of (var * ty) * pter'
  | C' of vc
  | AND' of pter' * pter'
  | OR'  of pter' * pter'
  | IMP' of pter' * pter'
  | NOT' of pter'
  | EQ' of pter' * pter'
  | EX'  of (var * ty ) * pter'
  | ALL' of (var * ty ) * pter'



(* Sub Datastructures *)

datatype xdirection = LEFT | RIGHT
datatype ydirection = UP | DOWN

datatype kind_of_def = CLASSIC | LEMMA | AXIOM

datatype apply =  DEFINITION of string 
                | MODUSPONENS of int * int
             
datatype rew = LAXIOM of string * xdirection
  | PREMISSE of int * xdirection
  | EQUALITY of sourceexpr
  | BETA
  | ETA

datatype pos = LA of string | PO of int 

datatype extension = STANDART | MORE

(* Commands for ND calculus *)
datatype cmds = ELIMCMD of extension
              | REMOVECMD of xdirection * int
              | LAMCMD
              | MOVECMD of xdirection
	      (* LEFT/RIGHT  *  UP/DOWN  * ANZ. DER LINES *)
              | SHIFTCMD of xdirection * ydirection * int
              | GOALSHIFTCMD of int
              | HELPCMD 
              | PRINTCMD
              | SHOWLABELSCMD
              | LATEXCMD of string (*title*)
              | EXITCMD 
              | INTROCMD of extension
              | CUTCMD of sourceexpr                  (* Cut Regel mit Eingabe der Formel *)
              | INSTANCECMD of int * sourceexpr       (* Spezialkommando Instanziierung *)
              | POSCMD of int
              | APPLYCMD of apply
              | SIMPLIFYCMD
              | UNDOCMD of int
              | RESETCMD

(* Commands for Equalityrules *)
datatype cmdeqs =   REFEQ
                  | SYMEQ of xdirection
                  | TRANSEQ of sourceexpr
                  | INSERTEQ of xdirection * (string * sourceexpr)
                  | REWEQ of pos list * rew


(* Datastructure of Parseroutput *)
datatype parseexpr =
    ForS of sourceexpr                         (* Zur direkten Formeleingabe *)
  | GoalS of sourceexpr                        (* Zur Theoremeingabe *)
  | LetVarS of string * ty                     (* Variablendeklaration "var x : typ->typ...;" *)
  | LetConstS of string * ty                   (* Konstantendeklaration "const x : typ->typ...;" *)
  | DefS of string * sourceexpr * kind_of_def  (* Definitionendeklaration  "def name = expr"*)
  | TypS of string                             (* Typdeklarationen "type X" *)
  | CmdS of cmds                               (* Kommandoausfuehrung *)
  | CmdEqS of cmdeqs                           (* Kommandos Gleichheitsregeln *)
  | SaveS of string
  | ErrorS of string                           (* Fuer Parser/Lexer Errors *)
  | EmptyS                                     (* leere Eingabe *)
  | FileEndS                                   (* File Loading End *)