structure Definition :> Definition =
struct 
    open Data
    type definition =   (( string, (Data.pter * Data.kind_of_def) ) Polyhash.hash_table )
	              * (string -> Env.env ref)
	
	(* Use the Polyhashtable class of mosml library, to store 
	 * pairs of ( Terms * (CLASSIC|LEMMA|AXIOM) ) by a key  
	 * [ a classic definition is a term which can be subtitute in a goal, but not more ]
	 * [ a definition of lemma is a definition, which can be added to the Premessis, but it
	 *   has to be prooven after goal !! ]
	 * [ an definition of an axiom can be added to the premessis all the time ]
	 *
	 * Save in case of an lemma, which has to be prooven later on, what Environment
	 * is valid for the lemma ( with Refernce of the Environement )
	 *)

    val standart_size = 20   (* Standart Size of the hash table *)
    exception no_entry


    fun empty() = 
	let
	    val hashtable = Polyhash.mkPolyTable( standart_size, no_entry )
	    val key2env = fn s => raise Error ("No such a lemma <" ^ s ^ "> exists!\n")
	in
	    (hashtable, key2env)
	end

    
    (* Inserts a new definition into Hashtable and
     * adds a new env ref, if mode = LEMMA 
     *)
    fun insert_def def ( key, p, LEMMA) envref =
	let 
	    val (hashtable, key2env) = def
	    val _ = Polyhash.insert hashtable (key,(p, LEMMA) )
	in 
	   (hashtable, fn s => if s=key then envref else key2env s )
	end
    
      | insert_def def ( key, p, mode) _ = 
	let 
	    val (hashtable, _ ) = def
	    val _ = Polyhash.insert hashtable (key, (p,mode) )
	in 
	   def
	end
    

    

    (* returns the term of a definition *)
    fun get_term def key = 
 	let 
	    val (hashtable, _) = def
	    val (result, _ ) = 
		Polyhash.find hashtable key handle no_entry => 
		    raise Error("There is no such definition for <"^key^">")
	in
	    result
	end

    (* returns CLASSIC | LEMMA | AXIOM *)
    fun get_kind def key =
	let 
	    val (hashtable, _ ) = def
	    val ( _, kind) = 
		Polyhash.find hashtable key handle no_entry => 
		    raise Error("There is no such definition for <"^key^">")
	in
	    kind
	end

    (* changes the Kind of an definition *)
    fun change_kind def (key, kind) = 
	let
	    val (hashtable, _) = def
	    val term = get_term def key
	    val _ = Polyhash.insert hashtable (key, (term ,kind) )
	in ()
	end

	   
    (* returns the Env.env ref if saved mode of the 
     * definition is LEMMA 
     *)
    fun get_envref def key = 
	let 
	    val (hashtable, key2env) = def
	in
	   (case (get_kind def key) of
		LEMMA => (key2env key)
	      | _     => raise Error ("You can't get the environment" 
				      ^ "for a NON LEMMA Definition !\n")
	   )
	end
	   
    fun nr_def def = 
	let
	    val (hashtable,_) = def
	in Polyhash.numItems hashtable
	end


(* -------------------------------------------------------------------- *)

    (* consider only basic cases for definition translation *)
    fun check_def' (A(a,b)) mapi2s def = (check_def' a mapi2s def) orelse (check_def' b mapi2s def)
      | check_def' (V (i))  mapi2s def = ((get_term def (mapi2s i); true) handle Error _ => false )
      | check_def' (NOT(a)) mapi2s def = check_def' a mapi2s def
      | check_def' _ _ _ = false
								     
    (* don't panic about Variable (V i) and their level -> they must be at base level !! *)
    fun translate_def' (A(a,b))        mapi2s def = A((translate_def' a mapi2s def),(translate_def' b mapi2s def))
      | translate_def' (t as (V (i)))  mapi2s def = ((get_term def (mapi2s i)) handle Error _ => t )
      | translate_def' (NOT(a))        mapi2s def = NOT(translate_def' a mapi2s def)
      | translate_def'  t _ _  = t



    fun check_def def mapi2s pter = check_def' pter mapi2s def

    fun translate_def def mapi2s pter = translate_def' pter mapi2s def

end



