(* Env stellt die Datenstruktur fuer die Umgebungen der 
 * - Konstanten(Namen und Typen)
 * - freien Variablen(Namen und Typen)
 * - gebundenen Variablen
 * - bekannten Typen
 * dar 
 *)

signature Env =
sig
    type env


(* ------------ Bearbeitung der Typ/Varlisten ------------------- *)
    (* Gibt eine initiale TypUmgebung zurueck *)
    val empty : unit -> env  

    (* Fuehre einen neuen Typ ein *)
    val new_type : env * string -> env   

    (* Konstanten bekommen einen Typ *)
    val update_ctypeenv : env * string * Data.ty -> env 
    
    (* Freie Variablen bekommen einen Typ *)
    val update_ftypeenv : env * string * Data.ty -> env 
    
    (* Die Liste der Konstanten wird geupdated *)
    val update_constmaplist : env * string list -> env

    (* Die Liste der freien Variablen wird geupdated *)
    val update_freemaplist : env * string list -> env
	
    (* Fuege der Umgebung eine Map von gebundenen Variablen hinzu  *)
    val update_bndmaplist :  env * string list -> env  

    (* Benennt kritische gebundene Variablen eines Ausdrucks um *)
    val renameBndVars : env * int list -> env
 
    (* Gibt den naechsten moeglichen Namen einer neuen Freien Variable zurueck *)
    val freeVarIntro : env * int -> string
(* ------------------- Instanziierungsmethoden -------- ----- *)

    (* Stellt sicher, dass Freelist/Konstlist und Bndlist disjunkt sind *)
(*    val assert_free_bnd : env -> env *)

    (* Generiert aus zwei Environments eine neue s. Impl. *)
    (*  val gen_instance_env : env * env -> env *)

    (* Generiert aus zwei Environments eine neue s. Impl.*)
    val back_instance_env : env * env -> env

   
(* ------------------ Mapping Generatoren -------------------- *)
    (* Gibt eine Mappingfunktion (string -> int) zurueck *)
    val freemaps2i : env -> ( string -> int)
    
    (* Gibt eine Mappingfunktion (int -> string) zurueck *)
    val freemapi2s : env -> ( int -> string )

    (* Gibt eine Mappingfunktion (string -> int) zurueck *)
    val constmaps2i : env -> ( string -> int)
    
    (* Gibt eine Mappingfunktion (int -> string) zurueck *)
    val constmapi2s : env -> ( int -> string )
 
    (* Gibt eine Mappingfunktion (string -> int) zurueck *)
    val bndmaps2i : env -> ( string -> int)
    
    (* Gibt eine Mappingfunktion (int -> string) zurueck *)
    val bndmapi2s : env -> ( int -> string )

    (* Gibt eine Mappingfunktion (string -> bool) zurueck *)
    val typmaps2b : env -> (string -> bool )
  
    (* Gibt den Typ einer codierten Variable zurueck *)
    val typi : env * int -> Data.ty  

    (* Gibt den Typ einer Konstanten zurueck *)
    val typc : env * string -> Data.ty

    (* Gibt den Typ einer Variable zurueck *)
    val typf : env * string -> Data.ty  


(* ------------------- Verschiedene Methoden ------------------*)
    (* Gibt die Anzahl der bekannten Konstanten zurueck *)
    val nr_consts : env -> int

    (* Gibt die Anzahl der bekannten freien Variablen zurueck *)
    val nr_freevars : env -> int
  
    (* Gibt die Anzahl der bekannten gebundenen Variablen zurueck *)
    val nr_bndvars : env -> int

end