local open Obj Lexing in


 open Lexing Parser;

 exception LexicalError of string * int * int (* (message, loc1, loc2) *)

 fun lexerError lexbuf s = 
     raise LexicalError (s, getLexemeStart lexbuf, getLexemeEnd lexbuf);

 val commentStart = ref 0;  (* Start of outermost comment being scanned *)
 
 fun commentNotClosed lexbuf =
     raise LexicalError ("Comment not terminated", 
                         !commentStart, getLexemeEnd lexbuf);
     
 val commentDepth = ref 0;  (* Current comment nesting *)

 (* Scan keywords as identifiers and use this function to distinguish them. *)
 (* If the set of keywords is large, use an auxiliary hashtable.            *)

fun keyword s = (* Keywords *)
     case s of
         "with"             => TWITH
 (* Hilfstoken *)
       | "type"             => TTYPE
       | "const"            => TCON 
       | "var"              => TVAR
       | "def"              => TDEF
       | "lemma"            => TLEMMA
       | "axiom"            => TAXIOM
       | "apply"            => TAPPLY
 (* Gleichheitsregeln *)
       | "rew"              => TREW
       | "pos"              => TPOS
       | "subst"            => TINSERT
       | "ref"              => TREF
       | "sym"              => TSYM
       | "trans"            => TTRANS
       | "beta"             => TBETA
       | "eta"              => TETA
       | "lam"              => TLAM
 (* ** Kommandos ** *)
       | "goal"             => TGOAL
       | "elim"             => TELIM
       | "remove"           => TREMOVE
       | "instance"         => TINSTANCE
       | "move"             => TMOVE
       | "shift"            => TSHIFT
       | "intro"            => TINTRO
       | "cut"              => TCUT
       | "simplify"         => TSIMPLIFY
       | "save"             => TSAVE
       | "help"             => THELP
       | "print"            => TPRINT
       | "labels"       => TSHOWLABELS
       | "latex"            => TLATEX
       | "undo"             => TUNDO
       | "reset"            => TRESET
       | "exit"             => TEXIT
       | "FALSE"            => TCONST "FALSE"
       | "TRUE"             => TCONST "TRUE"
       | "false"            => TCONST "FALSE"
       | "true"             => TCONST "TRUE"
       | _                  => TNAME s;
 
 
fun action_26 lexbuf = (
 lexerError lexbuf "Illegal symbol in input" )
and action_25 lexbuf = (
 TEOF )
and action_24 lexbuf = (
 TEOL )
and action_23 lexbuf = (
 TDOT )
and action_22 lexbuf = (
 TCOMMA )
and action_21 lexbuf = (
 TDOUBLEPT )
and action_20 lexbuf = (
 TSEMI )
and action_19 lexbuf = (
 TRPAR )
and action_18 lexbuf = (
 TLPAR )
and action_17 lexbuf = (
 TEXISTS )
and action_16 lexbuf = (
 TALL )
and action_15 lexbuf = (
 TLAMBDA )
and action_14 lexbuf = (
 TEQ )
and action_13 lexbuf = (
 TAEQUI )
and action_12 lexbuf = (
 TIMP )
and action_11 lexbuf = (
 TNOT )
and action_10 lexbuf = (
 TAND )
and action_9 lexbuf = (
 TOR )
and action_8 lexbuf = (
 TDASHARROW )
and action_7 lexbuf = (
 commentStart := getLexemeStart lexbuf;
                          commentDepth := 1; 
                          SkipComment lexbuf; Token lexbuf )
and action_6 lexbuf = (
 keyword (getLexeme lexbuf) )
and action_5 lexbuf = (
 case Int.fromString (getLexeme lexbuf) of
                               NONE   => lexerError lexbuf "internal error"
               		     | SOME i => TNR i
                        )
and action_4 lexbuf = (
 Token lexbuf )
and action_3 lexbuf = (
 SkipComment lexbuf )
and action_2 lexbuf = (
 commentNotClosed lexbuf )
and action_1 lexbuf = (
 commentDepth := !commentDepth + 1; 
                          SkipComment lexbuf )
and action_0 lexbuf = (
 commentDepth := !commentDepth - 1;  
                          if !commentDepth = 0 then ()
                          else SkipComment lexbuf 
                        )
and state_0 lexbuf = (
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"*" => state_35 lexbuf
 |  #"(" => state_34 lexbuf
 |  #"\^Z" => action_2 lexbuf
 |  #"\^@" => action_2 lexbuf
 |  _ => action_3 lexbuf
 end)
and state_1 lexbuf = (
 let val currChar = getNextChar lexbuf in
 if currChar >= #"A" andalso currChar <= #"Z" then  state_19 lexbuf
 else if currChar >= #"a" andalso currChar <= #"z" then  state_19 lexbuf
 else if currChar >= #"0" andalso currChar <= #"9" then  state_14 lexbuf
 else case currChar of
    #"\t" => action_4 lexbuf
 |  #"\r" => action_4 lexbuf
 |  #" " => action_4 lexbuf
 |  #"~" => state_21 lexbuf
 |  #"\\" => state_20 lexbuf
 |  #"=" => state_18 lexbuf
 |  #"<" => state_17 lexbuf
 |  #";" => action_20 lexbuf
 |  #":" => action_21 lexbuf
 |  #"/" => state_13 lexbuf
 |  #"." => action_23 lexbuf
 |  #"-" => state_11 lexbuf
 |  #"," => action_22 lexbuf
 |  #"+" => action_9 lexbuf
 |  #")" => action_19 lexbuf
 |  #"(" => state_7 lexbuf
 |  #"&" => action_10 lexbuf
 |  #"\n" => action_24 lexbuf
 |  #"\^@" => action_25 lexbuf
 |  _ => action_26 lexbuf
 end)
and state_7 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_18);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"*" => action_7 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_11 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_26);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #">" => action_8 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_13 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_26);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"\\" => action_17 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_14 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_5);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_22 lexbuf
 else backtrack lexbuf
 end)
and state_17 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_26);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"=" => state_26 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_18 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_14);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #">" => action_12 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_19 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_6);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_24 lexbuf
 else if currChar >= #"A" andalso currChar <= #"Z" then  state_24 lexbuf
 else if currChar >= #"a" andalso currChar <= #"z" then  state_24 lexbuf
 else case currChar of
    #"'" => state_24 lexbuf
 |  #"_" => state_24 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_20 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_15);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"/" => action_16 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_21 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_11);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_22 lexbuf
 else backtrack lexbuf
 end)
and state_22 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_5);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_22 lexbuf
 else backtrack lexbuf
 end)
and state_24 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_6);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_24 lexbuf
 else if currChar >= #"A" andalso currChar <= #"Z" then  state_24 lexbuf
 else if currChar >= #"a" andalso currChar <= #"z" then  state_24 lexbuf
 else case currChar of
    #"'" => state_24 lexbuf
 |  #"_" => state_24 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_26 lexbuf = (
 let val currChar = getNextChar lexbuf in
 case currChar of
    #">" => action_13 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_34 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_3);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"*" => action_1 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_35 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_3);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #")" => action_0 lexbuf
 |  _ => backtrack lexbuf
 end)
and Token lexbuf =
  (setLexLastAction lexbuf (magic dummyAction);
   setLexStartPos lexbuf (getLexCurrPos lexbuf);
   state_1 lexbuf)

and SkipComment lexbuf =
  (setLexLastAction lexbuf (magic dummyAction);
   setLexStartPos lexbuf (getLexCurrPos lexbuf);
   state_0 lexbuf)

(* The following checks type consistency of actions *)
val _ = fn _ => [action_26, action_25, action_24, action_23, action_22, action_21, action_20, action_19, action_18, action_17, action_16, action_15, action_14, action_13, action_12, action_11, action_10, action_9, action_8, action_7, action_6, action_5, action_4];
val _ = fn _ => [action_3, action_2, action_1, action_0];

end
