local
in
datatype token =
    TAEQUI
  | TALL
  | TAND
  | TAPPLY
  | TAXIOM
  | TBETA
  | TCOMMA
  | TCON
  | TCONST of string
  | TCUT
  | TDASHARROW
  | TDEF
  | TDOT
  | TDOUBLEPT
  | TELIM
  | TEOF
  | TEOL
  | TEQ
  | TETA
  | TEXISTS
  | TEXIT
  | TGOAL
  | THELP
  | TIMP
  | TINSERT
  | TINSTANCE
  | TINTRO
  | TJUNK
  | TLAM
  | TLAMBDA
  | TLATEX
  | TLEMMA
  | TLPAR
  | TMOVE
  | TNAME of string
  | TNOT
  | TNR of int
  | TOR
  | TPOS
  | TPRINT
  | TREF
  | TREMOVE
  | TRESET
  | TREW
  | TRPAR
  | TSAVE
  | TSEMI
  | TSHIFT
  | TSHOWLABELS
  | TSIMPLIFY
  | TSYM
  | TTRANS
  | TTYPE
  | TUNDO
  | TVAR
  | TWITH
end;

open Obj Parsing;
prim_val vector_ : int -> 'a -> 'a Vector.vector = 2 "make_vect";
prim_val update_ : 'a Vector.vector -> int -> 'a -> unit = 3 "set_vect_item";

open Data;

fun elim' ("more") = CmdS(ELIMCMD(MORE))
  | elim' s = raise Error "Parser Error: <elim [more]>\n"

fun intro' ("more") = CmdS(INTROCMD(MORE))
  | intro' s = raise Error "Parser Error: <intro [more]>\n"

fun move' ("r") = CmdS(MOVECMD (RIGHT))
  | move' ("l") = CmdS(MOVECMD (LEFT ))
  | move' s     = raise Error "Parser Error: <move r/l > to move from Right/Left to Left/Right \n" 

val shiftmsg =  ( "Parser Error: <shift (l/r) (u/d) [nr] > \n "
				   ^ "             <shift goal [nr]>\n" 
				   ^ "to cyclic shift premises/conclusions or \n"
				   ^ "goals by [nr] positions up/down \n" )

fun shift' ("l") (("u"), n) = CmdS(SHIFTCMD(LEFT,UP,n))
 |  shift' ("l") (("d"), n) = CmdS(SHIFTCMD(LEFT,DOWN,n))
 |  shift' ("r") (("u"), n) = CmdS(SHIFTCMD(RIGHT,UP,n))
 |  shift' ("r") (("d"), n) = CmdS(SHIFTCMD(RIGHT,DOWN,n))
 |  shift' s     n = raise Error shiftmsg

fun goalshift' ("goals") n  = CmdS(GOALSHIFTCMD(n))
 |  goalshift' s n = raise Error shiftmsg

fun remove' ("l" , n ) = CmdS(REMOVECMD(LEFT,n))
 |  remove' ("r" , n ) = CmdS(REMOVECMD(RIGHT,n))
 |  remove' s          = raise Error "Parser Error: <remove [l/r] (nr)> to remove nr on left/right hand side\n"

(*
fun apply' (nr, "l") = CmdS(APPLYCMD(REW(nr, LEFT)))
 |  apply' (nr, "r") = CmdS(APPLYCMD(REW(nr, RIGHT)))
 |  apply' _ = raise Error "Parser Error: <apply (NR|NAME) [NR|(LEFT|RIGHT)] > to apply some premisse \n"
*)

(* Rewriting *
 * Wenn rew [..] with <name> (l/r)  ==> wende das externe Lemma/Axiom <name>  *
 *                                        zum Rewriting an.                  *
 * Wenn rew [..] with <nr>   (l/r)  ==> wende die Praemisse <nr> zur Rew an 
 *
 * Wenn rew [..] with (A=B)         ==> generiere Lemma (A=B) und versuche zu rewriten ! 
 *)

fun rew_axiom' xs (name, "l") = CmdEqS(REWEQ(xs,(LAXIOM(name,LEFT))))
 |  rew_axiom' xs (name, "r") = CmdEqS(REWEQ(xs,(LAXIOM(name,RIGHT))))
 |  rew_axiom' _  _ = raise Error "Parser Error : <rew (1,2,1,...) with name (l/r)> to apply rewriting"	 

fun rew_prem' xs (nr, "l") = CmdEqS(REWEQ(xs,(PREMISSE(nr,LEFT))))
 |  rew_prem' xs (nr, "r") = CmdEqS(REWEQ(xs,(PREMISSE(nr,RIGHT))))
 |  rew_prem' _ _ = raise Error "Parser Error : <rew (1,2,1,...) with name (l/r)> to apply rewriting"	 

fun symmetric' ("l") = CmdEqS(SYMEQ(LEFT))
 |  symmetric' ("r") = CmdEqS(SYMEQ(RIGHT))
 |  symmetric' _ = raise Error "Parser Error : <sym [l/r]> to apply symmetric rule"

fun insert' ("l") (n,s) = CmdEqS(INSERTEQ(LEFT,(n,s)))
 |  insert' ("r") (n,s) = CmdEqS(INSERTEQ(RIGHT,(n,s)))
 |  insert' _ _ = raise Error "Parser Error: <insert name = formel [l/r]> to insert a formula for variable name"


(* checkt die Positionsliste *)
fun checkpos nil = nil
 |  checkpos (x::xs) = x::(checkpos' xs)
and checkpos' nil = nil
 |  checkpos' ((LA _)::xs) = raise Error "Please use Positions [Label](1|2)*"
 |  checkpos' (x::xs) = x::(checkpos' xs)
   

fun undo' n = CmdS(UNDOCMD n)


(* Line 78, file Parser.sml *)
val yytransl = #[
  257 (* TAEQUI *),
  258 (* TALL *),
  259 (* TAND *),
  260 (* TAPPLY *),
  261 (* TAXIOM *),
  262 (* TBETA *),
  263 (* TCOMMA *),
  264 (* TCON *),
  265 (* TCONST *),
  266 (* TCUT *),
  267 (* TDASHARROW *),
  268 (* TDEF *),
  269 (* TDOT *),
  270 (* TDOUBLEPT *),
  271 (* TELIM *),
  272 (* TEOF *),
  273 (* TEOL *),
  274 (* TEQ *),
  275 (* TETA *),
  276 (* TEXISTS *),
  277 (* TEXIT *),
  278 (* TGOAL *),
  279 (* THELP *),
  280 (* TIMP *),
  281 (* TINSERT *),
  282 (* TINSTANCE *),
  283 (* TINTRO *),
  284 (* TJUNK *),
  285 (* TLAM *),
  286 (* TLAMBDA *),
  287 (* TLATEX *),
  288 (* TLEMMA *),
  289 (* TLPAR *),
  290 (* TMOVE *),
  291 (* TNAME *),
  292 (* TNOT *),
  293 (* TNR *),
  294 (* TOR *),
  295 (* TPOS *),
  296 (* TPRINT *),
  297 (* TREF *),
  298 (* TREMOVE *),
  299 (* TRESET *),
  300 (* TREW *),
  301 (* TRPAR *),
  302 (* TSAVE *),
  303 (* TSEMI *),
  304 (* TSHIFT *),
  305 (* TSHOWLABELS *),
  306 (* TSIMPLIFY *),
  307 (* TSYM *),
  308 (* TTRANS *),
  309 (* TTYPE *),
  310 (* TUNDO *),
  311 (* TVAR *),
  312 (* TWITH *),
    0];

val yylhs = "\255\255\
\\001\000\001\000\001\000\001\000\001\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\003\000\003\000\003\000\003\000\003\000\003\000\003\000\004\000\
\\004\000\005\000\005\000\006\000\006\000\007\000\007\000\008\000\
\\008\000\009\000\009\000\010\000\010\000\011\000\011\000\011\000\
\\012\000\012\000\013\000\013\000\014\000\014\000\014\000\014\000\
\\014\000\000\000";

val yylen = "\002\000\
\\003\000\002\000\002\000\001\000\001\000\001\000\002\000\004\000\
\\004\000\002\000\005\000\005\000\005\000\004\000\003\000\003\000\
\\002\000\002\000\002\000\001\000\002\000\004\000\001\000\001\000\
\\002\000\002\000\004\000\004\000\004\000\005\000\003\000\003\000\
\\006\000\006\000\007\000\005\000\005\000\002\000\001\000\002\000\
\\003\000\001\000\002\000\002\000\003\000\001\000\002\000\001\000\
\\001\000\001\000\001\000\001\000\002\000\001\000\002\000\001\000\
\\001\000\004\000\006\000\004\000\004\000\006\000\006\000\001\000\
\\003\000\001\000\003\000\001\000\003\000\001\000\003\000\001\000\
\\003\000\001\000\002\000\001\000\002\000\001\000\001\000\003\000\
\\001\000\003\000\001\000\003\000\003\000\003\000\002\000\002\000\
\\001\000\002\000";

val yydefred = "\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\079\000\000\000\
\\000\000\000\000\005\000\004\000\000\000\056\000\000\000\050\000\
\\000\000\000\000\000\000\048\000\000\000\000\000\000\000\000\000\
\\000\000\078\000\000\000\000\000\051\000\023\000\000\000\054\000\
\\000\000\000\000\000\000\052\000\049\000\000\000\000\000\000\000\
\\000\000\000\000\090\000\000\000\006\000\057\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\076\000\000\000\040\000\000\000\
\\000\000\000\000\021\000\000\000\047\000\000\000\007\000\000\000\
\\000\000\043\000\000\000\053\000\000\000\000\000\018\000\075\000\
\\038\000\000\000\044\000\000\000\000\000\055\000\000\000\025\000\
\\026\000\010\000\019\000\000\000\003\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\077\000\000\000\000\000\041\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\080\000\045\000\000\000\000\000\089\000\000\000\031\000\
\\032\000\000\000\000\000\000\000\000\000\016\000\000\000\001\000\
\\065\000\067\000\069\000\071\000\073\000\060\000\000\000\083\000\
\\000\000\000\000\000\000\009\000\000\000\061\000\000\000\027\000\
\\022\000\058\000\000\000\000\000\000\000\088\000\000\000\087\000\
\\000\000\000\000\028\000\029\000\014\000\008\000\000\000\000\000\
\\000\000\013\000\011\000\000\000\000\000\012\000\086\000\085\000\
\\036\000\037\000\000\000\000\000\000\000\030\000\084\000\062\000\
\\082\000\063\000\059\000\000\000\033\000\034\000\035\000";

val yydgoto = "\002\000\
\\043\000\044\000\045\000\046\000\047\000\048\000\049\000\050\000\
\\051\000\052\000\053\000\129\000\130\000\111\000";

val yysindex = "\005\000\
\\013\255\000\000\222\254\229\254\235\254\247\254\000\000\131\255\
\\002\255\025\255\000\000\000\000\036\255\000\000\131\255\000\000\
\\042\255\043\255\063\255\000\000\068\255\092\255\096\255\131\255\
\\102\255\000\000\144\255\106\255\000\000\000\000\232\254\000\000\
\\227\254\112\255\127\255\000\000\000\000\139\255\131\255\141\255\
\\148\255\143\255\000\000\034\255\000\000\000\000\169\255\185\255\
\\164\255\151\255\187\255\130\255\000\000\006\255\000\000\154\255\
\\178\255\179\255\000\000\180\255\000\000\168\255\000\000\177\255\
\\181\255\000\000\170\255\000\000\182\255\152\255\000\000\000\000\
\\000\000\163\255\000\000\089\255\123\255\000\000\041\255\000\000\
\\000\000\000\000\000\000\184\255\000\000\186\255\144\255\144\255\
\\144\255\144\255\144\255\000\000\131\255\124\255\000\000\183\255\
\\124\255\188\255\131\255\124\255\131\255\131\255\131\255\124\255\
\\189\255\000\000\000\000\252\254\005\255\000\000\146\255\000\000\
\\000\000\131\255\173\255\174\255\167\255\000\000\124\255\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\124\255\000\000\
\\192\255\199\255\131\255\000\000\131\255\000\000\198\255\000\000\
\\000\000\000\000\200\255\131\255\089\255\000\000\089\255\000\000\
\\135\255\171\255\000\000\000\000\000\000\000\000\172\255\131\255\
\\124\255\000\000\000\000\131\255\131\255\000\000\000\000\000\000\
\\000\000\000\000\131\255\190\255\191\255\000\000\000\000\000\000\
\\000\000\000\000\000\000\175\255\000\000\000\000\000\000";

val yyrindex = "\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\055\255\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\057\255\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\065\255\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\071\255\000\000\000\000\
\\088\255\000\000\000\000\000\000\000\000\000\000\052\255\128\255\
\\069\255\091\255\083\255\072\255\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\103\255\000\000\
\\000\000\000\000\000\000\000\000\000\000\212\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\105\255\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\066\255\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000";

val yygindex = "\000\000\
\\000\000\000\000\248\255\127\000\130\000\126\000\129\000\131\000\
\\194\000\000\000\171\000\161\255\000\000\028\000";

val YYTABLESIZE = 226;
val yytable = "\059\000\
\\054\000\132\000\141\000\076\000\135\000\001\000\063\000\055\000\
\\139\000\056\000\074\000\143\000\075\000\057\000\003\000\070\000\
\\004\000\005\000\093\000\094\000\006\000\007\000\008\000\150\000\
\\009\000\058\000\077\000\010\000\011\000\012\000\081\000\151\000\
\\013\000\014\000\015\000\016\000\060\000\017\000\018\000\019\000\
\\142\000\020\000\021\000\022\000\023\000\024\000\025\000\026\000\
\\027\000\144\000\085\000\028\000\029\000\030\000\031\000\032\000\
\\033\000\169\000\034\000\061\000\035\000\036\000\037\000\038\000\
\\039\000\040\000\041\000\042\000\064\000\068\000\062\000\046\000\
\\074\000\042\000\074\000\117\000\064\000\118\000\081\000\065\000\
\\086\000\039\000\081\000\072\000\126\000\068\000\068\000\024\000\
\\074\000\074\000\134\000\070\000\136\000\137\000\138\000\074\000\
\\064\000\066\000\064\000\072\000\072\000\046\000\067\000\042\000\
\\020\000\146\000\072\000\070\000\070\000\074\000\081\000\039\000\
\\081\000\068\000\070\000\068\000\074\000\024\000\074\000\017\000\
\\072\000\015\000\154\000\108\000\155\000\109\000\068\000\072\000\
\\112\000\072\000\069\000\158\000\003\000\110\000\020\000\070\000\
\\071\000\070\000\007\000\007\000\161\000\113\000\073\000\168\000\
\\066\000\066\000\078\000\170\000\171\000\017\000\013\000\015\000\
\\007\000\162\000\172\000\114\000\127\000\115\000\128\000\116\000\
\\021\000\079\000\024\000\024\000\026\000\026\000\027\000\163\000\
\\159\000\164\000\160\000\165\000\066\000\080\000\066\000\082\000\
\\024\000\084\000\026\000\027\000\099\000\100\000\103\000\104\000\
\\083\000\088\000\087\000\089\000\090\000\091\000\095\000\096\000\
\\097\000\098\000\101\000\105\000\106\000\119\000\102\000\107\000\
\\131\000\145\000\120\000\149\000\152\000\133\000\140\000\147\000\
\\148\000\153\000\156\000\002\000\157\000\121\000\123\000\166\000\
\\167\000\122\000\124\000\175\000\072\000\125\000\092\000\000\000\
\\173\000\174\000";

val yycheck = "\008\000\
\\035\001\097\000\007\001\033\001\100\000\001\000\015\000\035\001\
\\104\000\037\001\035\001\007\001\037\001\035\001\002\001\024\000\
\\004\001\005\001\013\001\014\001\008\001\009\001\010\001\119\000\
\\012\001\035\001\056\001\015\001\016\001\017\001\039\000\127\000\
\\020\001\021\001\022\001\023\001\035\001\025\001\026\001\027\001\
\\045\001\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\\036\001\045\001\017\001\039\001\040\001\041\001\042\001\043\001\
\\044\001\153\000\046\001\035\001\048\001\049\001\050\001\051\001\
\\052\001\053\001\054\001\055\001\017\001\001\001\035\001\017\001\
\\001\001\017\001\003\001\035\001\035\001\037\001\013\001\037\001\
\\047\001\017\001\017\001\001\001\093\000\017\001\018\001\017\001\
\\017\001\018\001\099\000\001\001\101\000\102\000\103\000\024\001\
\\045\001\035\001\047\001\017\001\018\001\047\001\035\001\047\001\
\\017\001\114\000\024\001\017\001\018\001\038\001\045\001\047\001\
\\047\001\045\001\024\001\047\001\045\001\047\001\047\001\017\001\
\\038\001\017\001\131\000\035\001\133\000\037\001\035\001\045\001\
\\006\001\047\001\035\001\140\000\002\001\045\001\047\001\045\001\
\\035\001\047\001\009\001\009\001\006\001\019\001\037\001\152\000\
\\017\001\018\001\035\001\156\000\157\000\047\001\020\001\047\001\
\\009\001\019\001\163\000\033\001\033\001\035\001\035\001\037\001\
\\030\001\035\001\033\001\033\001\035\001\035\001\036\001\033\001\
\\141\000\035\001\143\000\037\001\045\001\035\001\047\001\035\001\
\\033\001\035\001\035\001\036\001\013\001\014\001\013\001\014\001\
\\037\001\001\001\018\001\024\001\038\001\003\001\037\001\014\001\
\\014\001\014\001\018\001\014\001\045\001\014\001\018\001\037\001\
\\018\001\056\001\017\001\037\001\013\001\018\001\018\001\035\001\
\\035\001\011\001\013\001\000\000\013\001\087\000\089\000\045\001\
\\045\001\088\000\090\000\045\001\027\000\091\000\052\000\255\255\
\\035\001\035\001";

val yyact = vector_ 91 (fn () => ((raise Fail "parser") : obj));
(* Rule 1, file Parser.grm, line 107 *)
val _ = update_ yyact 1
(fn () => repr(let
val d__1__ = peekVal 2 : Data.parseexpr
in
( (d__1__) ) end : Data.parseexpr))
;
(* Rule 2, file Parser.grm, line 109 *)
val _ = update_ yyact 2
(fn () => repr(let
val d__1__ = peekVal 1 : Data.parseexpr
in
( (d__1__) ) end : Data.parseexpr))
;
(* Rule 3, file Parser.grm, line 110 *)
val _ = update_ yyact 3
(fn () => repr(let
val d__1__ = peekVal 1 : Data.parseexpr
in
( (d__1__) ) end : Data.parseexpr))
;
(* Rule 4, file Parser.grm, line 111 *)
val _ = update_ yyact 4
(fn () => repr(let
in
( EmptyS ) end : Data.parseexpr))
;
(* Rule 5, file Parser.grm, line 112 *)
val _ = update_ yyact 5
(fn () => repr(let
in
( FileEndS ) end : Data.parseexpr))
;
(* Rule 6, file Parser.grm, line 116 *)
val _ = update_ yyact 6
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( ForS((d__1__)) ) end : Data.parseexpr))
;
(* Rule 7, file Parser.grm, line 117 *)
val _ = update_ yyact 7
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( GoalS((d__2__)) ) end : Data.parseexpr))
;
(* Rule 8, file Parser.grm, line 118 *)
val _ = update_ yyact 8
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.ty
in
( LetVarS((d__2__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 9, file Parser.grm, line 119 *)
val _ = update_ yyact 9
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.ty
in
( LetConstS((d__2__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 10, file Parser.grm, line 120 *)
val _ = update_ yyact 10
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( TypS((d__2__)) ) end : Data.parseexpr))
;
(* Rule 11, file Parser.grm, line 121 *)
val _ = update_ yyact 11
(fn () => repr(let
val d__2__ = peekVal 3 : string
val d__5__ = peekVal 0 : Data.sourceexpr
in
( DefS((d__2__), (d__5__), CLASSIC) ) end : Data.parseexpr))
;
(* Rule 12, file Parser.grm, line 122 *)
val _ = update_ yyact 12
(fn () => repr(let
val d__2__ = peekVal 3 : string
val d__5__ = peekVal 0 : Data.sourceexpr
in
( DefS((d__2__), (d__5__), LEMMA) ) end : Data.parseexpr))
;
(* Rule 13, file Parser.grm, line 123 *)
val _ = update_ yyact 13
(fn () => repr(let
val d__2__ = peekVal 3 : string
val d__5__ = peekVal 0 : Data.sourceexpr
in
( DefS((d__2__), (d__5__), AXIOM) ) end : Data.parseexpr))
;
(* Rule 14, file Parser.grm, line 125 *)
val _ = update_ yyact 14
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__3__ = peekVal 1 : string
val d__4__ = peekVal 0 : int
in
( shift' (d__2__) ((d__3__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 15, file Parser.grm, line 126 *)
val _ = update_ yyact 15
(fn () => repr(let
val d__2__ = peekVal 1 : string
val d__3__ = peekVal 0 : string
in
( shift' (d__2__) ((d__3__), 1) ) end : Data.parseexpr))
;
(* Rule 16, file Parser.grm, line 127 *)
val _ = update_ yyact 16
(fn () => repr(let
val d__2__ = peekVal 1 : string
val d__3__ = peekVal 0 : int
in
( goalshift' (d__2__) (d__3__)  ) end : Data.parseexpr))
;
(* Rule 17, file Parser.grm, line 128 *)
val _ = update_ yyact 17
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( goalshift' (d__2__) 1   ) end : Data.parseexpr))
;
(* Rule 18, file Parser.grm, line 129 *)
val _ = update_ yyact 18
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( move' (d__2__) ) end : Data.parseexpr))
;
(* Rule 19, file Parser.grm, line 130 *)
val _ = update_ yyact 19
(fn () => repr(let
val d__2__ = peekVal 0 : int
in
( undo' (d__2__) ) end : Data.parseexpr))
;
(* Rule 20, file Parser.grm, line 131 *)
val _ = update_ yyact 20
(fn () => repr(let
in
( undo' 1  ) end : Data.parseexpr))
;
(* Rule 21, file Parser.grm, line 133 *)
val _ = update_ yyact 21
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( CmdS(CUTCMD((d__2__))) ) end : Data.parseexpr))
;
(* Rule 22, file Parser.grm, line 134 *)
val _ = update_ yyact 22
(fn () => repr(let
val d__2__ = peekVal 2 : int
val d__4__ = peekVal 0 : Data.sourceexpr
in
( CmdS(INSTANCECMD((d__2__),(d__4__))) ) end : Data.parseexpr))
;
(* Rule 23, file Parser.grm, line 138 *)
val _ = update_ yyact 23
(fn () => repr(let
in
( CmdEqS(REFEQ) ) end : Data.parseexpr))
;
(* Rule 24, file Parser.grm, line 139 *)
val _ = update_ yyact 24
(fn () => repr(let
in
( CmdEqS(SYMEQ(RIGHT)) ) end : Data.parseexpr))
;
(* Rule 25, file Parser.grm, line 140 *)
val _ = update_ yyact 25
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( symmetric'((d__2__)) ) end : Data.parseexpr))
;
(* Rule 26, file Parser.grm, line 141 *)
val _ = update_ yyact 26
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( CmdEqS(TRANSEQ((d__2__))) ) end : Data.parseexpr))
;
(* Rule 27, file Parser.grm, line 143 *)
val _ = update_ yyact 27
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( CmdEqS(INSERTEQ(RIGHT, ((d__2__),(d__4__)))) ) end : Data.parseexpr))
;
(* Rule 28, file Parser.grm, line 144 *)
val _ = update_ yyact 28
(fn () => repr(let
val d__3__ = peekVal 1 : string
val d__4__ = peekVal 0 : string
in
( rew_axiom' nil ((d__3__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 29, file Parser.grm, line 145 *)
val _ = update_ yyact 29
(fn () => repr(let
val d__3__ = peekVal 1 : int
val d__4__ = peekVal 0 : string
in
( rew_prem'  nil ((d__3__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 30, file Parser.grm, line 146 *)
val _ = update_ yyact 30
(fn () => repr(let
val d__4__ = peekVal 1 : Data.sourceexpr
in
( CmdEqS(REWEQ(nil, EQUALITY((d__4__)))) ) end : Data.parseexpr))
;
(* Rule 31, file Parser.grm, line 147 *)
val _ = update_ yyact 31
(fn () => repr(let
in
( CmdEqS(REWEQ(nil, BETA)) ) end : Data.parseexpr))
;
(* Rule 32, file Parser.grm, line 148 *)
val _ = update_ yyact 32
(fn () => repr(let
in
( CmdEqS(REWEQ(nil, ETA)) ) end : Data.parseexpr))
;
(* Rule 33, file Parser.grm, line 149 *)
val _ = update_ yyact 33
(fn () => repr(let
val d__3__ = peekVal 3 : Data.pos list
val d__5__ = peekVal 1 : string
val d__6__ = peekVal 0 : string
in
( rew_axiom' (checkpos((d__3__)))  ((d__5__),(d__6__)) ) end : Data.parseexpr))
;
(* Rule 34, file Parser.grm, line 150 *)
val _ = update_ yyact 34
(fn () => repr(let
val d__3__ = peekVal 3 : Data.pos list
val d__5__ = peekVal 1 : int
val d__6__ = peekVal 0 : string
in
( rew_prem'  (checkpos((d__3__)))  ((d__5__),(d__6__)) ) end : Data.parseexpr))
;
(* Rule 35, file Parser.grm, line 151 *)
val _ = update_ yyact 35
(fn () => repr(let
val d__3__ = peekVal 4 : Data.pos list
val d__6__ = peekVal 1 : Data.sourceexpr
in
( CmdEqS(REWEQ((checkpos((d__3__))), EQUALITY((d__6__)))) ) end : Data.parseexpr))
;
(* Rule 36, file Parser.grm, line 152 *)
val _ = update_ yyact 36
(fn () => repr(let
val d__3__ = peekVal 2 : Data.pos list
in
( CmdEqS(REWEQ((checkpos((d__3__))), BETA)) ) end : Data.parseexpr))
;
(* Rule 37, file Parser.grm, line 153 *)
val _ = update_ yyact 37
(fn () => repr(let
val d__3__ = peekVal 2 : Data.pos list
in
( CmdEqS(REWEQ((checkpos((d__3__))), ETA)) ) end : Data.parseexpr))
;
(* Rule 38, file Parser.grm, line 155 *)
val _ = update_ yyact 38
(fn () => repr(let
val d__2__ = peekVal 0 : int
in
( CmdS(POSCMD (d__2__)) ) end : Data.parseexpr))
;
(* Rule 39, file Parser.grm, line 156 *)
val _ = update_ yyact 39
(fn () => repr(let
in
( CmdS(POSCMD 0)  ) end : Data.parseexpr))
;
(* Rule 40, file Parser.grm, line 158 *)
val _ = update_ yyact 40
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( CmdS(APPLYCMD(DEFINITION (d__2__))) ) end : Data.parseexpr))
;
(* Rule 41, file Parser.grm, line 160 *)
val _ = update_ yyact 41
(fn () => repr(let
val d__2__ = peekVal 1 : int
val d__3__ = peekVal 0 : int
in
( CmdS(APPLYCMD(MODUSPONENS((d__2__),(d__3__)))) ) end : Data.parseexpr))
;
(* Rule 42, file Parser.grm, line 161 *)
val _ = update_ yyact 42
(fn () => repr(let
in
( CmdS(INTROCMD(STANDART)) ) end : Data.parseexpr))
;
(* Rule 43, file Parser.grm, line 162 *)
val _ = update_ yyact 43
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( intro' (d__2__) ) end : Data.parseexpr))
;
(* Rule 44, file Parser.grm, line 163 *)
val _ = update_ yyact 44
(fn () => repr(let
val d__2__ = peekVal 0 : int
in
( CmdS(REMOVECMD (LEFT,(d__2__)))) end : Data.parseexpr))
;
(* Rule 45, file Parser.grm, line 164 *)
val _ = update_ yyact 45
(fn () => repr(let
val d__2__ = peekVal 1 : string
val d__3__ = peekVal 0 : int
in
( remove' ((d__2__), (d__3__)) ) end : Data.parseexpr))
;
(* Rule 46, file Parser.grm, line 165 *)
val _ = update_ yyact 46
(fn () => repr(let
in
( CmdS(ELIMCMD(STANDART)) ) end : Data.parseexpr))
;
(* Rule 47, file Parser.grm, line 166 *)
val _ = update_ yyact 47
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( elim' (d__2__) ) end : Data.parseexpr))
;
(* Rule 48, file Parser.grm, line 167 *)
val _ = update_ yyact 48
(fn () => repr(let
in
( CmdS(LAMCMD) ) end : Data.parseexpr))
;
(* Rule 49, file Parser.grm, line 168 *)
val _ = update_ yyact 49
(fn () => repr(let
in
( CmdS(SIMPLIFYCMD) ) end : Data.parseexpr))
;
(* Rule 50, file Parser.grm, line 169 *)
val _ = update_ yyact 50
(fn () => repr(let
in
( CmdS(HELPCMD) ) end : Data.parseexpr))
;
(* Rule 51, file Parser.grm, line 170 *)
val _ = update_ yyact 51
(fn () => repr(let
in
( CmdS(PRINTCMD) ) end : Data.parseexpr))
;
(* Rule 52, file Parser.grm, line 171 *)
val _ = update_ yyact 52
(fn () => repr(let
in
( CmdS(SHOWLABELSCMD) ) end : Data.parseexpr))
;
(* Rule 53, file Parser.grm, line 172 *)
val _ = update_ yyact 53
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( CmdS(LATEXCMD (d__2__)) ) end : Data.parseexpr))
;
(* Rule 54, file Parser.grm, line 173 *)
val _ = update_ yyact 54
(fn () => repr(let
in
( CmdS(RESETCMD) ) end : Data.parseexpr))
;
(* Rule 55, file Parser.grm, line 174 *)
val _ = update_ yyact 55
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( SaveS (d__2__) ) end : Data.parseexpr))
;
(* Rule 56, file Parser.grm, line 175 *)
val _ = update_ yyact 56
(fn () => repr(let
in
( CmdS(EXITCMD) ) end : Data.parseexpr))
;
(* Rule 57, file Parser.grm, line 179 *)
val _ = update_ yyact 57
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 58, file Parser.grm, line 180 *)
val _ = update_ yyact 58
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( LamS(((d__2__), TC "not_specified"), (d__4__)) ) end : Data.sourceexpr))
;
(* Rule 59, file Parser.grm, line 181 *)
val _ = update_ yyact 59
(fn () => repr(let
val d__2__ = peekVal 4 : string
val d__4__ = peekVal 2 : Data.ty
val d__6__ = peekVal 0 : Data.sourceexpr
in
( LamS(((d__2__),(d__4__)),(d__6__)) ) end : Data.sourceexpr))
;
(* Rule 60, file Parser.grm, line 182 *)
val _ = update_ yyact 60
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( AllS(((d__2__), TC "not_specified"), (d__4__)) ) end : Data.sourceexpr))
;
(* Rule 61, file Parser.grm, line 183 *)
val _ = update_ yyact 61
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( ExS (((d__2__), TC "not_specified"), (d__4__)) ) end : Data.sourceexpr))
;
(* Rule 62, file Parser.grm, line 184 *)
val _ = update_ yyact 62
(fn () => repr(let
val d__2__ = peekVal 4 : string
val d__4__ = peekVal 2 : Data.ty
val d__6__ = peekVal 0 : Data.sourceexpr
in
( AllS(((d__2__),(d__4__)),(d__6__)) ) end : Data.sourceexpr))
;
(* Rule 63, file Parser.grm, line 185 *)
val _ = update_ yyact 63
(fn () => repr(let
val d__2__ = peekVal 4 : string
val d__4__ = peekVal 2 : Data.ty
val d__6__ = peekVal 0 : Data.sourceexpr
in
( ExS (((d__2__),(d__4__)),(d__6__)) ) end : Data.sourceexpr))
;
(* Rule 64, file Parser.grm, line 189 *)
val _ = update_ yyact 64
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 65, file Parser.grm, line 190 *)
val _ = update_ yyact 65
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( EqS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 66, file Parser.grm, line 193 *)
val _ = update_ yyact 66
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 67, file Parser.grm, line 194 *)
val _ = update_ yyact 67
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( AndS(ImpS((d__1__),(d__3__)),ImpS((d__3__),(d__1__))) ) end : Data.sourceexpr))
;
(* Rule 68, file Parser.grm, line 197 *)
val _ = update_ yyact 68
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 69, file Parser.grm, line 198 *)
val _ = update_ yyact 69
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( ImpS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 70, file Parser.grm, line 201 *)
val _ = update_ yyact 70
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 71, file Parser.grm, line 202 *)
val _ = update_ yyact 71
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( OrS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 72, file Parser.grm, line 205 *)
val _ = update_ yyact 72
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 73, file Parser.grm, line 206 *)
val _ = update_ yyact 73
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( AndS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 74, file Parser.grm, line 209 *)
val _ = update_ yyact 74
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 75, file Parser.grm, line 210 *)
val _ = update_ yyact 75
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( NotS((d__2__)) ) end : Data.sourceexpr))
;
(* Rule 76, file Parser.grm, line 213 *)
val _ = update_ yyact 76
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 77, file Parser.grm, line 214 *)
val _ = update_ yyact 77
(fn () => repr(let
val d__1__ = peekVal 1 : Data.sourceexpr
val d__2__ = peekVal 0 : Data.sourceexpr
in
( AppS((d__1__), (d__2__)) ) end : Data.sourceexpr))
;
(* Rule 78, file Parser.grm, line 217 *)
val _ = update_ yyact 78
(fn () => repr(let
val d__1__ = peekVal 0 : string
in
( VarS (d__1__) ) end : Data.sourceexpr))
;
(* Rule 79, file Parser.grm, line 218 *)
val _ = update_ yyact 79
(fn () => repr(let
val d__1__ = peekVal 0 : string
in
( CstS (d__1__) ) end : Data.sourceexpr))
;
(* Rule 80, file Parser.grm, line 219 *)
val _ = update_ yyact 80
(fn () => repr(let
val d__2__ = peekVal 1 : Data.sourceexpr
in
( (d__2__) ) end : Data.sourceexpr))
;
(* Rule 81, file Parser.grm, line 222 *)
val _ = update_ yyact 81
(fn () => repr(let
val d__1__ = peekVal 0 : Data.ty
in
( (d__1__) ) end : Data.ty))
;
(* Rule 82, file Parser.grm, line 223 *)
val _ = update_ yyact 82
(fn () => repr(let
val d__1__ = peekVal 2 : Data.ty
val d__3__ = peekVal 0 : Data.ty
in
( TA((d__1__),(d__3__)) ) end : Data.ty))
;
(* Rule 83, file Parser.grm, line 226 *)
val _ = update_ yyact 83
(fn () => repr(let
val d__1__ = peekVal 0 : string
in
( TC((d__1__)) ) end : Data.ty))
;
(* Rule 84, file Parser.grm, line 227 *)
val _ = update_ yyact 84
(fn () => repr(let
val d__2__ = peekVal 1 : Data.ty
in
( (d__2__) ) end : Data.ty))
;
(* Rule 85, file Parser.grm, line 231 *)
val _ = update_ yyact 85
(fn () => repr(let
val d__1__ = peekVal 2 : int
val d__3__ = peekVal 0 : Data.pos list
in
( (PO (d__1__))::(d__3__) ) end : Data.pos list))
;
(* Rule 86, file Parser.grm, line 232 *)
val _ = update_ yyact 86
(fn () => repr(let
val d__1__ = peekVal 2 : string
val d__3__ = peekVal 0 : Data.pos list
in
( (LA (d__1__))::(d__3__) ) end : Data.pos list))
;
(* Rule 87, file Parser.grm, line 233 *)
val _ = update_ yyact 87
(fn () => repr(let
val d__1__ = peekVal 1 : int
in
( (PO (d__1__))::nil ) end : Data.pos list))
;
(* Rule 88, file Parser.grm, line 234 *)
val _ = update_ yyact 88
(fn () => repr(let
val d__1__ = peekVal 1 : string
in
( (LA (d__1__))::nil ) end : Data.pos list))
;
(* Rule 89, file Parser.grm, line 235 *)
val _ = update_ yyact 89
(fn () => repr(let
in
( nil ) end : Data.pos list))
;
(* Entry Main *)
val _ = update_ yyact 90 (fn () => raise yyexit (peekVal 0));
val yytables : parseTables =
  ( yyact,
    yytransl,
    yylhs,
    yylen,
    yydefred,
    yydgoto,
    yysindex,
    yyrindex,
    yygindex,
    YYTABLESIZE,
    yytable,
    yycheck );
fun Main lexer lexbuf = yyparse yytables 1 lexer lexbuf;
