local open Data in

(* --------------------------------------------------------- *)
(* |             AUSGABE ROUTINEN FUER STDOUT              | *)
(* --------------------------------------------------------- *)


(* Label Printing mode *)
val showlabels = ref false
fun set_showlabels() = ( showlabels := ( !showlabels = false ) ;
			print ("> Label Printing : " ^ (if !showlabels then "on" else "off") ^ "\n\n") )
fun get_showlabels() = !showlabels 

(* Typausgabe - Funktionen *)
fun ty2string (TA (a,b)) = ( atty2string a ) ^ "->" ^ (ty2string b)
  | ty2string t          = atty2string t
and atty2string (TC (a)) = a
  | atty2string t        = "( " ^ ty2string t ^ " )"

(* DEBUG AUSGABEN *)
fun pter2string (A(a,b)) = "A(" ^ pter2string (a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (V(n))   = "V " ^ Int.toString(n)
  | pter2string (GAMMA(t,i)) = "GAMMA ("^Int.toString(i)^" :"^ty2string(t)^")"
  | pter2string (L((t,s),e)) = "L(" ^ ty2string t ^ ", " ^ pter2string (e) ^ ")"
  | pter2string (EX((t,s),e)) = "EX ("  ^ ty2string t ^ ", " ^ pter2string (e) ^ ")"
  | pter2string (ALL((t,s),e)) = "ALL(" ^ ty2string t ^ ", " ^ pter2string (e) ^ ")"
  | pter2string (C(s)) =  "C(" ^ s ^ ")"
  | pter2string (AND(a,b))  = "AND(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (OR(a,b))  = "OR(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (IMP(a,b))  = "IMP(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (EQ(a,b))  = "EQ(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (NOT(a))  = "NOT(" ^ pter2string(a) ^ ")"
  
fun parse2string (GoalS _) = "GoalS"
  | parse2string (ForS _)  = "ForS"
  | parse2string (LetVarS _)  = "LetVarS"
  | parse2string (LetConstS _) = "LetConstS"
  | parse2string (DefS _)  = "DefS"
  | parse2string (TypS _)  = "TypS"
  | parse2string (CmdS _)  = "CmdS"
  | parse2string (CmdEqS _)   = "EqS"
  | parse2string (SaveS _) = "Save Goal"
  | parse2string (ErrorS _)= "Parser Error"
  | parse2string (FileEndS)= "File loaded"
  | parse2string (EmptyS)  = "EmptyS"



(* Durch AND verknuepfte Teilausdruecke in der Praemisse werden
 * zeilenweise ausgegeben [1] Praemisse 1
 *                        [2] Praemisse 2  usw.                 
 *)
fun print_left (AND(a,b)) fmap bmap andlevel = 
    let
	val _= print("["^Int.toString(andlevel)^"] ")
	val _= print (PrettyPrint.getPPString a (fmap,bmap))
	val _= print("\n")
    in print_left b fmap bmap (andlevel + 1)
    end
  | print_left t fmap bmap 1 = 
    let
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end 
  | print_left t fmap bmap andlevel = 
    let
	val _= print("["^Int.toString(andlevel)^"] ")
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end

(* fun print_right t fmap bmap = print (PrettyPrint.getPPString t (fmap,bmap)) *)
fun print_right (OR(a,b)) fmap bmap orlevel = 
    let
	val _= print("["^Int.toString(orlevel)^"] ")
	val _= print (PrettyPrint.getPPString a (fmap,bmap))
	val _= print("\n")
    in print_right b fmap bmap (orlevel + 1)
    end
  | print_right t fmap bmap 1 = 
    let
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end 
  | print_right t fmap bmap orlevel = 
    let
	val _= print("["^Int.toString(orlevel)^"] ")
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end

fun print_right_equality t fmap bmap _ = 
    let
	val (s, labels) = (PrettyPrint.getEqualStrings t (fmap,bmap))
	val _ = print (s ^ "\n")
	val _ = if (get_showlabels()) then print (labels ^ "\n") else ()
    in () end

(* goals haben die Form   A => B 
 * Die Praemisse wird mittels Funktion print_left ausgegeben 
 * Die Konklusion mittels Funktion print_right 
 * Praemissen werden nur fuer das erste Goal ausgegeben,
 * der Uebersicht halber!!
 *)
fun startexp (IMP (C "TRUE", b)) fmap bmap 1 = 
    print_right_equality b fmap bmap 1 (* orlevel *)

  | startexp (IMP (a,b) ) fmap bmap 1 = 
    let
	val _ = print_left a fmap bmap 1  (* andlevel *)
	val _ = print("\n==>\n") 
	val _ = print_right b fmap bmap 1 (* orlevel *)
    in () 
    end
(* Falls nicht erstes Goal, printe nur Conclusion *)
  | startexp (IMP (a,b))  fmap bmap _ = print_right b fmap bmap 1
  | startexp t fmap bmap _ = print_right t fmap bmap 1


(* Hauptroutine: Jedes goal innerhalb der Liste aller goals, wird gedruckt *)
fun print_goal' x fmap bmap (l, max) = 
    let 
	val _ = if (l=1) then print ("> ") else ()
	val _ = ( print ("Goal " ^ Int.toString(l) ^ "/" 
			 ^ Int.toString(max) ^ ": \n"))
    in 
	(startexp x fmap bmap l; print ("\n"))
    end





(* --------------------------------------------------------- *)
(* |                  LATEX AUSGABEN                       | *)
(* --------------------------------------------------------- *)


val initvmap = fn x:int => raise Error (" INIT VMAP ERROR !! ")
fun cons(f,s) = fn x:int => if x = 0 then s else f(x-1)

datatype latexout = C of char | E of int

fun gettokens s = 
    let 
	val chs = explode s 
    
	val (_,newchs) = foldl ( fn (c ,( n, liste)) => 
	      ( case c of
		   (#" ") => ( n+1, liste )
                  | z       => if (n > 0 ) then ( 0, ((C z)::(E n)::liste) ) else (0, (C z)::liste ) 
		       )
		       ) (0, nil) chs
    in
	newchs
    end


fun filter ("plus") = "+"
  | filter ("times") ="*"
  | filter ("minus") = "-"
  | filter ("less") = "$<$"
  | filter ("lesseq") = "$\\leq$"
  | filter ("greater") = "$>$"
  | filter ("greatereq") = "$\\geq$"
  | filter s = 
    let 
	val slist = String.fields (fn c => c = (#"_") ) (s)
    in
	if ( (List.length(slist)) > 1 ) 
	    then
		foldl ( fn (s,result) => result ^ "\\_" ^ s ) (hd(slist)) (tl(slist))
	else
	    (hd(slist))
    end  


fun comm2string (ELIMCMD _) = "Eliminierung"
  | comm2string (REMOVECMD (d,i)) = (case d of LEFT => "Left " | RIGHT => "Right ")^ "Remove " ^ Int.toString(i)
  | comm2string (LAMCMD) = "$\\beta / \\eta$ Reduktion"
  | comm2string (MOVECMD x) = "Bool (Move) " ^ ( case x of RIGHT => "rechts" | LEFT => "links" )
  | comm2string (SHIFTCMD (x, y, anz )) = "Bool (Shift) " ^
    (case x of LEFT => "Left " | RIGHT => "Right ") 
   ^ Int.toString(anz) ^ ( case y of UP => " up" | DOWN => " down" )
  | comm2string (GOALSHIFTCMD i) = "Subgoalshifting "^Int.toString(i) ^ " positions"
  | comm2string (INTROCMD _) = "Introduction"
  | comm2string (APPLYCMD x) = 
	( case x of
	     DEFINITION(s) => "Apply "^filter(s)
	   | MODUSPONENS(i,j) => ( "Apply " 
				  ^ Int.toString(i) 
				  ^ " to " ^ Int.toString(j) ))
		 
(*  | comm2string (SUBSTEQ (_,(mode,d)) ) = "Rewitution mit " ^ 
	      ( case mode of
		    LAXIOM(name) => name 
		  | PREMISSE(nr) => "Praemisse (" ^ Int.toString(nr) ^ ") " )
		    ^ (case d of LEFT => "Left " | RIGHT => "Right ") *)

  | comm2string (CUTCMD _) = "Cut Rule"
  | comm2string (INSTANCECMD (i,_)) ="Instanziiere $\\Gamma$ "^Int.toString(i)
  | comm2string (SIMPLIFYCMD) = "Bool"
(* Der Vollstaendigkeit halber *)
  | comm2string (HELPCMD) = "Hilfe"
  | comm2string (PRINTCMD) = "Print"
  | comm2string (SHOWLABELSCMD) = "Show Labels"
  | comm2string (LATEXCMD s) = "Latex " ^ s 
  | comm2string (UNDOCMD i) = "Undo" ^ Int.toString(i)
  | comm2string (EXITCMD) = "Exit"
  | comm2string (RESETCMD) = "Reset"
  | comm2string (POSCMD _) = "Positionsabfrage"
 


fun commeq2string (REFEQ) = "=Reflex"
  | commeq2string (SYMEQ _) = "=Symmetrie"
  | commeq2string (TRANSEQ _) = "=Trans"
  | commeq2string (INSERTEQ (_,(name,_))) = "Einsetzung Var "^name
  | commeq2string (REWEQ (_,(mode)) ) = "Ersetzung durch " ^ 
	      ( case mode of
		    (LAXIOM(name,_)) => name 
		  | (EQUALITY (_)) => "neues Lemma"
		  | (PREMISSE(nr,d)) => "Praemisse (" ^ Int.toString(nr) ^ ") " 
		    ^ (case d of LEFT => "links " | RIGHT => "rechts ") 
		  | (BETA) => "Beta-Reduktion"
		  | (ETA) => "Eta-Reduktion" )


(* Typausgabe - Funktionen *)
fun ty2latex (TA (a,b)) = ( atty2latex a ) ^ " $\\rightarrow$ " ^ (ty2latex b)
  | ty2latex t          = atty2latex t
and atty2latex (TC ("bool")) = " $\\mathbb{B}$ "
  | atty2latex (TC (a)) = a
  | atty2latex t        = "( " ^ ty2latex t ^ " )"


fun concat (s::ss) = (concat ss) ^ s
  | concat (nil)   = ""

fun genspace i = if (i = 0) then "" else (" ") ^ (genspace (i-1))

(* Achtung Input Liste ist reversiert !!! *)
fun LatexFilter' ( (C #"/")::(C #"\\")::strings) result = LatexFilter' strings (("$\\forall$")::result)
  | LatexFilter' ( (C #"\\")::(C #"/")::strings) result = LatexFilter' strings (("$\\exists$")::result)
  | LatexFilter' ( (C #"\\")::strings) result = LatexFilter' (strings) (("$\\lambda$")::result) 
  | LatexFilter' ( (C #">")::(C #"=")::strings) result = LatexFilter' strings (("$\\Rightarrow$")::result)
(*  | LatexFilter' ( (C #"=")::strings) result = LatexFilter' strings (("$\\doteq$")::result) *)
  | LatexFilter' ( (C #"+")::strings) result = LatexFilter' strings (("$\\lor$")::result)
  | LatexFilter' ( (C #"&")::strings) result = LatexFilter' strings (("$\\land$")::result)
  | LatexFilter' ( (C #"~")::strings) result = LatexFilter' strings (("$\\lnot$")::result)
  | LatexFilter' ( (E i)::(C #"\n")::strings)  result = 
    let 
	val command = "\\hspace*{" ^ Int.toString(i) ^ "mm} "
    in 
	LatexFilter' strings ( ("\\\\\n")::command::result)
    end
  | LatexFilter' ( (E i)::strings) result = LatexFilter' strings ((genspace i)::result)
  | LatexFilter' ( (C #"\n")::strings) result = LatexFilter' strings (("\\\\\n")::result)
  | LatexFilter' ( (C #"E")::(C #"U")::(C #"R")::(C #"T")::strings) result = LatexFilter' strings (("$\\top$")::result)
  | LatexFilter' ( (C #"E")::(C #"S")::(C #"L")::(C #"A")::(C #"F")::strings) result = 
      LatexFilter' strings (("$\\bot$")::result)
  | LatexFilter' ( (C #"?")::strings) result = LatexFilter' strings (("$\\Gamma$")::result)
  (* replaces commando char _  of latex by \_ *)
  | LatexFilter' ( (C #"_")::strings) result = LatexFilter' strings (("\\_")::result )
  | LatexFilter' ( (C #"s")::(C #"u")::(C #"l")::(C #"p")::strings) result  = 
      LatexFilter' strings (("+")::result)
  | LatexFilter' ( (C #"s")::(C #"u")::(C #"n")::(C #"i")::(C #"m")::strings) result  = 
      LatexFilter' strings (("-")::result)
  | LatexFilter' ( (C #"s")::(C #"e")::(C #"m")::(C #"i")::(C #"t")::strings) result  = 
      LatexFilter' strings (("*")::result)

  | LatexFilter' ( (C #"s")::(C #"s")::(C #"e")::(C #"l")::strings) result  = 
      LatexFilter' strings (("$<$")::result)
  | LatexFilter' ( (C #"r")::(C #"e")::(C #"t")::(C #"a")::(C #"e")::(C #"r")::(C #"g")::strings) result  = 
      LatexFilter' strings (("$>$")::result)
  | LatexFilter' ( (C #"q")::(C #"e")::(C #"r")::(C #"e")::(C #"t")::(C #"a")::(C #"e")::(C #"r")::(C #"g")::strings) 
      result  =  LatexFilter' strings (("$\\geq$")::result)
  | LatexFilter' ( (C #"q")::(C #"e")::(C #"s")::(C #"s")::(C #"e")::(C #"l")::strings) result  = 
      LatexFilter' strings (("$\\leq$")::result)
  
  | LatexFilter' ( (C c)::strings) result = LatexFilter' strings ( (Char.toString(c))::result )
  | LatexFilter' nil result = rev result 

fun LatexFilter s = 
    let
	val tokens = gettokens s
	val subst_tokens = LatexFilter' tokens nil
    in
	concat subst_tokens
    end



(*     **************************************************************        *)


fun lprint_left (AND(a,b)) fmap bmap andlevel = 
    let
	val res' = "("^Int.toString(andlevel)^") "
	val res  = res' ^  LatexFilter(PrettyPrint.getPPString a (fmap,bmap)) ^ " \\\\\n"
    in res ^ (lprint_left b fmap bmap (andlevel + 1))
    end

  | lprint_left t fmap bmap andlevel = 
    let
	val res' =  if (andlevel = 1 ) then "\\hspace*{5mm} " else "("^Int.toString(andlevel)^") "
	val res  = res' ^ LatexFilter(PrettyPrint.getPPString t (fmap,bmap)) ^ "\\\\\n"
    in res
    end

(* fun lprint_right t fmap bmap  = LatexFilter(PrettyPrint.getPPString t (fmap,bmap)) *)
fun lprint_right (OR(a,b)) fmap bmap orlevel = 
    let
	val res' = "("^Int.toString(orlevel)^") "
	val res  = res' ^  LatexFilter(PrettyPrint.getPPString a (fmap,bmap)) ^ " \\\\\n"
    in res ^ (lprint_right b fmap bmap (orlevel + 1))
    end

  | lprint_right t fmap bmap orlevel = 
    let
	val res' =  if (orlevel = 1 ) then "\\hspace*{5mm} " else "("^Int.toString(orlevel)^") "
	val res  = res' ^ LatexFilter(PrettyPrint.getPPString t (fmap,bmap)) ^ "\\\\\n"
    in res
    end

(* goals haben die Form   A => B 
 * Die Praemisse wird mittels Funktion print_left ausgegeben 
 * Die Konklusion mittels Funktion print_right 
 *)
fun lstartexp (IMP (a,b) ) fmap bmap= 
    let
	val res'' = lprint_left a fmap bmap 1
	val res'  = res'' ^ ("\\textbf{$\\Rightarrow$\\\\}\n")
	val res   = res' ^ lprint_right b fmap bmap 1 ^ "\\\\\n"
    in res
    end
  |lstartexp t fmap bmap = lprint_right t fmap bmap 1

fun latex_goals (x, fmap, bmap ) = lstartexp x fmap bmap 
fun print_goal (x, fmap, bmap ) (n, max) = print_goal' x fmap bmap (n,max) 

end (* of local open *)				    