(* Translates the sml Files into one large ned.sml *)

(* List of files, must be copied including decision if structure is needed *)
val input_files = 
[ ("Hilfe.sml", true ), 
  ("Paths.sml",true ),
  ("Data.sml",true ),
  ("ETT.sml",true ),
  ("PrettyPrint.sml",true ),
  ("Visual.sml",true ),
  ("Typecheck.sml",true ),
  ("Env.sml",false),
  ("Definition.sml", false ), 
  ("Gamma.sml", false),
  ("Goal.sml", false),
  ("Position.sml",true ),
  ("LatexOutput.sml",true ),
  ("Equality.sml",true ),
  ("Intro.sml",true ),
  ("Parser.sml",true ),
  ("Lexer.sml",true ),
  ("Elim.sml",true ),
  ("Instance.sml",true ),
  ("Cut.sml",true ),
  ("Move.sml",true ),
  ("Remove.sml",true ),
  ("Shift.sml",true ),
  ("Bool.sml",true ),
  ("Lambda.sml",true ),
  ("Apply.sml",true ),
  ("Unify.sml",false),
  ("Rewriting.sml",true ),
  ("RefSymTrans.sml",true ),
  ("Insert.sml",true ),
  ("Main.sml",true )   ]
 

val bibs = (  "val bibs = [ \"Int\" , \"Polyhash\", \"String\", \"Char\", \"Listsort\", \"PP\" , \"Process\" , \n "
	    ^ "             \"IO\", \"Obj\", \"Lexing\", \"Parsing\", \"Nonstdio\" ] ;\n\n" )

val fold = ("foldl (fn (b,_)=> load b ) () bibs ;\n\n")


fun copy instream outstream =
     case TextIO.inputLine instream of
	"" => ()
      | s  => ( ( if not (String.isPrefix "(*-NED-*)" s) 
		      then (TextIO.output(outstream,s))
		  else () );
	       copy instream outstream ) 

fun copy_filter instream outstream name = 
      case TextIO.inputLine instream of
	"" => ()
      | s  => let
	    
	    (* Umwandlung der Structures mit Signatures *)
		  val bez = String.substring(name, 0, (String.size(name)) - 4)
		  val oldline = "structure "^bez^" :> "^bez^" ="
		  val newline = "structure "^bez^" =\n"
	      in
		(( if (String.isPrefix oldline s) then (TextIO.output(outstream, newline)) 
		   else (TextIO.output(outstream, s)) );
		   copy_filter instream outstream name)
	      end 


fun generate_welcome(outstream) = 
let
    val _ = TextIO.output(outstream,(  "(*\n"
				     ^ " * Interactive Proof Assistant NED for Higher-Order-Logic * \n"
				     ^ " * Version 1.3 -- 2005 \n"
				     ^ " * ned.sml (compact) \n"
				     ^ " *) \n\n" ))
in () end
    



fun generate_bibs(outstream) =
let
    val _ = TextIO.output (outstream, "val _ = print (\"\\n\\n>> Loading Libraries ...\\n\\n\") \n")
    val _ = TextIO.output (outstream, bibs)
    val _ = TextIO.output (outstream, fold)
    val _ = TextIO.output (outstream, "val _ = print (\"\\n\\n>> Loading NED ...\\n\\n\") \n")
in () end

fun generate_header(name) = 
    ( "(* ==========================================================================================\n"
     ^"                             " ^ name ^"\n"
     ^"   ========================================================================================== *)\n\n" )
     
fun generate_struct_header(name) = (  "structure " ^ String.substring(name, 0, (String.size(name)) - 4) ^"= \n"
				    ^ "struct\n" )
    
fun generate_struct_end(name) = ("end (* of structure "^name^" *)\n" )


fun merge liste outstream =
    foldl ( fn ((s, d),_)=> 
	    let
		fun write(s) = TextIO.output(outstream, s)
		val instream = TextIO.openIn(s)
		val _ = TextIO.output(outstream, generate_header(s))

		val _ = if d then ( write(generate_struct_header(s)) ;
				   copy instream outstream )
			else copy_filter instream outstream s
	
		val _ = TextIO.closeIn(instream)
		
		val _ = if d then write(generate_struct_end(s)) else ()
	    in ()
	    end
    
	   ) () liste


fun generate_structNEDHeader(outstream) = 
let
    fun write(s) = TextIO.output(outstream, s ^ "\n")
    val _ = write ("signature NED = ")
    val _ = write ("sig")
    val _ = write (" (* Loads a NED - File and starts the NED - Interpreter *) ")
    val _ = write ("    val load  : string -> unit \n")
    val _ = write (" (* Starts the NED - Interpreter  *) ")
    val _ = write ("    val start : unit -> unit \n")
    val _ = write ("end")
    val _ = write ("")
    val _ = write ("structure NED :> NED = ")
    val _ = write ("struct ")
in () 
end

fun generate_structNEDEnd(outstream) = 
    let
	fun write(s) = TextIO.output(outstream, s ^ "\n")
	val _ = write ("")
	val _ = write ("fun load (s) = Main.start (s::nil) \n")
	val _ = write ("fun start () = Main.start nil \n")
	val _ = write ("end")
	val _ = write ("\n\nval _ = print (\"Start NED with:\\nNED.start()  or\\nNED.load(NED_FILE)\\n\\n\\n \")")
    in ()
    end


val _ = 
    let
	val outputFile = hd(CommandLine.arguments()) handle empty => "ned.sml"
	val outstream = TextIO.openOut(outputFile)
	val _ = generate_welcome(outstream)
	val _ = generate_bibs(outstream)
	val _ = generate_structNEDHeader(outstream)
	val _ = merge input_files outstream
	val _ = generate_structNEDEnd(outstream)
	val _ = TextIO.closeOut(outstream)
    in () end