(*
 * Interactive Proof Assistant NED for Higher-Order-Logic * 
 * Version 1.3 -- 2005 
 * ned.sml (compact) 
 *) 

val _ = print ("\n\n>> Loading Libraries ...\n\n") 
val bibs = [ "Int" , "Polyhash", "String", "Char", "Listsort", "PP" , "Process" , 
              "IO", "Obj", "Lexing", "Parsing", "Nonstdio" ] ;

foldl (fn (b,_)=> load b ) () bibs ;

val _ = print ("\n\n>> Loading NED ...\n\n") 
signature NED = 
sig
 (* Loads a NED - File and starts the NED - Interpreter *) 
    val load  : string -> unit 

 (* Starts the NED - Interpreter  *) 
    val start : unit -> unit 

end

structure NED :> NED = 
struct 
(* ==========================================================================================
                             Hilfe.sml
   ========================================================================================== *)

structure Hilfe= 
struct
(* Hilfe.sml - gibt einen Ueberblick der einzelnen Kommandos 
 * mit Anwendungshilfe 
 *)

fun line(n:int)= implode(List.tabulate(n,fn x=> #"-" )) ^ "\n"

val list = [ ("type X\t\t\t",      "declares a new type X\n") ,
	     ("var x:t\t\t\t",     "declares a free variable x with Type t\n"),
	     ("const x:t\t\t",     "declares a constant x with Type t\n"),
	     ("def name = TER\t\t","defines a shorthand name for TERM\n"),
	     ("lemma name = FOR\t","specifies a new lemma name, which must be proven after usage\n"),
	     ("axiom name = FOR\t","specifies a new axiom name\n"),
	     ("goal FOR\t\t",      "specifies a new theorem to proof\n"),
	     ("",line(80)),
	     ("ref\t\t\t",         "Reflexivity Rule \n"),
	     ("sym\t\t\t",         "Symmetric Rule \n"),
	     ("trans TER\t\t",    "Transitivity Rule \n"),
	     ("subst var = TER \t", "Substitution Rule \n"),
	     ("pos [nr]\t\t",      "Calls a java program to determine subterm positions \n"),
	     ("rew (pos) with beta\t","applies the Beta-Rule on position pos \n"),
	     ("rew (pos) with eta\t", "applies the Eta-Rule on position pos \n"),
	     ("rew (pos) with (N=M)\t","Rewriting Rule: introduces the new lemma (N=M) and tries \n"),
	     ("\t\t\t",             "to rewrite the subterm on position pos with the new lemma\n"),
	     ("rew (pos) with \t\t", "Rewriting Rule: rewrites the subterm on position pos\n"),
	     ("    (name/nr) (l/r)\t", "with the (l)eft/(r)ighthandside of an axiom/lemma \n"),
	     ("\t\t\t",            "(name) or a premise (nr) \n"),
	     ("",line(80)),
	     ("intro\t\t\t",       " |- x => y\t\t << intro >>\t\tx |- y\n"),
	     ("\t\t\t",            " |- x & y\t\t << intro >>\t\t|- x   (2/1) \n\t\t\t\t\t\t\t\t\t|- y   (2/2)\n"),
	     ("\t\t\t",            " |- ~x\t\t\t << intro >>\t\tx |- FALSE\n"),
	     ("\t\t\t",            " |- /\\ x. f(x)\t\t << intro >>\t\t|- f(? 0) (Gamma Var)\n"),
	     ("\t\t\t",            " |- \\/ x. f(x)\t\t << intro >>\t\t|- x\n"),
	     ("","\n"),
	     ("elim \t\t\t",       " x + y |- z\t\t << elim  >>\t\tx |- z   (2/1) \n\t\t\t\t\t\t\t\t\ty |- z   (2/2)\n"),
	     ("\t\t\t",            " \\/ x. f(x) |- z\t << elim  >>\t\tf(? 0) |- z\n"),
	     ("\t\t\t",            " /\\ x. f(x) |- z\t << elim  >>\t\tx |- z\n"),
	     ("","\n"),
	     ("instance ganr = FOR\t", " |- (? ganr) \t\t << instance >> \t |- FOR \n"),
	     ("","\n"),
	     ("cut FOR \t\t",      " x |- y\t\t\t << cut FOR >> \t\tx|-FOR + y   (1/2) \n\t\t\t\t\t\t\t\t\tx & FOR|- y   (2/2)\n"),
	     ("","\n"),
	     ("move l\t\t\t",      " [1] x \n"),
	     ("\t\t\t",            " [2] y |- z\t\t << move l >>\t\t[1] y |- ~x + z\n"),
	     ("","\n"),
	     ("move r\t\t\t",      " x |- [1] y \n"),
	     ("\t\t\t",            "      [2] z \t\t << move r >>\t\t ~y & x |- z\n"),
	     ("","\n"),
	     ("remove [l/r] (nr)\t", " removes the the premise(l) / conclusion(r) nr\n"),
	     ("shift (l/r)(u/d)[nr]\t", " Cyclic (u)p/(d)own shift of premises(l)/conclusions(r)\n"),
	     ("\t\t\t",                 " by nr positions\n"),
	     ("lam\t\t\t",         " forces a beta- and eta reduction on the current formula\n"),
	     ("simplify\t\t",      " applies the BOOL-Rule in order to simplify the formula\n"),
	     ("",line(80)),
	     ("print\t\t\t",       " outputs the current goals\n"),
	     ("labels\t\t\t",      " switches label printing mode \n"),
	     ("latex\t\t\t",       " outputs the proof to a latex file\n"),
	     ("undo\t\t\t" ,       " makes the last command undone\n"),
	     ("reset\t\t\t",       " all goals/type/var declarations will be deleted \n"),
	     ("exit\t\t\t" ,       " exits the program \n")	    
]
	   

fun main() =let 
		val _ = print ("Available Commands : \n"^line(80))
		val f = foldl ( fn ((cmd,desc), n) => print (cmd ^ desc) ) () list
	    in
		f
	    end
end (* of structure Hilfe.sml *)
(* ==========================================================================================
                             Paths.sml
   ========================================================================================== *)

structure Paths= 
struct
(* Contains Path to external Files *)
(* please modify in case of IO-ERRORS *)

val latex_path = "./LatexOut/"
val position_path = "./"
end (* of structure Paths.sml *)
(* ==========================================================================================
                             Data.sml
   ========================================================================================== *)

structure Data= 
struct
(* Data.sml  -- the abstract syntax datatype *)
exception Error of string


type tc  = string;
type vc  = string;
type var = int;

datatype ty = TC of tc | TA of ty * ty 

(* Main Datastructures *)             
datatype sourceexpr = 
    AppS of sourceexpr * sourceexpr
  | VarS of string
  | LamS of (string * ty ) * sourceexpr
  | CstS of string                         
  | AndS of sourceexpr * sourceexpr
  | OrS  of sourceexpr * sourceexpr
  | ImpS of sourceexpr * sourceexpr
  | NotS of sourceexpr
  | EqS  of sourceexpr * sourceexpr
  | ExS  of (string * ty ) * sourceexpr
  | AllS of (string * ty ) * sourceexpr
 (* Es folgen weitere Typen, die zum Parsen benoetigt werden *) 



datatype pter =                    (* interne Repraesentation der Formeln *)
    A of pter * pter               (* Variablenname wird durchgeschleust  *)
  | V of var
  | GAMMA of ( ty * int )          (* Fuer Gamma Variablen mit Typ und Nr. *)
  | L of (ty * var) * pter
  | C of vc
  | AND of pter * pter
  | OR  of pter * pter
  | IMP of pter * pter
  | NOT of pter
  | EQ of pter * pter
  | EX  of (ty * var) * pter
  | ALL of (ty * var) * pter  


datatype pter' =                   (* uebergangsrepresentation *)
    A' of pter' * pter'  
  | V' of var
  | L' of (var * ty) * pter'
  | C' of vc
  | AND' of pter' * pter'
  | OR'  of pter' * pter'
  | IMP' of pter' * pter'
  | NOT' of pter'
  | EQ' of pter' * pter'
  | EX'  of (var * ty ) * pter'
  | ALL' of (var * ty ) * pter'



(* Sub Datastructures *)

datatype xdirection = LEFT | RIGHT
datatype ydirection = UP | DOWN

datatype kind_of_def = CLASSIC | LEMMA | AXIOM

datatype apply =  DEFINITION of string 
                | MODUSPONENS of int * int
             
datatype rew = LAXIOM of string * xdirection
  | PREMISSE of int * xdirection
  | EQUALITY of sourceexpr
  | BETA
  | ETA

datatype pos = LA of string | PO of int 

datatype extension = STANDART | MORE

(* Commands for ND calculus *)
datatype cmds = ELIMCMD of extension
              | REMOVECMD of xdirection * int
              | LAMCMD
              | MOVECMD of xdirection
	      (* LEFT/RIGHT  *  UP/DOWN  * ANZ. DER LINES *)
              | SHIFTCMD of xdirection * ydirection * int
              | GOALSHIFTCMD of int
              | HELPCMD 
              | PRINTCMD
              | SHOWLABELSCMD
              | LATEXCMD of string (*title*)
              | EXITCMD 
              | INTROCMD of extension
              | CUTCMD of sourceexpr                  (* Cut Regel mit Eingabe der Formel *)
              | INSTANCECMD of int * sourceexpr       (* Spezialkommando Instanziierung *)
              | POSCMD of int
              | APPLYCMD of apply
              | SIMPLIFYCMD
              | UNDOCMD of int
              | RESETCMD

(* Commands for Equalityrules *)
datatype cmdeqs =   REFEQ
                  | SYMEQ of xdirection
                  | TRANSEQ of sourceexpr
                  | INSERTEQ of xdirection * (string * sourceexpr)
                  | REWEQ of pos list * rew


(* Datastructure of Parseroutput *)
datatype parseexpr =
    ForS of sourceexpr                         (* Zur direkten Formeleingabe *)
  | GoalS of sourceexpr                        (* Zur Theoremeingabe *)
  | LetVarS of string * ty                     (* Variablendeklaration "var x : typ->typ...;" *)
  | LetConstS of string * ty                   (* Konstantendeklaration "const x : typ->typ...;" *)
  | DefS of string * sourceexpr * kind_of_def  (* Definitionendeklaration  "def name = expr"*)
  | TypS of string                             (* Typdeklarationen "type X" *)
  | CmdS of cmds                               (* Kommandoausfuehrung *)
  | CmdEqS of cmdeqs                           (* Kommandos Gleichheitsregeln *)
  | SaveS of string
  | ErrorS of string                           (* Fuer Parser/Lexer Errors *)
  | EmptyS                                     (* leere Eingabe *)
  | FileEndS                                   (* File Loading End *)
end (* of structure Data.sml *)
(* ==========================================================================================
                             ETT.sml
   ========================================================================================== *)

structure ETT= 
struct
(* ETT.sml implementiert grundlegende Routinen zur 
 * Modifizierung einfach getypter Lambda terme 
 *   ! Arbeitet mit DeBrujin Darstellung !
 *)
local open Data in


(* Erhoeht die freien Variablen um y positionen *)
(* Wenn y = 1 ==> Standart up  *)
fun up l (C c) y             = C c
  | up l (V x) y             = V(if x>=l then x+y else x)
  | up l (g as (GAMMA _) ) y = g   
  | up l (A(m,m')) y         = A(up l m y, up l m' y)
  | up l (L((t,i),m)) y      = L((t,i), up (l+1) m y)
  | up l (EX((t,i),m)) y     = EX((t,i), up (l+1) m y)
  | up l (ALL((t,i),m)) y    = ALL((t,i), up (l+1) m y)
  | up l (AND(m,m')) y       = AND(up l m y, up l m' y)
  | up l (OR(m,m')) y        = OR(up l m y, up l m' y)
  | up l (IMP(m,m')) y       = IMP(up l m y, up l m' y)
  | up l (EQ(m,m')) y        = EQ (up l m y , up l m' y)
  | up l (NOT(m)) y          = NOT(up l m y)

(* Erniedrigt die freien Variablen um y positionen *)
(* Wenn y = 1 ==> Standart down  *)
fun down l (C c) y           = C c
  | down l (V x) y           = V(if x>=l then x-y else x)
  | down l (g as (GAMMA _)) y= g   
  | down l (A(m,m')) y       = A(down l m y , down l m' y)
  | down l (L((t,i),m)) y    = L((t,i), down (l+1) m y)
  | down l (EX((t,i),m)) y   = EX((t,i), down (l+1) m y)
  | down l (ALL((t,i),m)) y  = ALL((t,i), down (l+1) m y)
  | down l (AND(m,m')) y     = AND(down l m y , down l m' y)
  | down l (OR(m,m')) y      = OR(down l m y, down l m' y)
  | down l (IMP(m,m')) y     = IMP(down l m y, down l m' y)
  | down l (EQ(m,m')) y      = EQ(down l m y, down l m' y)
  | down l (NOT(m)) y        = NOT(down l m y)

fun sub (C c) y n            = C c
  | sub (V x) y n            = if x=y then n else V x
  | sub (g as (GAMMA _)) y n = g (* Substitution findet in Gamma.sml statt *)
  | sub (A(m,m')) y n        = A(sub m y n, sub m' y n)
  | sub (L((t,i),m)) y n     = L((t,i), sub m (y+1) (up 0 n 1))
  | sub (EX((t,i),m)) y n    = EX((t,i), sub m (y+1) (up 0 n 1))
  | sub (ALL((t,i),m)) y n   = ALL((t,i), sub m (y+1) (up 0 n 1))
  | sub (AND(m,m')) y n      = AND( sub m y n , sub m' y n )
  | sub (OR(m,m')) y n       = OR( sub m y n , sub m' y n )
  | sub (IMP(m,m')) y n      = IMP( sub m y n , sub m' y n )
  | sub (EQ(m,m')) y n       =  EQ( sub m y n, sub m' y n )
  | sub (NOT(m)) y n         = NOT( sub m y n )


fun free x (C c)             = false
  | free x (V y)             = x=y
  | free x (g as (GAMMA _) ) = false (* !!!! *)
  | free x (A(m,m'))         = free x m orelse free x m'
  | free x (L((t,i),m))      = free (x+1) m
  | free x (EX((t,i),m))     = free (x+1) m
  | free x (ALL((t,i),m))    = free (x+1) m
  | free x (AND(m,m'))       = free x m orelse free x m'
  | free x (OR(m,m'))        = free x m orelse free x m'
  | free x (IMP(m,m'))       = free x m orelse free x m'
  | free x (EQ(m,m'))        = free x m orelse free x m'
  | free x (NOT(m))          = free x m


(* Brauche neuen Gleichheitstest, wegen unterschiedlicher Variablennamen *)
fun equal ( (EX((t1,i1),f1)), (EX((t2,i2),f2)) )  = (t1=t2) andalso equal(f1,f2)
  | equal ( (ALL((t1,i1),f1)), (ALL((t2,i2),f2))) = (t1=t2) andalso equal(f1,f2)
  | equal ( (L ((t1,i1),f1)), (L ((t2,i2),f2)) )  = (t1=t2) andalso equal(f1,f2)
  | equal ( A(x1,y1) , A(x2,y2) )                 = equal(x1,x2) andalso equal(y1,y2)
  | equal ( AND(x1,y1) , AND(x2,y2) )             = equal(x1,x2) andalso equal(y1,y2)
  | equal ( OR(x1,y1) , OR(x2,y2) )               = equal(x1,x2) andalso equal(y1,y2)
  | equal ( IMP(x1,y1) , IMP(x2,y2) )             = equal(x1,x2) andalso equal(y1,y2)
  | equal ( EQ(x1,y1) , EQ(x2,y2) )               = equal(x1,x2) andalso equal(y1,y2)
  | equal ( NOT(m1) , NOT(m2) )                   = equal(m1,m2)
  | equal ( a, b)                                 = (a=b) 


(* Bekommt zusaetzliches Argument freelistlength zum richtigen umrechnen *)
(* Urspruenglich ist Mapping von [freexs]@[bndxs] *)
(* Spaeter soll aber Mapping nur noch auf bndxs beziehen !*) 
(*
fun com (C' c)          flength = C c
  | com (V' x)          flength = V x
  | com (A'(m,m'))      flength = A(com m flength, com m' flength)
  | com (L'((x,t),m))   flength = L((t,x-flength), sub (up 0 (com m flength) 1) (x+1) (V 0))
  | com (EX'((x,t),m))  flength = EX((t,x-flength), sub (up 0 (com m flength) 1) (x+1) (V 0))
  | com (ALL'((x,t),m)) flength = ALL((t,x-flength), sub (up 0 (com m flength) 1) (x+1) (V 0))
  | com (AND'(m,m'))    flength = AND(com m flength, com m' flength)
  | com (OR'(m,m'))     flength = OR(com m flength, com m' flength)
  | com (IMP'(m,m'))    flength = IMP(com m flength, com m' flength)
  | com (EQ'(m,m'))     flength = EQ(com m flength, com m' flength)
  | com (NOT'(m))       flength = NOT(com m flength)
*)

(* Ersetzt subterm p durch subterm n *)
fun termsub (t as (C c)) p n = if equal(p,t) then n else t
  | termsub (t as (V x)) p n = if equal(p,t) then n else t
  | termsub (t as ((GAMMA _))) p n = if equal(p,t) then n else t
  | termsub (t as (A(m,m'))) p n = if equal(p,t) then n else (A (termsub m p n, termsub m' p n) )
  | termsub (t as (L((typ,i),m))) p n = if equal(p,t) then n else (L((typ,i), termsub m (up 0 p 1) (up 0 n 1)))
  | termsub (t as (EX((typ,i),m))) p n = if equal(p,t) then n else (EX((typ,i), termsub m (up 0 p 1) (up 0 n 1)))
  | termsub (t as (ALL((typ,i),m))) p n = if equal(p,t) then n else (ALL((typ,i), termsub m (up 0 p 1) (up 0 n 1)))
  | termsub (t as (AND(m,m'))) p n = if equal(p,t) then n else (AND( termsub m p n , termsub m' p n ))
  | termsub (t as (OR(m,m'))) p n = if equal(p,t) then n else (OR( termsub m p n , termsub m' p n ))
  | termsub (t as (IMP(m,m'))) p n = if equal(p,t) then n else (IMP( termsub m p n , termsub m' p n ))
  | termsub (t as (EQ(m,m'))) p n = if equal(p,t) then n else (EQ( termsub m p n , termsub m' p n ))
  | termsub (t as (NOT(m))) p n = if equal(p,t) then n else (NOT( termsub m p n ))



(* Gibt eine Liste freier Variablen inkl. Gamma Variablen zurueck *)
fun getfreevars' (C c) n xs               = xs 
                                        (* muss V(y-n) ausgeben, da evtl. erhoeht *)
  | getfreevars' (V y) n xs               = if ( y >= n ) then ((V (y-n))::xs) else xs
  | getfreevars' (g as (GAMMA _) ) n xs   = (g::xs)
  | getfreevars' (A(m,m')) n xs           = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (L((t,i),m)) n xs        = getfreevars' m (n+1) xs
  | getfreevars' (EX((t,i),m)) n xs       = getfreevars' m (n+1) xs
  | getfreevars' (ALL((t,i),m)) n xs      = getfreevars' m (n+1) xs
  | getfreevars' (AND(m,m')) n xs         = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (OR(m,m')) n xs          = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (IMP(m,m')) n xs         = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (EQ(m,m')) n xs          = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (NOT(m)) n xs            = (getfreevars' m n xs)
fun getfreevars term = getfreevars' term 0 nil


(* gibt eine Liste gebundener Variablen zurueck, die wegen Einsetzung,
   umbenannt werden muessen 
 *)

fun decide i m (bmap,fmap,cmap) l =
    let
	val bndname  = bmap i
	val freelist = getfreevars' m (l+1) nil
	val f = fn (p,result) => 
	    let
		val i = ( case p of
		         (V n) => n
		       | _ => (~1)  ) (* produziert Fehler *) 
		val fname = (fmap i) handle Error _ => ""
		val cname = (cmap i) handle Error _ => ""
	    in
		result orelse (bndname = fname) orelse (bndname = cname) 	
	    end
	val critical = foldl f false freelist
    in
	if critical then [i] else nil
    end

fun getbndvars' l (C c)            _    = nil
  | getbndvars' l (V x)            _    = nil
  | getbndvars' l (g as (GAMMA _)) _    = nil
  | getbndvars' l (A(m,m'))        maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (L((t,i),m))     maps = (decide i m maps l) @ (getbndvars' (l+1) m maps)
  | getbndvars' l (EX((t,i),m))    maps = (decide i m maps l)  @ (getbndvars' (l+1) m maps)
  | getbndvars' l (ALL((t,i),m))   maps = (decide i m maps l)  @ (getbndvars' (l+1) m maps)
  | getbndvars' l (AND(m,m'))      maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (OR(m,m'))       maps = (getbndvars' l m maps) @ (getbndvars' l m' maps) 
  | getbndvars' l (IMP(m,m'))      maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (EQ(m,m'))       maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (NOT(m))         maps = (getbndvars' l m maps) 

fun getCriticalBnds pter maps = getbndvars' 0 pter maps




end (* of local open *)

end (* of structure ETT.sml *)
(* ==========================================================================================
                             PrettyPrint.sml
   ========================================================================================== *)

structure PrettyPrint= 
struct
(* 
 *   Pretty Printer for HOL Formula
 *)
local open Data PP in 

(* Ausdruecke werden mit moeglichst wenigen Klammern 
 * geprintet;
 *  fmap - stellt die Variablenumgebung der freien Variablen dar ( int -> string )
 *  bmap - Variablenumgebung der gebundenen Variablen ( int -> string ) 
 *  vmap - wird dynamisch angepasst V 0 -> neu gebundene Variable  
 *)


val initvmap = fn x:int => raise Error (" INIT VMAP ERROR !! ")
fun cons(f,s) = fn x:int => if x = 0 then s else f(x-1)


fun ppexpr (fmap,bmap) pps e0 = 
    let 
	val vmap = initvmap


	fun exp (L((t,i),e))   fmap bmap vmap l = (begin_block pps INCONSISTENT 0;
						   add_string pps "\\ ";
						   add_string pps (bmap i);
						   add_string pps ". ";
						   add_break pps (0, 1); 
						   exp e fmap bmap (cons(vmap,bmap i)) (l+1);
						   end_block pps)
	  | exp (EX((t,i),e))  fmap bmap vmap l = (begin_block pps CONSISTENT 0;
						   add_string pps "/\\ ";
						   add_string pps (bmap i);
						   add_string pps ". ";
						   (* add_break pps (0, 1); *)
						   exp e fmap bmap (cons(vmap,bmap i)) (l+1);
						   end_block pps)
	  | exp (ALL((t,i),e)) fmap bmap vmap l = (begin_block pps CONSISTENT 0;
						   add_string pps "\\/ ";
						   add_string pps (bmap i);
						   add_string pps ". ";
						   (* add_break pps (0, 1); *)
						   exp e fmap bmap (cons(vmap,bmap i)) (l+1);
						   end_block pps)
	  | exp t fmap bmap  vmap l = eqexp t fmap bmap vmap l
	    
	and eqexp (EQ(a,b)) fmap bmap vmap l =  (begin_block pps CONSISTENT 0;
						 impexp a fmap bmap vmap l; 
						 add_string pps " = ";
						 add_break pps (0, 1); 
						 eqexp b fmap bmap vmap l;
						 end_block pps)
	  | eqexp t         fmap bmap vmap l = impexp t fmap bmap vmap l 
	    
	and impexp (IMP(a,b))   fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
						     orexp a fmap bmap vmap l; 
						     add_string pps " => ";
						     add_break pps (0, 1); 
						     impexp b fmap bmap vmap l;
						     end_block pps)
	  | impexp t            fmap bmap  vmap l = orexp t fmap bmap vmap l
	    
	and orexp (OR(a,b)) fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
						     andexp a fmap bmap vmap l; 
						     add_string pps " + ";
						     add_break pps (0, 1); 
						     orexp b fmap bmap vmap l;
						     end_block pps)
	  | orexp t         fmap bmap  vmap l = andexp t fmap bmap vmap l
	    
	and andexp (AND(a,b)) fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
						     negexp a fmap bmap vmap l; 
						     add_string pps " & ";
						     add_break pps (0, 1); 
						     andexp b fmap bmap vmap l;
						     end_block pps)
	  | andexp t          fmap bmap  vmap l = negexp t fmap bmap vmap l
	    
	and negexp (NOT(a)) fmap bmap  vmap l = (add_string pps "~ " ; negexp (a) fmap bmap vmap l)
	  | negexp t        fmap bmap  vmap l = appexp t fmap bmap vmap l
	    
	and appexp (A(a,b)) fmap bmap  vmap l =(begin_block pps CONSISTENT 0;
						     appexp a fmap bmap vmap l; 
						     add_string pps " ";
						     (* add_break pps (0, 1); *)
						     atexp b fmap bmap vmap l;
						     end_block pps)
 	  | appexp t        fmap bmap  vmap l = atexp t fmap bmap vmap l
	    
	and atexp (V(n)) fmap bmap  vmap l = add_string pps (if ( n >= l ) then fmap (n-l) else vmap n)
	  | atexp (GAMMA(_,i)) fmap bmap vmap l = add_string pps ("? "^Int.toString(i))
	  | atexp (C(s)) fmap bmap  vmap l = add_string pps s
	  | atexp t      fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
					      add_string pps "( " ;
					      exp t fmap bmap vmap l ;
					      add_string pps  " )" ;
					      end_block pps )
	    
	    

    in
        begin_block pps INCONSISTENT 0; 
        exp e0 fmap bmap vmap 0;
        end_block pps
    end


(* For Meta installation val _ = installPP ppexpr; *)

fun getPPString term maps = pp_to_string 80 (ppexpr maps) term 


(* For Equality Deduction Mode *)
(* ohne Pretty Printing, aber mit Positions Labels *)

fun ppequal pter (fmap,bmap) = 
    let 
	val vmap = initvmap
	val label = ref 65 (* starte bei #"A" *)

	fun putLabel (label, size) = 
	    let
		val label2string = Char.toString(Char.chr(label))
	        fun blanks i = (implode (List.tabulate (i, (fn n => #" "))))
	    in ( label2string ^ (if size < 0 then "" else (blanks size)) )
	    end
	    
	fun exp (L((t,i),e))   (fmab, bmap, vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( exp e (fmap, bmap, (cons(vmap,bmap i))) (l+1))

		val fs  = "\\ " ^ (bmap i) ^ ". " ^ rets
		val ls  = ( putLabel (clabel, 3 + String.size(bmap i)) ) ^ retl
	    in (fs, ls)
	    end
	
	  | exp (EX((t,i),e))   (fmab, bmap, vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( exp e (fmap, bmap, (cons(vmap,bmap i))) (l+1))

		val fs  = "/\\ " ^ (bmap i) ^ ". " ^ rets
		val ls  = ( putLabel (clabel, 4 + String.size(bmap i)) ) ^ retl
	    in (fs, ls)
	    end

	  | exp (ALL((t,i),e))   (fmab, bmap, vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( exp e (fmap, bmap, (cons(vmap,bmap i))) (l+1))

		val fs  = "\\/ " ^ (bmap i) ^ ". " ^ rets
		val ls  = ( putLabel (clabel, 4 + String.size(bmap i)) ) ^ retl
	    in (fs, ls)
	    end

	  | exp t maps l = eqexp t maps l
	    
	and eqexp (EQ(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( impexp a maps l ) 
		val s = " = " 
		val (rets2, retl2) = ( eqexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 1) ^ retl2
	    in (fs, ls)
	    end
						 
	  | eqexp t maps l = impexp t maps l 
	    
	and impexp (IMP(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( orexp a maps l ) 
		val s = " => " 
		val (rets2, retl2) = ( impexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 2) ^ retl2
	    in (fs, ls)
	    end
	  | impexp t maps l = orexp t maps l
	    
	and orexp (OR(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( andexp a maps l ) 
		val s = " + " 
		val (rets2, retl2) = ( orexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 1) ^ retl2
	    in (fs, ls)
	    end

	  | orexp t maps l = andexp t maps l
	    
	and andexp (AND(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( negexp a maps l ) 
		val s = " & " 
		val (rets2, retl2) = ( andexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 1) ^ retl2
	    in (fs, ls)
	    end

	  | andexp t maps l = negexp t maps l
	    
	and negexp (NOT(a)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( negexp a maps l ) 
		val s = "~ " 

		val fs = s ^ rets
		val ls = putLabel(clabel, 1) ^ retl
	    in (fs, ls)
	    end

	  | negexp t maps l = appexp t maps l
	    
	and appexp (A(a,b)) maps l = 
	    let val (rets1, retl1) = appexp a maps l
		val (rets2, retl2) = atexp b maps l 
		val fs = rets1 ^ " " ^ rets2
		val ls = retl1 ^ " " ^ retl2
	    in (fs, ls)
	    end

	  | appexp t maps l = atexp t maps l
	    
	and atexp (V(n)) (fmap,_ , vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val s = (if ( n >= l ) then fmap (n-l) else vmap n)
		val size = String.size(s)
	    in (s, putLabel(clabel, size - 1))  (* put clabel instead 32 to label variables also *)
	    end
	  | atexp (GAMMA(_,i)) _ _ = 
	    let val i2s = Int.toString(i) 
		val s  = ("? " ^ i2s)
	    in (s, putLabel(32, 1 + String.size(i2s))) 
	    end
	  | atexp (C(s)) _ _ = 
	    let val clabel = (!label) before (label := !label + 1)
		val size = String.size(s)
	    in (s, putLabel(clabel, size - 1)) (* put clabel instead 32 to label constants also *)
	    end
	  | atexp t maps l = 
	    let val (rets, retl) = exp t maps l 
		val fs = "( " ^ rets ^ " )"
		val ls = "  " ^ retl ^ "  "
	    in (fs, ls)
	    end
    in
	(label := 65; 
	 exp pter (fmap, bmap, vmap) 0)
    end


fun getEqualStrings term maps = ppequal term maps


end; (* of local open *) 
end (* of structure PrettyPrint.sml *)
(* ==========================================================================================
                             Visual.sml
   ========================================================================================== *)

structure Visual= 
struct
local open Data in

(* --------------------------------------------------------- *)
(* |             AUSGABE ROUTINEN FUER STDOUT              | *)
(* --------------------------------------------------------- *)


(* Label Printing mode *)
val showlabels = ref false
fun set_showlabels() = ( showlabels := ( !showlabels = false ) ;
			print ("> Label Printing : " ^ (if !showlabels then "on" else "off") ^ "\n\n") )
fun get_showlabels() = !showlabels 

(* Typausgabe - Funktionen *)
fun ty2string (TA (a,b)) = ( atty2string a ) ^ "->" ^ (ty2string b)
  | ty2string t          = atty2string t
and atty2string (TC (a)) = a
  | atty2string t        = "( " ^ ty2string t ^ " )"

(* DEBUG AUSGABEN *)
fun pter2string (A(a,b)) = "A(" ^ pter2string (a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (V(n))   = "V " ^ Int.toString(n)
  | pter2string (GAMMA(t,i)) = "GAMMA ("^Int.toString(i)^" :"^ty2string(t)^")"
  | pter2string (L((t,s),e)) = "L(" ^ ty2string t ^ ", " ^ pter2string (e) ^ ")"
  | pter2string (EX((t,s),e)) = "EX ("  ^ ty2string t ^ ", " ^ pter2string (e) ^ ")"
  | pter2string (ALL((t,s),e)) = "ALL(" ^ ty2string t ^ ", " ^ pter2string (e) ^ ")"
  | pter2string (C(s)) =  "C(" ^ s ^ ")"
  | pter2string (AND(a,b))  = "AND(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (OR(a,b))  = "OR(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (IMP(a,b))  = "IMP(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (EQ(a,b))  = "EQ(" ^ pter2string(a) ^ ", " ^ pter2string(b) ^ ")"
  | pter2string (NOT(a))  = "NOT(" ^ pter2string(a) ^ ")"
  
fun parse2string (GoalS _) = "GoalS"
  | parse2string (ForS _)  = "ForS"
  | parse2string (LetVarS _)  = "LetVarS"
  | parse2string (LetConstS _) = "LetConstS"
  | parse2string (DefS _)  = "DefS"
  | parse2string (TypS _)  = "TypS"
  | parse2string (CmdS _)  = "CmdS"
  | parse2string (CmdEqS _)   = "EqS"
  | parse2string (SaveS _) = "Save Goal"
  | parse2string (ErrorS _)= "Parser Error"
  | parse2string (FileEndS)= "File loaded"
  | parse2string (EmptyS)  = "EmptyS"



(* Durch AND verknuepfte Teilausdruecke in der Praemisse werden
 * zeilenweise ausgegeben [1] Praemisse 1
 *                        [2] Praemisse 2  usw.                 
 *)
fun print_left (AND(a,b)) fmap bmap andlevel = 
    let
	val _= print("["^Int.toString(andlevel)^"] ")
	val _= print (PrettyPrint.getPPString a (fmap,bmap))
	val _= print("\n")
    in print_left b fmap bmap (andlevel + 1)
    end
  | print_left t fmap bmap 1 = 
    let
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end 
  | print_left t fmap bmap andlevel = 
    let
	val _= print("["^Int.toString(andlevel)^"] ")
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end

(* fun print_right t fmap bmap = print (PrettyPrint.getPPString t (fmap,bmap)) *)
fun print_right (OR(a,b)) fmap bmap orlevel = 
    let
	val _= print("["^Int.toString(orlevel)^"] ")
	val _= print (PrettyPrint.getPPString a (fmap,bmap))
	val _= print("\n")
    in print_right b fmap bmap (orlevel + 1)
    end
  | print_right t fmap bmap 1 = 
    let
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end 
  | print_right t fmap bmap orlevel = 
    let
	val _= print("["^Int.toString(orlevel)^"] ")
        val _ = print (PrettyPrint.getPPString t (fmap,bmap)) 
	val _= print("\n")
    in ()
    end

fun print_right_equality t fmap bmap _ = 
    let
	val (s, labels) = (PrettyPrint.getEqualStrings t (fmap,bmap))
	val _ = print (s ^ "\n")
	val _ = if (get_showlabels()) then print (labels ^ "\n") else ()
    in () end

(* goals haben die Form   A => B 
 * Die Praemisse wird mittels Funktion print_left ausgegeben 
 * Die Konklusion mittels Funktion print_right 
 * Praemissen werden nur fuer das erste Goal ausgegeben,
 * der Uebersicht halber!!
 *)
fun startexp (IMP (C "TRUE", b)) fmap bmap 1 = 
    print_right_equality b fmap bmap 1 (* orlevel *)

  | startexp (IMP (a,b) ) fmap bmap 1 = 
    let
	val _ = print_left a fmap bmap 1  (* andlevel *)
	val _ = print("\n==>\n") 
	val _ = print_right b fmap bmap 1 (* orlevel *)
    in () 
    end
(* Falls nicht erstes Goal, printe nur Conclusion *)
  | startexp (IMP (a,b))  fmap bmap _ = print_right b fmap bmap 1
  | startexp t fmap bmap _ = print_right t fmap bmap 1


(* Hauptroutine: Jedes goal innerhalb der Liste aller goals, wird gedruckt *)
fun print_goal' x fmap bmap (l, max) = 
    let 
	val _ = if (l=1) then print ("> ") else ()
	val _ = ( print ("Goal " ^ Int.toString(l) ^ "/" 
			 ^ Int.toString(max) ^ ": \n"))
    in 
	(startexp x fmap bmap l; print ("\n"))
    end





(* --------------------------------------------------------- *)
(* |                  LATEX AUSGABEN                       | *)
(* --------------------------------------------------------- *)


val initvmap = fn x:int => raise Error (" INIT VMAP ERROR !! ")
fun cons(f,s) = fn x:int => if x = 0 then s else f(x-1)

datatype latexout = C of char | E of int

fun gettokens s = 
    let 
	val chs = explode s 
    
	val (_,newchs) = foldl ( fn (c ,( n, liste)) => 
	      ( case c of
		   (#" ") => ( n+1, liste )
                  | z       => if (n > 0 ) then ( 0, ((C z)::(E n)::liste) ) else (0, (C z)::liste ) 
		       )
		       ) (0, nil) chs
    in
	newchs
    end


fun filter ("plus") = "+"
  | filter ("times") ="*"
  | filter ("minus") = "-"
  | filter ("less") = "$<$"
  | filter ("lesseq") = "$\\leq$"
  | filter ("greater") = "$>$"
  | filter ("greatereq") = "$\\geq$"
  | filter s = 
    let 
	val slist = String.fields (fn c => c = (#"_") ) (s)
    in
	if ( (List.length(slist)) > 1 ) 
	    then
		foldl ( fn (s,result) => result ^ "\\_" ^ s ) (hd(slist)) (tl(slist))
	else
	    (hd(slist))
    end  


fun comm2string (ELIMCMD _) = "Eliminierung"
  | comm2string (REMOVECMD (d,i)) = (case d of LEFT => "Left " | RIGHT => "Right ")^ "Remove " ^ Int.toString(i)
  | comm2string (LAMCMD) = "$\\beta / \\eta$ Reduktion"
  | comm2string (MOVECMD x) = "Bool (Move) " ^ ( case x of RIGHT => "rechts" | LEFT => "links" )
  | comm2string (SHIFTCMD (x, y, anz )) = "Bool (Shift) " ^
    (case x of LEFT => "Left " | RIGHT => "Right ") 
   ^ Int.toString(anz) ^ ( case y of UP => " up" | DOWN => " down" )
  | comm2string (GOALSHIFTCMD i) = "Subgoalshifting "^Int.toString(i) ^ " positions"
  | comm2string (INTROCMD _) = "Introduction"
  | comm2string (APPLYCMD x) = 
	( case x of
	     DEFINITION(s) => "Apply "^filter(s)
	   | MODUSPONENS(i,j) => ( "Apply " 
				  ^ Int.toString(i) 
				  ^ " to " ^ Int.toString(j) ))
		 
(*  | comm2string (SUBSTEQ (_,(mode,d)) ) = "Rewitution mit " ^ 
	      ( case mode of
		    LAXIOM(name) => name 
		  | PREMISSE(nr) => "Praemisse (" ^ Int.toString(nr) ^ ") " )
		    ^ (case d of LEFT => "Left " | RIGHT => "Right ") *)

  | comm2string (CUTCMD _) = "Cut Rule"
  | comm2string (INSTANCECMD (i,_)) ="Instanziiere $\\Gamma$ "^Int.toString(i)
  | comm2string (SIMPLIFYCMD) = "Bool"
(* Der Vollstaendigkeit halber *)
  | comm2string (HELPCMD) = "Hilfe"
  | comm2string (PRINTCMD) = "Print"
  | comm2string (SHOWLABELSCMD) = "Show Labels"
  | comm2string (LATEXCMD s) = "Latex " ^ s 
  | comm2string (UNDOCMD i) = "Undo" ^ Int.toString(i)
  | comm2string (EXITCMD) = "Exit"
  | comm2string (RESETCMD) = "Reset"
  | comm2string (POSCMD _) = "Positionsabfrage"
 


fun commeq2string (REFEQ) = "=Reflex"
  | commeq2string (SYMEQ _) = "=Symmetrie"
  | commeq2string (TRANSEQ _) = "=Trans"
  | commeq2string (INSERTEQ (_,(name,_))) = "Einsetzung Var "^name
  | commeq2string (REWEQ (_,(mode)) ) = "Ersetzung durch " ^ 
	      ( case mode of
		    (LAXIOM(name,_)) => name 
		  | (EQUALITY (_)) => "neues Lemma"
		  | (PREMISSE(nr,d)) => "Praemisse (" ^ Int.toString(nr) ^ ") " 
		    ^ (case d of LEFT => "links " | RIGHT => "rechts ") 
		  | (BETA) => "Beta-Reduktion"
		  | (ETA) => "Eta-Reduktion" )


(* Typausgabe - Funktionen *)
fun ty2latex (TA (a,b)) = ( atty2latex a ) ^ " $\\rightarrow$ " ^ (ty2latex b)
  | ty2latex t          = atty2latex t
and atty2latex (TC ("bool")) = " $\\mathbb{B}$ "
  | atty2latex (TC (a)) = a
  | atty2latex t        = "( " ^ ty2latex t ^ " )"


fun concat (s::ss) = (concat ss) ^ s
  | concat (nil)   = ""

fun genspace i = if (i = 0) then "" else (" ") ^ (genspace (i-1))

(* Achtung Input Liste ist reversiert !!! *)
fun LatexFilter' ( (C #"/")::(C #"\\")::strings) result = LatexFilter' strings (("$\\forall$")::result)
  | LatexFilter' ( (C #"\\")::(C #"/")::strings) result = LatexFilter' strings (("$\\exists$")::result)
  | LatexFilter' ( (C #"\\")::strings) result = LatexFilter' (strings) (("$\\lambda$")::result) 
  | LatexFilter' ( (C #">")::(C #"=")::strings) result = LatexFilter' strings (("$\\Rightarrow$")::result)
(*  | LatexFilter' ( (C #"=")::strings) result = LatexFilter' strings (("$\\doteq$")::result) *)
  | LatexFilter' ( (C #"+")::strings) result = LatexFilter' strings (("$\\lor$")::result)
  | LatexFilter' ( (C #"&")::strings) result = LatexFilter' strings (("$\\land$")::result)
  | LatexFilter' ( (C #"~")::strings) result = LatexFilter' strings (("$\\lnot$")::result)
  | LatexFilter' ( (E i)::(C #"\n")::strings)  result = 
    let 
	val command = "\\hspace*{" ^ Int.toString(i) ^ "mm} "
    in 
	LatexFilter' strings ( ("\\\\\n")::command::result)
    end
  | LatexFilter' ( (E i)::strings) result = LatexFilter' strings ((genspace i)::result)
  | LatexFilter' ( (C #"\n")::strings) result = LatexFilter' strings (("\\\\\n")::result)
  | LatexFilter' ( (C #"E")::(C #"U")::(C #"R")::(C #"T")::strings) result = LatexFilter' strings (("$\\top$")::result)
  | LatexFilter' ( (C #"E")::(C #"S")::(C #"L")::(C #"A")::(C #"F")::strings) result = 
      LatexFilter' strings (("$\\bot$")::result)
  | LatexFilter' ( (C #"?")::strings) result = LatexFilter' strings (("$\\Gamma$")::result)
  (* replaces commando char _  of latex by \_ *)
  | LatexFilter' ( (C #"_")::strings) result = LatexFilter' strings (("\\_")::result )
  | LatexFilter' ( (C #"s")::(C #"u")::(C #"l")::(C #"p")::strings) result  = 
      LatexFilter' strings (("+")::result)
  | LatexFilter' ( (C #"s")::(C #"u")::(C #"n")::(C #"i")::(C #"m")::strings) result  = 
      LatexFilter' strings (("-")::result)
  | LatexFilter' ( (C #"s")::(C #"e")::(C #"m")::(C #"i")::(C #"t")::strings) result  = 
      LatexFilter' strings (("*")::result)

  | LatexFilter' ( (C #"s")::(C #"s")::(C #"e")::(C #"l")::strings) result  = 
      LatexFilter' strings (("$<$")::result)
  | LatexFilter' ( (C #"r")::(C #"e")::(C #"t")::(C #"a")::(C #"e")::(C #"r")::(C #"g")::strings) result  = 
      LatexFilter' strings (("$>$")::result)
  | LatexFilter' ( (C #"q")::(C #"e")::(C #"r")::(C #"e")::(C #"t")::(C #"a")::(C #"e")::(C #"r")::(C #"g")::strings) 
      result  =  LatexFilter' strings (("$\\geq$")::result)
  | LatexFilter' ( (C #"q")::(C #"e")::(C #"s")::(C #"s")::(C #"e")::(C #"l")::strings) result  = 
      LatexFilter' strings (("$\\leq$")::result)
  
  | LatexFilter' ( (C c)::strings) result = LatexFilter' strings ( (Char.toString(c))::result )
  | LatexFilter' nil result = rev result 

fun LatexFilter s = 
    let
	val tokens = gettokens s
	val subst_tokens = LatexFilter' tokens nil
    in
	concat subst_tokens
    end



(*     **************************************************************        *)


fun lprint_left (AND(a,b)) fmap bmap andlevel = 
    let
	val res' = "("^Int.toString(andlevel)^") "
	val res  = res' ^  LatexFilter(PrettyPrint.getPPString a (fmap,bmap)) ^ " \\\\\n"
    in res ^ (lprint_left b fmap bmap (andlevel + 1))
    end

  | lprint_left t fmap bmap andlevel = 
    let
	val res' =  if (andlevel = 1 ) then "\\hspace*{5mm} " else "("^Int.toString(andlevel)^") "
	val res  = res' ^ LatexFilter(PrettyPrint.getPPString t (fmap,bmap)) ^ "\\\\\n"
    in res
    end

(* fun lprint_right t fmap bmap  = LatexFilter(PrettyPrint.getPPString t (fmap,bmap)) *)
fun lprint_right (OR(a,b)) fmap bmap orlevel = 
    let
	val res' = "("^Int.toString(orlevel)^") "
	val res  = res' ^  LatexFilter(PrettyPrint.getPPString a (fmap,bmap)) ^ " \\\\\n"
    in res ^ (lprint_right b fmap bmap (orlevel + 1))
    end

  | lprint_right t fmap bmap orlevel = 
    let
	val res' =  if (orlevel = 1 ) then "\\hspace*{5mm} " else "("^Int.toString(orlevel)^") "
	val res  = res' ^ LatexFilter(PrettyPrint.getPPString t (fmap,bmap)) ^ "\\\\\n"
    in res
    end

(* goals haben die Form   A => B 
 * Die Praemisse wird mittels Funktion print_left ausgegeben 
 * Die Konklusion mittels Funktion print_right 
 *)
fun lstartexp (IMP (a,b) ) fmap bmap= 
    let
	val res'' = lprint_left a fmap bmap 1
	val res'  = res'' ^ ("\\textbf{$\\Rightarrow$\\\\}\n")
	val res   = res' ^ lprint_right b fmap bmap 1 ^ "\\\\\n"
    in res
    end
  |lstartexp t fmap bmap = lprint_right t fmap bmap 1

fun latex_goals (x, fmap, bmap ) = lstartexp x fmap bmap 
fun print_goal (x, fmap, bmap ) (n, max) = print_goal' x fmap bmap (n,max) 

end (* of local open *)				    
end (* of structure Visual.sml *)
(* ==========================================================================================
                             Typecheck.sml
   ========================================================================================== *)

structure Typecheck= 
struct
(* Typechecker : pter -> (int -> ty) -> (string -> bool) ->  bool *)
(*               term     freevar map      typ map           result  *)

local open Data in

exception illtyped of string
fun cons t (g : var -> ty) = fn x => if (x=0) then t else g (x-1)



(* [checktypetree]
 * checktypetree : ty -> ( string -> bool ) -> bool
 * checkt, ob alle Blaetter im Typbaum ty bekannt sind
 *)
fun checktypetree (TA(a,b)) map = (checktypetree a map) andalso (checktypetree b map)
  | checktypetree (TC t) map = map t

(* basetype_test : (string->bool) -> ( ty -> bool )  
 * gibt ty -> bool funktion zurueck, die checkt, ob alle
 * Blaetter Typen in ty bekannt sind!
 *)
val basetype_test  = fn (map: string -> bool) => fn (t:ty) => checktypetree t map

(* Gibt die Ordnung eines Typs an *)
fun type_ordnung (TC _ )    =  0
  | type_ordnung (TA (a,b)) =  Int.max(1 + (type_ordnung a), type_ordnung b)


fun tycheck' (C c) tyc _ _ = tyc c
  | tycheck' (V x) _ ga _  = ga x
  | tycheck' (GAMMA (t,_)) _ _ _ = t
  | tycheck' (A(m,m')) tyc ga ta = 
    ( case (tycheck' m tyc ga ta) of
	  (TA (t1, t2)) => if (t1 = (tycheck' m' tyc ga ta) ) then t2 
			   else raise illtyped ("Ill typed Application")
	| _             => raise illtyped ("Ill typed Application")
	      )

  | tycheck' (L((t,_),e)) tyc ga ta = 
	  if ( ta t ) then TA(t, tycheck' e tyc (cons t ga) ta)
	  else raise illtyped ("Ill typed Lambda construct")

  | tycheck' (EX((t,_),e)) tyc ga ta = 
	      if ( ta t ) then 
		  ( case tycheck' e tyc (cons t ga) ta of
			(t' as (TC ("bool"))) => t'
		      | _ => raise illtyped "Ill typed Existence Quantifier" )
	      else raise illtyped ("Ill typed - Unknown base types in Ex. Quantifier")
				  
  | tycheck' (ALL((t,_),e)) tyc ga ta = 
		  if (ta t) then 
		      ( case tycheck' e tyc (cons t ga) ta of
			    (t' as (TC ("bool"))) => t'
			  | _ => raise illtyped "Ill typed All Quantifier" )
		  else raise illtyped ("Ill typed - Unknown base types in All Quantifier") 
				  
  | tycheck' (AND(m,m')) tyc ga ta = 
		      (case ( tycheck' m tyc ga ta , tycheck' m' tyc ga ta ) of
			   (TC "bool" , TC "bool" ) => TC "bool"
			 | _ => raise illtyped ("Ill typed Conjunction") )

  | tycheck' (OR(m,m')) tyc ga ta = 
			   (case ( tycheck' m tyc ga ta , tycheck' m' tyc ga ta ) of
				(TC "bool" , TC "bool" ) => TC "bool"
			      | _ => raise illtyped ("Ill typed Disjunction" ) )

  | tycheck' (IMP(m,m')) tyc ga ta =
				(case ( tycheck' m tyc ga ta , tycheck' m' tyc ga ta ) of
				     (TC "bool" , TC "bool" ) => TC "bool"
				   | _ => raise illtyped ("Ill typed Implication") )

  | tycheck' (EQ(m,m')) tyc ga ta = 
	   let val t1 = tycheck' m tyc ga ta
	       val t2 = tycheck' m' tyc ga ta
	   in if ( t1=t2 ) 
		  then TC "bool"
	      else raise illtyped ("Ill Typed Equation")
	   end

  | tycheck' (NOT(m)) tyc ga ta = 
	   ( case ( tycheck' m tyc ga ta) of
		 (t' as (TC "bool"))  => t'
	       | _ => raise illtyped ("Ill typed NOT Application") )
		 
fun tycheck pt (tyc, ga, ta) = (tycheck' pt tyc ga (basetype_test ta) ; true ) handle illtyped _ => false

fun get_type pt (tyc, ga, ta) = tycheck' pt tyc ga (basetype_test ta) (* returns the type of an pter *)

end (* of local open *)
end (* of structure Typecheck.sml *)
(* ==========================================================================================
                             Env.sml
   ========================================================================================== *)

(* Struktur Env.sml implementiert Signatur Env.sig *)
(* Erklaerung siehe Env.sig                        *)

structure Env =
struct 

    open Data

    (* Datenstruktur env ist ein record bestehend aus 6 Komponenten *)
    type env = { tc : string -> ty ,                 (*  Const-Typumgebung      *)
                 tf : string -> ty ,                 (*  Var - Typumgebung      *)
		 clist : string list ,               (*  Const List             *)
		 flist : string list,                (*  Free Var  - Int List   *)
		 blist : string list,                (*  Bound Var - Int List   *) 
		 tlist : string list }               (*  Type List              *)
	    

(* --------------------------------------------------------------------*)
(* |                   BEGIN Hilfsprozeduren                          |*)
(* --------------------------------------------------------------------*)


    (* [debug_print]
     * DEBUGGING
     *)
    fun debug_print xs =  ("\n[ " ^ (foldl (fn (s,aus) => (aus ^ ", " ^ s) ) "" xs ) ^ " ]\n")

    
    (* [mapstring2bool]
     * generiert aus Typenliste eine Funktion
     * map : string -> bool mit
     * map[typ] = typ ist in typliste
     *)
    fun mapstring2bool (xs:string list) =
	let
	    val init = (fn s:string => false)
	    val map = foldl (fn (x,f) =>( fn s:string => if (s=x) then true else f s ) ) init xs
	in 
	    map
	end


    (* [mapstring2int]
     * generiert aus einer string list xs eine Funktion map
     * mit map[s] = position von s in liste xs
     *)
    fun mapstring2int (xs:string list) = 
	let
	    val (_,map) = 
		(foldl (fn (x,(n,f)) => 
			if (( f x ; false ) handle Error _ => true )
			    then (n+1, fn v => if v = x then n else f v) 
			else (n,f) ) (0,( fn v => raise Error ("Variable " ^ v 
							       ^ " muss zunaechst" 
							       ^ " deklariert werden!\n") 
					 )
				      ) xs 
		 )
	in
	    map
	end

    (* [mapint2string]
     * generiert aus einer string list xs eine Funktion map
     * mit map[i] = string an position i in liste xs
     *)
    fun mapint2string (xs:string list) = 
	(fn i => List.nth((xs,i)) handle Subscript => "" )


  
   (* [concat]
    * konkateniert zwei Stringlisten, allerdings duerfen positionen der alten liste
    * nicht veraendert werden und doppelauftreten nicht vorkommen
    *)
(*
    fun concat (xs:string list) nil = xs
      | concat nil ys = ys
      | concat xs (y::ys) = (concat xs ys)@( if (List.exists( fn s => s = y) xs) then nil else [y]  )
*)

    (* [next]
     * gibt zu einem String den numerisch folgenden Nachfolger
     * Bsp : x --> x0  oder  x2 --> x3 usw.
     *)
    fun digit_pos xs = foldl ( fn (c,n) => if Char.isDigit(c) then n else n+1 ) 0 xs 
    fun next s = 
	let
	    val ss = explode(s)
	    val pos = digit_pos ss
	    val prefix = String.substring(s, 0, pos)
	    val postfix = String.substring(s,pos, String.size(s) - pos)
	    val newpostfix = case Int.fromString(postfix) of
		               NONE   => "0"
			     | SOME i => Int.toString (i+1)
	in
	    prefix^newpostfix
	end


    (* [rename]
     * nimmt newxs neue string list
     * und (cs,ys,zs) alte string listen
     * prueft fuer jedes Element x aus newxs, ob
     * x in cs, ys oder zs vorkommt
     *    wenn ja   => wird x mihilfe von next umbenannt und erneut ueberprueft
     *    wenn nein => wird x uebernommen
     * Bei jedem Renaming von x muss ebenfalls ueberprueft werden, ob
     * next(x) nicht bereits Element der ueberarbeiteten Liste ist
     *)
(*  
  fun rename' nil    (cs, ys, zs)   resxs = resxs
      | rename'(x::xs) (cs, ys, zs)   resxs = if ( (List.exists( fn s => s = x ) ys) orelse 
						   (List.exists( fn s => s = x ) zs) orelse
						   (List.exists( fn s => s = x ) cs) orelse
						   (List.exists( fn s => s = x ) resxs )    ) 
					     then rename' (next(x)::xs) (cs, ys, zs) resxs
					     else rename' xs (cs, ys, zs)  (resxs @ [x])
    fun rename newxs (cs, ys, zs) = rename' newxs (cs, ys, zs) nil
*)

    fun extend bmap nil = bmap
      | extend bmap (x::xs) = if (List.exists ( fn s => s = x ) bmap) 
				  then extend bmap xs 
			      else extend (bmap @ [x]) xs 


(* --------------------------------------------------------------------*)
(* |                   ENDE Hilfsprozeduren                           |*)
(* --------------------------------------------------------------------*)


(* --------------------------------------------------------------------*)
(* |                   BEGIN Hauptmethoden                            |*)
(* --------------------------------------------------------------------*)


    (* [empty]
     * legt initiale Variablen/Konstanten Typumgebung,
     * leere Free/Bound Variablen Listen und 
     * initiale Typliste ["bool"] an 
     *)
    fun empty() = 
	let 
	    val init_typeenv_consts : string -> ty  = 
		fn x:string => if (x="TRUE") orelse (x="FALSE") 
				   then (TC "bool")
			       else raise Error ("Constante " ^ x 
						 ^ " muss zunaechst deklariert werden!\n")
	    val init_typeenv_freevars : string -> ty  = 
		fn x:string => raise Error ("Variable " ^ x 
					    ^ " muss zunaechst deklariert werden!\n")
	    val init_types   = ["bool"]        
	in 
	    { tc = init_typeenv_consts ,
	      tf = init_typeenv_freevars , 
	      clist = ["TRUE","FALSE"] ,                 
	      flist = nil ,
	      blist = nil ,
	      tlist = init_types }
	end
    
	      

    (* [new_type]
     * Erweitert die Typliste um den Typ <ntype> 
     *)
     fun new_type( oldenv:env, ntype:string) = 
	 let 
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv
	 in
	     (* Doppelauftreten werden vermieden *)
	     if (List.exists (fn s => s=ntype) typelist) 
		 then 
		     oldenv
	         else
		  { tc = ctypeenv, 
		    tf = ftypeenv,
		    clist = cmap,
		    flist = fmap,
		    blist = bmap,
		    tlist = (ntype::typelist) }
	 end
     
    
     (* [update_ctypeenv]
      * einer neuen Constanten <c> wird ein Typ t zugeordnet
      *)
     fun update_ctypeenv(oldenv:env,c:string,t:ty) = 
	 let
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv

	     (* prueft ob alle Basistypen in t ueberhaupt deklariert wurde *)
	     val typmap = mapstring2bool typelist
	     val _ = if (Typecheck.checktypetree t typmap) then () 
		     else raise Error ("Some Base Types in " ^ c ^ " are unknown \n")

	     val known =    (  (ctypeenv c; true ) handle Error _ => false   ) orelse
		            (  (ftypeenv c; true ) handle Error _ => false   )

	     val newctypeenv = (fn s => if s=c then t else ctypeenv s)
	 in
	     (* known=true  <=> Konstante c bereits deklariert wurde *)
	     if (known) then 
		 raise Error ("Konstante/Variable " ^c^ " wurde bereits deklariert\n")
	     else { tc = newctypeenv, 
		    tf = ftypeenv,
		    clist = cmap,
		    flist = fmap,
		    blist = bmap,
		    tlist = typelist }
	 end

      (* [update_ftypeenv]
      * einer neuen Variablen <v> wird ein Typ t zugeordnet
      *)
     fun update_ftypeenv(oldenv:env,v:string,t:ty) = 
	 let
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv

	     (* prueft ob alle Basistypen in t ueberhaupt deklariert wurde *)
	     val typmap = mapstring2bool typelist
	     val _ = if (Typecheck.checktypetree t typmap) then () 
		     else raise Error ("Some Base Types in " ^ v ^ " are unknown \n")

	     val known =    (  (ftypeenv v; true ) handle Error _ => false   ) orelse
		            (  (ctypeenv v; true ) handle Error _ => false   )
	     val newftypeenv = (fn s => if s=v then t else ftypeenv s)
	 in
	     (* known=true  <=> Konstante v bzw. Freie Variable v bereits deklariert wurde *)
	     if (known) then 
		 raise Error ("Variable/Konstante " ^v^ " wurde bereits deklariert\n")
	     else { tc = ctypeenv, 
		    tf = newftypeenv,
		    clist = cmap,
		    flist = fmap,
		    blist = bmap,
		    tlist = typelist }
	 end
 


     (* [update_constmaplist]
      * der Constmapliste werden die Konstanten der Liste ms hinzugefuegt 
      *)
     fun update_constmaplist(oldenv:env, ms:string list) = 
	 let 
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv
	 in
	     { tc = ctypeenv, 
	       tf = ftypeenv,
	       clist = extend cmap ms,
	       flist = fmap,
	       blist = bmap,
	       tlist = typelist }
	 end


     (* [update_freemaplist]
      * der Freemapliste werden die Variablen der Liste ms hinzugefuegt 
      *)
     fun update_freemaplist(oldenv:env, ms:string list) = 
	 let 
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv
	 in
	     { tc = ctypeenv, 
	       tf = ftypeenv,
	       clist = cmap,
	       flist = extend fmap ms,
	       blist = bmap,
	       tlist = typelist }
	 end
		
  
     (* [update_bndmaplist]
      * da neue gebundene Variablen hinzugefuegt werden,
      * kann es zu Konflikten mit bereits vorhandenen
      * Konstanten, freien Variablen oder gebundenen Variablen kommen
      * => Umbenennung der alten gebundenen Variablen 
      *)
     fun update_bndmaplist(oldenv:env, ms:string list) =
	 let 
	    (* DEBUG val _ = foldl (fn (s,_) => print (s ^ "\n")) () ms *) 
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv

	 (*    val newbmap = rename bmap ( cmap, fmap, ms ) *)
 	 (* !!! Die alten Geb. Var. werden veraendert, nicht die neuen !!! *)
	     val newbmap = extend bmap ms
	 in
	     { tc = ctypeenv, 
	       tf = ftypeenv,
	       clist = cmap,
	       flist = fmap,
	     (*  blist = newbmap @ ms , *)
	       blist = newbmap,
	       tlist = typelist }
	 end

(* [renameBndVars] 
 * nimmt Liste von Gebundenen Variablen ( Integers ! )
 * transformiert diese in eine Stringliste und 
 * nennt die Schnittmenge dieser Liste mit der bmap liste
 * um !
 *)  

     fun renameBndVars' nil f = f
       | renameBndVars' (b::bs) f = renameBndVars' bs ( fn s=> if s=b then next b else f s )
     fun renameBndVars(oldenv:env, is: int list) =
	 let
	      val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv

	      val bndmapi2s = mapint2string(bmap)
	      val slist = List.map bndmapi2s is
	      val f = renameBndVars' slist (fn s:string => s)
	      val newbmap = List.map f bmap
	 in
	      { tc = ctypeenv, 
	       tf = ftypeenv,
	       clist = cmap,
	       flist = fmap,
	       blist = newbmap,
	       tlist = typelist }
	 end

(* [freeVarInt]
 * searches the next free name, that isn't used yet in freemap or constmap
 * is called by Intro All /Elim Exists
 *) 


     fun freeVarIntro (oldenv:env, i:int) = 
    	 let
	      val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv

	      val bndmapi2s = mapint2string(bmap)
	      val oldname = bndmapi2s i 
		  
	      fun search_newname s = 
		  if (List.exists (fn z=>z=s) cmap) orelse
		     (List.exists (fn z=>z=s) fmap) 
 		      then search_newname (next s) else s
	 in
	     search_newname oldname
	 end

(* --------------------------------------------------------------------*)
(* |                   ENDE Hauptmethoden                             |*)
(* --------------------------------------------------------------------*)


(* --------------------------------------------------------------------*)
(* |                   BEGIN Instanziierungsmethoden                  |*)
(* --------------------------------------------------------------------*)
  (* [assert_free_bnd]
      * Stellt sicher, dass bndmap und (freemap, constmap) invariant disjunkt sind 
      *)
 (*    fun assert_free_bnd(oldenv:env) = 
	 let
	     val { tc = ctypeenv, 
		   tf = ftypeenv,
		   clist = cmap,
		   flist = fmap,
		   blist = bmap,
		   tlist = typelist } = oldenv

	     val newbmap = rename bmap (cmap, fmap, nil)
	 in 
	     { tc = ctypeenv, 
	       tf = ftypeenv,
	       clist = cmap,
	       flist = fmap,
	       blist = newbmap,
	       tlist = typelist }
	 end
*)
 

(*
    
     (* [gen_instance_env] 
      * gebe neue Env zurueck  ( Im Falle einer Instanziierung )
      * mit:
      * - BndVarMap der aktuellen/oldenv Umgebung
      * - ConstMap, FreeVarMap, Typelist, Typmap der gammaenv
      *)
      fun gen_instance_env( oldenv, gammaenv) = 
	 let
	     val oldbmap = #blist oldenv

	     val { tc = gammactypeenv, 
		   tf = gammaftypeenv,
		   clist = gammacmap,
		   flist = gammafmap,
		   blist = _ ,
		   tlist = gammatypelist } = gammaenv
	 in
	     { tc = gammactypeenv, 
	       tf = gammaftypeenv,
	       clist = gammacmap,
	       flist = gammafmap,
	       blist = oldbmap,
	       tlist = gammatypelist }
	 end
 *)    

     (* [back_instance_env]
      * Die aktuelle Environment wird zurueckkopiert und dabei nur die neue Bnd Map Liste
      * uebernommen, allerdings muss nun sichergestellt werden, dass in der Instanziierung
      * keine gebundenen Variablen verwendet wurden, die in der aktuellen Umgebung erst 
      * frei geworden sind. Deshalb der assert_free check !
      *)
      fun back_instance_env( oldenv:env, instenv:env ) = 
	  let
	      val { tc = ctypeenv, 
 		    tf = ftypeenv,
		    clist = cmap,
		    flist = fmap,
		    blist = bmap,
		    tlist = typelist } = oldenv

	      val instbmap = #blist instenv
	  
	      val newenv = { tc = ctypeenv, 
			     tf = ftypeenv,
			     clist = cmap,
			     flist = fmap,
			     blist = instbmap,
			     tlist = typelist }
	  in
	    (*  assert_free_bnd(newenv) *)
	      newenv
	  end
      
      
(* --------------------------------------------------------------------*)
(* |                   ENDE Instanziierungsmethoden                   |*)
(* --------------------------------------------------------------------*)



(* --------------------------------------------------------------------*)
(* |             BEGIN MAPPING Generatoren Methoden                   |*)
(* --------------------------------------------------------------------*)
  
  
  (* Alle folgenden Methoden generieren aus den entsprechenden
   * Listen Mapping Funktionen
   *)
      fun freemaps2i ( oldenv:env ) = let
				      val fmap = #flist oldenv
				  in mapstring2int fmap
				  end
			      
      fun freemapi2s ( oldenv:env ) = let
				      val fmap  = #flist oldenv
				  in mapint2string fmap
				  end
	      
      fun bndmaps2i ( oldenv:env ) = let
				     val bmap = #blist oldenv
				 in mapstring2int bmap
				 end
      fun bndmapi2s ( oldenv:env ) = let
				     val bmap = #blist oldenv
				 in mapint2string bmap
				 end
			     

      fun constmapi2s (oldenv:env) = let
					 val cmap = #clist oldenv
				     in mapint2string cmap
				     end

      fun constmaps2i (oldenv : env) = let
					   val cmap = #clist oldenv
				       in mapstring2int cmap
				       end

      fun typmaps2b (oldenv:env) = let
				   val typelist = #tlist oldenv
			       in mapstring2bool typelist
			       end
			
      (* [typi]
       * gibt den Typ der Variable mit der Nummer i zurueck
       *)
      fun typi( oldenv:env, x:int) = let 
				     val ftypeenv = #tf oldenv
				     val fmap = #flist oldenv
				     val variable = (mapint2string fmap) x 
				 in 
				     ftypeenv variable
				 end

      (* [typc]
       * gibt den Typ der Konstante c zurueck
       *)		     
      fun typc( oldenv:env, c:string) = let 
					val ctypeenv = #tc oldenv
				    in 
					ctypeenv c
				    end
	
      (* [typf]
       * gibt den Typ der Variable v zurueck
       *)		     
      fun typf( oldenv:env, v:string) = let 
					val ftypeenv = #tf oldenv
				    in 
					ftypeenv v 
				    end
				
(* --------------------------------------------------------------------*)
(* |                   ENDE MAPPING Generatoren Methoden              |*)
(* --------------------------------------------------------------------*)

(* --------------------------------------------------------------------*)
(* |                   Verschiedene Methoden                          |*)
(* --------------------------------------------------------------------*)

      (* [nr_consts]
       * gibt die anzahl der deklarierten Konstanten zurueck 
       *)
      fun nr_consts (oldenv:env) = let
				   val cmap = #clist oldenv
			       in List.length(cmap)
			       end
			   
      (* [nr_freevars]
       * gibt die anzahl der deklarierten freien Variablen zurueck 
       *)
      fun nr_freevars (oldenv:env) = let
				     val fmap = #flist oldenv
				 in List.length(fmap)
				 end

      (* [nr_bndvars]
       * gibt die Anzahl der bekannten gebundenen Variablen zurueck
       *)
      fun nr_bndvars (oldenv:env) = let
				    val bmap = #blist oldenv
				in List.length(bmap)
				end
			  
			  
end (* of struct *)



(* ==========================================================================================
                             Definition.sml
   ========================================================================================== *)

structure Definition =
struct 
    open Data
    type definition =   (( string, (Data.pter * Data.kind_of_def) ) Polyhash.hash_table )
	              * (string -> Env.env ref)
	
	(* Use the Polyhashtable class of mosml library, to store 
	 * pairs of ( Terms * (CLASSIC|LEMMA|AXIOM) ) by a key  
	 * [ a classic definition is a term which can be subtitute in a goal, but not more ]
	 * [ a definition of lemma is a definition, which can be added to the Premessis, but it
	 *   has to be prooven after goal !! ]
	 * [ an definition of an axiom can be added to the premessis all the time ]
	 *
	 * Save in case of an lemma, which has to be prooven later on, what Environment
	 * is valid for the lemma ( with Refernce of the Environement )
	 *)

    val standart_size = 20   (* Standart Size of the hash table *)
    exception no_entry


    fun empty() = 
	let
	    val hashtable = Polyhash.mkPolyTable( standart_size, no_entry )
	    val key2env = fn s => raise Error ("No such a lemma <" ^ s ^ "> exists!\n")
	in
	    (hashtable, key2env)
	end

    
    (* Inserts a new definition into Hashtable and
     * adds a new env ref, if mode = LEMMA 
     *)
    fun insert_def def ( key, p, LEMMA) envref =
	let 
	    val (hashtable, key2env) = def
	    val _ = Polyhash.insert hashtable (key,(p, LEMMA) )
	in 
	   (hashtable, fn s => if s=key then envref else key2env s )
	end
    
      | insert_def def ( key, p, mode) _ = 
	let 
	    val (hashtable, _ ) = def
	    val _ = Polyhash.insert hashtable (key, (p,mode) )
	in 
	   def
	end
    

    

    (* returns the term of a definition *)
    fun get_term def key = 
 	let 
	    val (hashtable, _) = def
	    val (result, _ ) = 
		Polyhash.find hashtable key handle no_entry => 
		    raise Error("There is no such definition for <"^key^">")
	in
	    result
	end

    (* returns CLASSIC | LEMMA | AXIOM *)
    fun get_kind def key =
	let 
	    val (hashtable, _ ) = def
	    val ( _, kind) = 
		Polyhash.find hashtable key handle no_entry => 
		    raise Error("There is no such definition for <"^key^">")
	in
	    kind
	end

    (* changes the Kind of an definition *)
    fun change_kind def (key, kind) = 
	let
	    val (hashtable, _) = def
	    val term = get_term def key
	    val _ = Polyhash.insert hashtable (key, (term ,kind) )
	in ()
	end

	   
    (* returns the Env.env ref if saved mode of the 
     * definition is LEMMA 
     *)
    fun get_envref def key = 
	let 
	    val (hashtable, key2env) = def
	in
	   (case (get_kind def key) of
		LEMMA => (key2env key)
	      | _     => raise Error ("You can't get the environment" 
				      ^ "for a NON LEMMA Definition !\n")
	   )
	end
	   
    fun nr_def def = 
	let
	    val (hashtable,_) = def
	in Polyhash.numItems hashtable
	end


(* -------------------------------------------------------------------- *)

    (* consider only basic cases for definition translation *)
    fun check_def' (A(a,b)) mapi2s def = (check_def' a mapi2s def) orelse (check_def' b mapi2s def)
      | check_def' (V (i))  mapi2s def = ((get_term def (mapi2s i); true) handle Error _ => false )
      | check_def' (NOT(a)) mapi2s def = check_def' a mapi2s def
      | check_def' _ _ _ = false
								     
    (* don't panic about Variable (V i) and their level -> they must be at base level !! *)
    fun translate_def' (A(a,b))        mapi2s def = A((translate_def' a mapi2s def),(translate_def' b mapi2s def))
      | translate_def' (t as (V (i)))  mapi2s def = ((get_term def (mapi2s i)) handle Error _ => t )
      | translate_def' (NOT(a))        mapi2s def = NOT(translate_def' a mapi2s def)
      | translate_def'  t _ _  = t



    fun check_def def mapi2s pter = check_def' pter mapi2s def

    fun translate_def def mapi2s pter = translate_def' pter mapi2s def

end



(* ==========================================================================================
                             Gamma.sml
   ========================================================================================== *)

structure Gamma =
struct 

    open Data
    (* Ein Knoten kann (nicht markiert), entdeckt oder geschl. sein *)
    datatype modus = nonmarked | discovered | closed   

    (* Ein Knoten im Graphen ist entweder Gamma oder Delta Variable *)
    datatype nodevalue = GAMMA_VAR of int | DELTA_VAR of int
    (* State Merkfeld, Information Gamma/Delta int, Nachfolgeknoten *)

    (* Graph Datenstruktur *)
    datatype state = G of modus * nodevalue * node list       
    withtype node = state ref                           (* Knoten ist eine Zeiger auf state *)
    type graph = node list                              (* Graph ist eine Liste von Knoten *)

    (* Interne exceptions *)
    exception cycle
    exception notfound
    exception neverhappen

    type gamma = graph  (* Die Gamma/Delta Variablen werden in einem Graph
			 * verwaltet 
			 * Der Abhaengigkeitstest erfolgt durch Cyclustest!
			 *)


    (* ----------------------------------------------------------------- *)
    (* |     BEGIN        Hilfsprozeduren Graphbearbeitung             | *)
    (* ----------------------------------------------------------------- *)
	
    fun cons x gs  = ref (G(nonmarked, x ,gs))
    fun assign (g as ref(G(m,x,_))) gs = g := G(m,x,gs)
	
    fun create_edge (k1 as ref(G(m1,x1,g1))) k2 = (k1 := G(m1,x1, (k2::g1)))
    
    fun remove_edge (k1 as ref(G(m1,x1,g1))) k2 = 
	let 
	    fun filter (nil, element) result     = result
	      | filter ((x::xs), element) result = 
		if (x = element) then filter (xs, element) result
		else filter (xs, element) (x::result)
	in
	    k1 := G(m1, x1, filter (g1, k2) nil)
	end


    (* sucht im Graph den Knoten mit dem Wert x *)
    fun find_node nil x = raise notfound
      | find_node ( (g as (ref(G(_, v, _))))::gs ) x = 
	if (x=v) then g else find_node gs x
    


    fun discover (g as ref(G(_,x,cs))) = (g:=G(discovered,x,cs))
    fun close (g as ref(G(_,x,cs))) = (g:=G(closed,x,cs))
	
    fun  reset nil = ()
      |  reset ((g as (ref (G(nonmarked, x,gs))))::xr) = reset xr 
      |  reset ((g as (ref (G(_, x, gs))))::xr) = (g:=G(nonmarked, x, gs); reset (gs@xr) )
	
	
    (* Zyklustest *)
    (* DFS Algorithmus *)
	
    fun cyclic g = ( reset g ; cyclic' g )
    and cyclic' nil = false
      | cyclic' ((g as ref(G(closed,_,_)))::gs) = cyclic gs
      | cyclic' ((g as ref(G(nonmarked,x,cs)))::gs) =( ( discover g ; 
							dfs [g] ; 
							close g ;
							cyclic' gs ) handle cycle => true )
      | cyclic' _ = raise neverhappen
	
    and dfs nil = ()
      | dfs ((s as ref(G(discovered, x, nil)))::stack ) = ( close s ; dfs stack ) (* pop s and backtrack *)
      | dfs ((s as ref(G(discovered, x, childs )))::stack) = 
	( case (nextnonmarked (childs)) of
	    nil => ( close s; dfs stack )
	  | (c::nil) =>( discover c; dfs (c::s::stack) )
	  | _ => raise neverhappen
		)
      | dfs _ = raise neverhappen

    and nextnonmarked (nil) = nil 
      | nextnonmarked ( (c as ref(G(nonmarked,_,_)))::cs) = [c]
      | nextnonmarked ( (c as ref(G(discovered,_,_)))::cs) = raise cycle
      | nextnonmarked ( (c as ref(G(closed,_,_)))::cs) = nextnonmarked cs
	
	
    (* ----------------------------------------------------------------- *)
    (* |     ENDE         Hilfsprozeduren Graphbearbeitung             | *)
    (* ----------------------------------------------------------------- *)


    (* ----------------------------------------------------------------- *)
    (* |     BEGIN        Hilfsprozeduren Gamma Routinen               | *)
    (* ----------------------------------------------------------------- *)

		
	(* fuegt allen Gamma Knoten innerhalb des Graphens eine 
	 * Kante zum Delta Knoten hinzu 
	 *)

	fun delta_update graph =
	    let 
		val delta = hd(graph) (* da gerade hinzugefuegt *)
	    in
		foldl ( fn (node, _ ) => 
		       case (node) of
			   (ref (G(_, GAMMA_VAR(_),_))) => create_edge node delta
			 | _ => ()
		       )      
		        () (tl(graph))
	    end




    (* ----------------------------------------------------------------- *)
    (* |     BEGIN        Gamma Haupt Routinen                         | *)
    (* ----------------------------------------------------------------- *)

	

    (* Legt initialen Graph an : leer *)
    fun empty () = nil



    (* Fuegt Eine Gamma Varible in den Graph ein *)
    fun add_gamma (gs, i) = 
	let 
	    val new_node = cons (GAMMA_VAR i) nil
	    val new_graph = (new_node::gs)
	in
	    new_graph
	end
    
    
    (* Fuegt Eine Delta Varible in den Graph ein *)
    fun add_delta (gs, i) =
	let
	    val new_node = cons (DELTA_VAR i) nil
	    val new_graph = (new_node::gs)

	    val _ = delta_update(new_graph)
	in 
	    new_graph 
	end

    
    
    (* Prueft, ob eine Substitution zulaessig ist, sprich kein
     * Cyclus im Graph ist, nachdem alle Abhaengigkeitsedges hinzugefuegt
     * wurden
     *)
    
    fun substitute gs (i, terms ) =
	let
	    (* gamma_node ist der Knoten der Gammavariable i im Graph *)
	    val gamma_node = ( find_node gs (GAMMA_VAR i) ) 
	                      handle notfound => 
				 raise Error ("There is no Gamma Variable ?" ^ Int.toString(i) ^ "\n")

	    (* Liste mit allen Edges, die durch die Substitution bedingt
	     * dem Graphen hinzugefuegt werden
	     * Im Falle einer nicht erlaubten Substitution ( <=> Cyclus im Graph )
	     * werden all_new_edges wieder geloescht 
	     *)
	    val all_new_edges =
		foldl ( fn (t, edges) => 
		       ( case t of
			   (V i)    => ( (find_node gs (DELTA_VAR i) , gamma_node) :: edges )
			 |(GAMMA (_,i)) => ( (find_node gs (GAMMA_VAR i) , gamma_node) :: edges )
			 | t => raise Error (" Internal Error Gamma.sml; not only variables in list \n")
			       ) handle notfound => edges 
			     ) nil terms
	       

	    (* fuege nun diese edges dem Graph hinzu *)
	    val _ = List.app (fn (n1,n2) => create_edge n1 n2) all_new_edges

	    val zyklisch = cyclic gs
	in
	    (* Ist Graph nun zyklisch, loesche wieder alle Edges und gib false zurueck *)
	    if (zyklisch) 
		then ( List.app (fn (n1,n2) => remove_edge n1 n2) all_new_edges; false )
	        else true
	end
		


    (* counts the number of Gamma nodes in the graph *)
    fun nr_gammavars gs = 
	foldl ( fn (node,n) => 
	       case node of 
		   ( ref (G(_,GAMMA_VAR(_),_))) => n+1
		 | _ => n
		       ) 0 gs



		  
end (* of struct *)



(* ==========================================================================================
                             Goal.sml
   ========================================================================================== *)

(* Struktur Goal.sml implementiert Signatur Goal.sig *)
(* Erklaerung siehe Goal.sig *)

structure Goal =
struct
  open Data
  type goal =  Env.env list * pter list * Gamma.gamma * Definition.definition
  (*    goal ist ein 4-Tupel aus :
        Typumgebung * Praeterme * Gamma Variablen
        * Definition Hash Table
    
   *    [env1,env2,...] gehoert zu [pter1,pter2,...]    
   *    
   *)

(* --------------------------------------------------------------------*)
(* |                   BEGIN Hilfsprozeduren                          |*)
(* --------------------------------------------------------------------*)


  (* Signalisiert Main.sml, dass Proof Done ! *)      
  fun done_goal() =
      let
	  val done_gamma = Gamma.empty()     (* Gamma Graph inital leer *)
	  val done_def   = Definition.empty()  (* lege leere Definitionen table an *)
	  val done_pter  = (C "PROOF DONE")
      in
	 ( nil , done_pter::nil , done_gamma , done_def )  
      end
     
  (* Erste Uebersetzung der geparsten Ausdruecke in Uebergangsrepresentation 
   * sexpr -> pter' 
  
  fun sexpr2pter' (AppS(a,b))     map = A'(sexpr2pter' a map, sexpr2pter' b map)
    | sexpr2pter' (CstS(s) )      map = C'(s)
    (*  Ist eine Variable nicht in gesmaps2i ==> Konstante oder unbekannt *)
    (*  Im Falle unbekannt, merkt der Typchecker dies !! *)
    | sexpr2pter' (VarS(s) )      map = ( ( V'(map s) ) handle Error _ => (C' (s)) )
    | sexpr2pter' (LamS((s,t),e)) map = L'((map s,t), sexpr2pter' e map)
    | sexpr2pter' (ExS((s,t),e))  map = EX'((map s,t), sexpr2pter' e map)
    | sexpr2pter' (AllS((s,t),e)) map = ALL'((map s,t), sexpr2pter' e map)
    | sexpr2pter' (AndS(a,b))     map = AND'(sexpr2pter' a map, sexpr2pter' b map)
    | sexpr2pter' (OrS(a,b))      map = OR'(sexpr2pter' a map, sexpr2pter' b map)
    | sexpr2pter' (ImpS(a,b))     map = IMP'(sexpr2pter' a map, sexpr2pter' b map)
    | sexpr2pter' (EqS(a,b))      map = EQ' (sexpr2pter' a map, sexpr2pter' b map)
    | sexpr2pter' (NotS a)        map = NOT'(sexpr2pter' a map)
  *)    
  (* Prueft, ob Elemente in xs bereits in map deklariert wurden *)
(*  fun kaper_free xs map = foldl ( fn ( x, b) => (map x; false) handle Error _ => b ) true xs *)
 

(* Returns  Bound Variable List *)
  fun allbndvar' (AppS( a, b ))  bs = (allbndvar' a bs) @ (allbndvar' b bs)
    | allbndvar' (CstS _)        bs = bs
    | allbndvar' (VarS _)        bs = bs	  
    | allbndvar' (LamS((s,t),e)) bs = (allbndvar' e (s::bs))
    | allbndvar' (ExS((s,t),e))  bs = (allbndvar' e (s::bs))
    | allbndvar' (AllS((s,t),e)) bs = (allbndvar' e (s::bs))
    | allbndvar' (AndS(a,b))     bs = (allbndvar' a bs) @ (allbndvar' b bs)
    | allbndvar' (OrS(a,b))      bs = (allbndvar' a bs) @ (allbndvar' b bs)
    | allbndvar' (ImpS(a,b))     bs = (allbndvar' a bs) @ (allbndvar' b bs)
    | allbndvar' (EqS(a,b))      bs = (allbndvar' a bs) @ (allbndvar' b bs)
    | allbndvar' (NotS(a))       bs = allbndvar' a bs
  fun allbndvar sexpr = allbndvar' sexpr nil
      
  (* String Ordnung *)
  val str_order = fn (x:string,y:string) => if x < y then LESS else if x > y then GREATER else EQUAL

  (* remove_dbl loescht das Doppelauftreten der Variablen in der Liste *)
  fun remove_dbl xs = foldr ( fn (x,nil) => [x] | (x,ys) => if x = hd(ys) then ys else x::ys ) 
      nil (Listsort.sort str_order xs)


  fun sexpr2pter (AppS(a,b))     trans map typmap = A(sexpr2pter a trans map typmap, sexpr2pter b trans map typmap)
    | sexpr2pter (CstS(s) )      trans map typmap = C(s)
    (*  Ist eine Variable nicht in der map ==> Konstante oder unbekannt *)
    (*  Im Falle unbekannt, merkt der Typchecker dies !! *)
    | sexpr2pter (VarS(s) )      trans map typmap = ((V (map s)) handle Error _ => (C(s)))
    | sexpr2pter (LamS((s,t),e)) trans map typmap = 
      let 
	  val newtype = if (t=TC("not_specified")) 
			then ((typmap s) handle Error _ => TC ("bool"))
			else t  
      in 
	  L((newtype,(trans s)), sexpr2pter e trans (fn t => if t=s then 0 else (map t) + 1) typmap )
      end
    | sexpr2pter (ExS((s,t),e))  trans map typmap = 
      let 
	  val newtype = if (t=TC("not_specified")) 
			then ((typmap s) handle Error _ => TC ("bool"))
			else t  
      in 
	  EX((newtype,(trans s)), sexpr2pter e trans (fn t => if t=s then 0 else (map t) + 1) typmap )
      end
    | sexpr2pter (AllS((s,t),e)) trans map typmap = 
      let 
	  val newtype = if (t=TC("not_specified")) 
			then ((typmap s) handle Error _ => TC ("bool"))
			else t  
      in 
	  ALL((newtype,(trans s)), sexpr2pter e trans (fn t => if t=s then 0 else (map t) + 1) typmap )
      end
    | sexpr2pter (AndS(a,b))     trans map typmap = AND(sexpr2pter a trans map typmap, sexpr2pter b trans map typmap )
    | sexpr2pter (OrS(a,b))      trans map typmap = OR (sexpr2pter a trans map typmap, sexpr2pter b trans map typmap )
    | sexpr2pter (ImpS(a,b))     trans map typmap = IMP(sexpr2pter a trans map typmap, sexpr2pter b trans map typmap )
    | sexpr2pter (EqS(a,b))      trans map typmap = EQ (sexpr2pter a trans map typmap, sexpr2pter b trans map typmap )
    | sexpr2pter (NotS a)        trans map typmap = NOT(sexpr2pter a trans map typmap)




     
(* --------------------------------------------------------------------*)
(* |                   ENDE Hilfsprozeduren                           |*)
(* --------------------------------------------------------------------*)
      
      
      

(* --------------------------------------------------------------------*)
(* |        BEGIN Methoden zur Bearbeitung des First Goal             |*)
(* --------------------------------------------------------------------*)



(* [init_goal]
 * legt initiales Goal an 
 *)
  fun init_goal() = 
      let 
	  val init_gamma = Gamma.empty()     (* Gamma Graph inital leer *)
	  val init_env = Env.empty()         (* lege leere Typ/Var.umgebung an *)
	  val init_def = Definition.empty()  (* lege leere Definitionen table an *)
		 
      in ( [init_env] , nil, init_gamma , init_def )    (* Goalliste list ist initial NIL *)
      end

 
 
  (* [init_goal_env]
   * legt initiales Goal mithilfe eines alten Goals an
   *)
  fun init_goal_env(oldgoal) = 
      let   
	  val (old_envs, _ , old_gamma, olddef)  = oldgoal
      in ( old_envs, nil, old_gamma, olddef)
      end
  
 (* [ first_goal ]
  *  - scanne geparsten Ausdruck und update entsprechend gebundene Variablen
  *  - Umwandlung von sexpr -> pter
  *  - Stelle pter A im Implikativen Format dar : TRUE => A
  *)

  fun first_goal(oldgoal, sexpr) = 
      let 
	  val (type_envs, pters, gamma, def) = oldgoal
	  (* type_envs liste invariant nicht leer *)
	  val type_env = hd(type_envs) handle empty => raise Error("Environmentlist empty!")
	  
	  (* scanne nach gebundenen Variablen *)
	  val bndxs = remove_dbl( allbndvar sexpr )

	  (* !!! Mache die neue Boundvar Liste der Umgebung bekannt !!! *)
     	  val newenv = Env.update_bndmaplist(type_env, bndxs)
	      
	  (* Mapping von Freelist@Bndlist temporaer *)
	  val freemaps2i = Env.freemaps2i(newenv) 
	  val bndmaps2i  = Env.bndmaps2i(newenv)
	  val freetypmap = fn s => Env.typf(newenv, s)

	  (* Beginne Uebersetzung in DeBrujn Darstellung *)
	  val newpter'  = sexpr2pter sexpr bndmaps2i freemaps2i freetypmap
	  (* Intern liegen alle Terme /Formeln im Deduktionsformat vor *)
	  val newpter   = IMP ( C "TRUE" , newpter' )  
	 (* Debug
	  val _ = print ("\n\n" ^ ( Visual.pter2string newpter)  ^ "\n\n")
	  *)
      in
	  (* Typecheck wird spaeter durchgefuehrt *) 
	  ( newenv::tl(type_envs), pters@[newpter], gamma, def )  
      end
  


  (* [ get_first_goal ]
   * liefert den zum ersten Goal gehoerigen Ausdruck 
   *)
  fun get_first_goal(oldgoal) = 
      let 
	  val (_, pters,_,_) = oldgoal
      in 
	  hd(pters) handle Empty => raise Error "no avtive goals\n"
      end
  


 (* [ update_first_goal ]
  * ersetzt das erste goal durch einen Ausdruck pter 
  *)
  fun update_first_goal(oldgoal, pter) = 
      let 
	  val (type_envs, pters, gamma, def ) = oldgoal
	  val type_env = hd(type_envs) handle empty => raise Error("Environmentlist empty!")
	      
      in 
	  if (null(pters)) then raise Error "First goal doesn't exist, so it can't be updated \n"
	  else 
	      (* Stimmen die Typen des FirstGoal und des 
	       * Ausdrucks pter ueberein, darf ersetzt werden 
	       *)
	      if ( Typecheck.tycheck pter ( (fn c => Env.typc (type_env, c)),
		                            (fn i => Env.typi (type_env, i)),
		                            (Env.typmaps2b(type_env))         ) )
		  then ( case pter of
			(C "TRUE") => if (null(tl(pters)) andalso null(tl(type_envs))) 
					  then ( print ("> PROOF DONE\n"); done_goal() )
				      else (tl(type_envs), tl(pters), gamma, def)
					  
		      | (C "FALSE") => (print ("> THEOREM NOT TRUE try it again \n"); init_goal() )
		      | p => (type_envs, (p::(tl(pters))), gamma ,def)
					  )
	      else raise Error ("Formula is illtyped, must have type bool")
		 
      end 

(* [remove_first_goal]
 * loescht das erste Goal und entsprechende Umgebung
 *)

  fun remove_first_goal ( goal ) =
      let
  	  val (type_envs, pters, gamma, def) = goal
      in
	  if (null(type_envs) orelse null(pters) ) then 
	      raise Error ("There is no first goal removeble!\n")
	  else
	      (tl(type_envs), tl(pters), gamma, def)
      end
		      


(* [typecheck]
 * prueft, ob der Typ des ersten Goals Bool ist
 *)
  fun typecheck(goal) = 
      let
	  val (type_envs, pters,_,_) = goal
	  val type_env = hd(type_envs)
      in
	  if (null(pters)) 
	      then raise Error "First goal doesn't exists, so it cannot be typechecked\n"
	  else 
	      ( Typecheck.tycheck (hd(pters)) ( (fn c => Env.typc (type_env, c)),
					        (fn i => Env.typi (type_env, i)), 
						(Env.typmaps2b(type_env))        ) )
      end
  
(* --------------------------------------------------------------------*)
(* |        ENDE Methoden zur Bearbeitung des First Goal              |*)
(* --------------------------------------------------------------------*)


(* --------------------------------------------------------------------*)
(* |        BEGIN Methoden zur Erweiterung der Goalliste              |*)
(* --------------------------------------------------------------------*)

  (* [ split_goal ] 
   * erweitere die Liste der Goals, indem der Ausdruck pter
   * als weiteres Subgoal hinzugefuegt wird.
   *)
  
  fun split_goal(oldgoal, new_pter) = 
      let 
	  val (type_envs, pters, gamma, def) = oldgoal
	  val type_env = hd(type_envs)
	  val pter     = hd(pters)

	  (* fuege das subgoal nach dem aktuellen ein
	   * und dupliziere die Umgebung !!!
	   *)
	  val new_type_envs = type_env::type_env::(tl(type_envs))
	  val new_pters     = pter::new_pter::(tl(pters))

      in 
	  (* New_Pter muss den Typ Bool haben *)
	  if ( Typecheck.tycheck new_pter ( (fn c => Env.typc (type_env, c)),
	                                    (fn i => Env.typi (type_env, i)), 
	                                    (Env.typmaps2b(type_env))        ) )
	    (*   then (type_envs@[type_env], pters@[pter], gamma, def)  *)
	      then ( new_type_envs, new_pters, gamma, def )
	  (* Dupliziere Umgebungen *)
	  else (type_envs, pters, gamma, def)
      end


  (* [ add_goal ] 
   * erweitere die Liste der Goals, indem der erste Ausdruck des addgoals
   * zusammen mit seiner Umgebung als weiteres Subgoal hinzugefuegt wird.
   *)
  
  fun add_goal(oldgoal, addgoal) = 
      let 
	  val (type_envs, pters, gamma, def) = oldgoal
	  val (add_type_envs, add_pters, _ ,_ ) = addgoal
	  val add_type_env = hd(add_type_envs)
	  val add_pter    = hd(add_pters)
	     
	  (* fuege das subgoal am Ende ein  *)
	  val new_type_envs = type_envs @ [add_type_env]
	  val new_pters     = pters @ [add_pter]

      in 
	  ( new_type_envs, new_pters, gamma, def )
      end



  
 (* [update_ctypeenv]
   * Konstante s mit Typ t, wird der Konstanten/Typumgebung hinzugefuegt
   * hierzu werden Methoden der Struktur Env aufgerufen
   *)

  fun update_ctypeenv(oldgoal, s, t) = 
      let 
	  val (old_envs, pters, gamma,def) = oldgoal
	  val old_env = hd(old_envs)

	  val newenv' = Env.update_ctypeenv(old_env,s,t)
	  val newenv  = Env.update_constmaplist(newenv',[s])

      in
	  ( newenv::tl(old_envs), pters, gamma, def)
      end
 
  (* [update_ftypeenv]
   * Variable s mit Typ t, wird der Variablen/Typumgebung hinzugefuegt
   * hierzu werden Methoden der Struktur Env aufgerufen
   *)

   fun update_ftypeenv (oldgoal, s, t) = 
      let 
	  val (old_envs, pters, gamma,def) = oldgoal
	  val old_env = hd(old_envs)

	  val newenv' = Env.update_ftypeenv(old_env,s,t)
	  val newenv  = Env.update_freemaplist(newenv',[s])

      in
	  ( newenv::tl(old_envs), pters, gamma, def)
      end


 (* [update_bndvar]
   * Gebundene Variable mit Namen s wird der Umgebung hinzugefuegt
   * Diese Methode wird lediglich zur Uebersetzung von Gleichheitstermen
   * gebraucht
   *)
  fun update_bndvar (oldgoal, s) =
      let 
	  val (old_envs, pters, gamma, def) = oldgoal
	  val old_env = hd(old_envs)
	  val new_env = Env.update_bndmaplist(old_env, [s])
      in
	  ( new_env::tl(old_envs) , pters , gamma, def )
      end

  (* [new_type]
   * Typ s wird der Typliste hinzugefuegt
   * hierzu wird die Methode Env.new_type der Struktur Env aufgerufen
   *)

  fun new_type(oldgoal, s) = 
      let 
	  val (old_envs, pters, gamma, def) = oldgoal
	  val old_env = hd(old_envs)
	  val new_env = Env.new_type(old_env,s)
      in
	  ( new_env::tl(old_envs) , pters , gamma, def )
      end

 


 (* [kaper_check]
   * Checkt, ob freie Variablen durch Instanziierungen in gebundenen Ausdruecken 
   * gekapert wurden.
   * Checkt lediglich ERSTES Goal  !! Achtung !!
   * Die Umbenennung kann mehrmals erfolgen, da es zu einem Dominoeffekt kommen
   * kann -> aber terminiert, da Freie Variablen endlich !!
   *)

  fun kaper_check(oldgoal) = 
      let
	  val (old_envs, pters, gamma, def) = oldgoal
	  val old_env = hd(old_envs) 
	  val pter = hd(pters)

	  val firstbndmapi2s   = Env.bndmapi2s(old_env)
	  val freemapi2s       = Env.freemapi2s(old_env)
	  val constmapi2s      = Env.constmapi2s(old_env)
	  val firstmaps = (firstbndmapi2s, freemapi2s, constmapi2s)
	      
	  fun loopUempty env nil = env
	    | loopUempty env cs  = 
	      let val new_env   = Env.renameBndVars(env, cs)
		  val bndmapi2s = Env.bndmapi2s(new_env)
		  val maps      = (bndmapi2s, freemapi2s, constmapi2s)
		  val newcs     = ETT.getCriticalBnds pter maps
	      in loopUempty new_env newcs 
	      end
	  
	  val crit_bnds = ETT.getCriticalBnds pter firstmaps
	 (* DEBUG val _ = foldl( fn (i,_) => print (Int.toString(i) ^ " ") ) () crit_bnds *)
	  val new_env =  loopUempty old_env crit_bnds
	      
      in
	  (new_env::tl(old_envs), pters, gamma, def)

      end

(* [freeVarIntro]
 * calls the Env.freeVarIntro
 * more information there
 *)

  fun freeVarIntro(oldgoal, i) =
      let
	  val (old_envs, pters, gamma, def) = oldgoal
	  val oldenv = hd(old_envs) 
      in
	  Env.freeVarIntro(oldenv, i)
      end


(* --------------------------------------------------------------------*)
(* |        ENDE Methoden zur Erweiterung der Goalliste               |*)
(* --------------------------------------------------------------------*)

(* --------------------------------------------------------------------*)
(* |                        BEGIN Gamma Methoden                      |*)
(* --------------------------------------------------------------------*)


(* [next_gamme_nr]
 * liefert die naechste freie Gamma Nr 
 *)

  fun next_gamma_nr(oldgoal) = 
      let
	  val (_,_,gamma,_) = oldgoal
      in
	  (* Zaehle ab 0 *)
	  Gamma.nr_gammavars(gamma) 
      end


  (* [add_delta] 
   * Eine neue Delta Variable wird dem Graphen hinzugefuegt
   * Umgebung hinzugefuegt
   *)
  fun add_delta(oldgoal, i) = 
      let
	  val (envs,pters,gamma,def) = oldgoal
	  val newgamma = Gamma.add_delta(gamma, i)
      in
	 (envs, pters, newgamma, def)
      end
  


 (* [add_gamma] 
  * Eine neue Gamma Variable wird dem Graphen hinzugefuegt
  * Umgebung hinzugefuegt
  *)
  fun add_gamma(oldgoal) = 
      let
	  val (envs,pters,gamma,def) = oldgoal
	  val nextnr = Gamma.nr_gammavars(gamma)
	  val newgamma = Gamma.add_gamma(gamma, nextnr)
      in
	  (envs, pters, newgamma, def)
      end
  

  (* [substitute_gamma] 
   * gibt true zurueck, wenn die Ersetzung ok ist
   * false, wenn es nicht moeglich zu ersetzen
   *)
  fun substitute_gamma oldgoal ( i, vars) =
      let
	  val (_,_,gamma,_) = oldgoal
      in
	  Gamma.substitute gamma (i,vars)
      end

  (* [merge_envs]
   * mergt nur die Umgebungen des oldgoals und des instgoals, so dass
   * - BndVarMap des Instgoals uebernommen werden
   * - FreeVarMap, Typelist und Typemap des oldgoals
   *)

  fun merge_envs(oldgoal, instgoal) = 
      let
	  val (old_envs, pters, old_gamma, def) = oldgoal
	  val old_env = hd(old_envs)
	  val (inst_envs, _, _ ,_) = instgoal	  
	  val inst_env = hd(inst_envs)
	  val merge_env = Env.back_instance_env(old_env,inst_env)
      in
	  (merge_env::tl(old_envs), pters, old_gamma, def)
      end


(* --------------------------------------------------------------------*)
(* |                        ENDE Gamma Methoden                       |*)
(* --------------------------------------------------------------------*)
 

(* --------------------------------------------------------------------*)
(* |                     BEGIN Definitionen Methoden                  |*)
(* --------------------------------------------------------------------*)


  (* [new_definition]
   * def <name> = <formel>        modus = CLASSIC
   * axiom <name> = <formel>      modus = AXIOM
   * lemma <name> = <formel>      modus = LEMMA
   * - lege temporaeres goal aus formel an
   * - bestimme den Typ t der formel
   * - update Definitionen Hashtable keyword = name
   * - lege freie variable <name> des Typs t an
   *)
  fun new_definition ( goal, name, formel, modus) = 
      let
	  val (_ , pters, gamma, def ) = goal

	  (* lege temporaeres goal an mit formel *)
	  val (new_type_envs, def_pters, _ , _ ) =  first_goal(goal, formel)

	  val new_type_env = hd(new_type_envs)
	  val def_pter = case (hd(def_pters)) of (* the desired expression *)
	                  (IMP(C "TRUE", p)) => p
			    | _ => raise Error ("interner Fehler xy\n")

	  (* take the type of the def_pter *)
	  val def_type = ( Typecheck.get_type (def_pter) 
			                   ( (fn c => Env.typc (new_type_env, c)),
  					     (fn i => Env.typi (new_type_env, i)),
					     (Env.typmaps2b(new_type_env))        )
			 ) handle Typecheck.illtyped e => 
	      raise Error ("Definition/Axiom/Lemma was illtyped: ^ e \n")

	  (* update the Definition table *)
	  (* create a ref of current Environment 
	   * only saved if mode = LEMMA 
	   *)
	  val envref = ref (new_type_env) 
	  val newdef = Definition.insert_def def (name, def_pter, modus) envref
	     
	  (* !!! DEBUG !!! *)
	  (* val _ = print ("AXIOM " ^ name ^ " :" ^ Visual.pter2string(def_pter) ^ "\n") *)
	  (* update the goals *)
	  val newgoal' = (new_type_envs, pters, gamma, newdef)
	  (* makes definition <name> as a new free variable known *)
	  val newgoal  = update_ftypeenv(newgoal', name, def_type)
      in
	  newgoal
      end


  (* [save_goal]
   * Saves the current solved goal under AXIOM name 
   *)

  fun save_goal (goal, name, pter) = 
      let
	  val (type_envs, pters, gamma, def) = goal
	  val dummyenvref = ref (Env.empty())
	  val newdef = Definition.insert_def def (name, pter, AXIOM) dummyenvref
	  val newgoal = (type_envs, pters, gamma, newdef)
      in
	  newgoal
      end


  (* [check_def]
   * gibt true zurueck, wenn der Teilausdruck for eine Definition beinhaltet 
   * ruft Definition.check_def auf
   *)
  fun check_def(goal, for) = 
      let
	  val (type_envs,_,_, def ) = goal
	  val type_env = hd(type_envs)
      in
	  Definition.check_def def (Env.freemapi2s(type_env)) for
      end

  (* [translate_def]
   * ersetzt alle auf erster Ebene gefundenen Definitionen innerhalb
   * des Terms <pter>
   *)

  fun translate_def(goal, pter) = 
      let
	  val (type_envs,_,_,def) = goal
	  val type_env = hd(type_envs)
      in
	  Definition.translate_def def (Env.freemapi2s(type_env)) pter
      end

   (* [get_definition] 
   * gibt die fuer den Key gespeicherte Definition aus
   * der Hashtable zurueck 
   *)

  fun get_definition(goal,name) =
      let
	  val (_,_,_,def) = goal
      in
	  Definition.get_term def name
      end

 (* [get_definitions_kind] 
   * gibt die Art der fuer den Key gespeicherten Definition aus
   * der Hashtable zurueck, sprich CLASSIC | LEMMA | AXIOM
   *)

  fun get_definitions_kind(goal, name) =
      let
	  val ( _,_,_,def ) = goal
      in 
	  Definition.get_kind def name
      end
	  
  (* [change_definitions_kind]
   * aendert den Modus der Definition 
   * i.e. von LEMMA -> AXIOM ,wenn Lemma
   * bereits hinzugefuegt wurde! (vgl.Apply.sml)
   *)
  fun change_definitions_kind (goal, name, kind) =
      let
	  val ( _,_,_,def ) = goal
      in 
	  Definition.change_kind def (name,kind)
      end



  (* [get_envref]
   * gibt die Referenz auf die Umgebung zurueck, die
   * bei der Definition des Lemmas aktuell war
   *)
  fun get_envref(goal, name) =
      let
	  val ( _,_,_,def ) = goal
      in 
	  Definition.get_envref def name
      end
(* --------------------------------------------------------------------*)
(* |                     ENDE Definitionen Methoden                   |*)
(* --------------------------------------------------------------------*)

 
(* --------------------------------------------------------------------*)
(* | BEGIN Tunnelfunktionen ( Aufruf dergleichen Methoden in Env.sml )|*)
(* --------------------------------------------------------------------*)

(* Fuer Erklaerung der Methoden, siehe entsprechende Methoden 
 * innerhalb der Struktur Env.sml
 *)

  fun freemaps2i(oldgoal) = let 
			    val (envs,_,_,_) = oldgoal
			    val env = hd(envs)
			in
			    ( Env.freemaps2i(env))
			end

  fun freemapi2s(oldgoal) = let 
			    val (envs,_,_,_) = oldgoal
			    val env = hd(envs)
			in
			    ( Env.freemapi2s(env))
			end

  fun bndmapi2s (oldgoal ) = let
				 val (envs,_,_,_) = oldgoal
				 val env = hd(envs)
			     in
				 (Env.bndmapi2s (env))
			     end

  fun bndmaps2i (oldgoal ) = let
				 val (envs,_,_,_) = oldgoal
				 val env = hd(envs)
			     in
				 (Env.bndmaps2i (env))
			     end

   fun typi(oldgoal) = let
			  val (envs,_,_,_) = oldgoal
			  val env = hd(envs)
		      in 
			  fn i => (Env.typi(env,i))
		      end

  fun typc(oldgoal) = let
			  val (envs,_,_,_) = oldgoal
			  val env = hd(envs)
		      in 
			  fn c => (Env.typc(env,c))
		      end


  fun typemap(oldgoal) = let
			     val (envs,_,_,_) = oldgoal
			     val env = hd(envs)
			 in 
			     (Env.typmaps2b(env))
			 end

  fun nr_freevars(oldgoal) = let
				 val (envs,_,_,_) = oldgoal
				 val env = hd(envs)
			     in
				 (Env.nr_freevars(env))
			     end

  fun nr_bndvars(oldgoal) = let
				 val (envs,_,_,_) = oldgoal
				 val env = hd(envs)
			     in
				 (Env.nr_bndvars(env))
			     end


(* --------------------------------------------------------------------*)
(* |                        ENDE Tunnelfunktionen                     |*)
(* --------------------------------------------------------------------*)

                        

(* --------------------------------------------------------------------*)
(* |                     BEGIN Zusatzmethoden                         |*)
(* --------------------------------------------------------------------*)


(* [get_env]
 * gibt die aktuelle Umgebung env zurueck
 *)
  fun get_env(oldgoal) = 
      let 
	  val ( envs , _ ,_,_ ) = oldgoal
	  val env = hd(envs)
      in
	  env
      end
  
(* [set_env]
 * setzt die aktuelle Umgebung 
 *)
  fun set_env(oldgoal, newenv) = 
      let
	  val ( old_envs , pters , gamma,def) = oldgoal
      in
	  (newenv::tl(old_envs), pters, gamma, def)
      end
  
(* [get_nr_of_goals]
 * gibt die Anzahl der aktiven Goals/Subgoals zurueck
 *)
  fun get_nr_of_goals(oldgoal) = 
      let 
	  val (_, pters ,_,_) = oldgoal
      in 
	  List.length(pters)
      end

(* [cyclic_shift]
 * shiftet die pters und envs cyclic nach oben 
 * verwendet in Instance.sml
 *) 			    

  fun cyclic_shift(oldgoal) = 
      let
	  val (envs, pters, gamma , def ) = oldgoal
      in
	  case (envs, pters) of
	      (nil, nil )                => oldgoal
	    | ( (e::env) , (p::pter) )   => (env @ [e], pter @ [p], gamma, def)
	    | _                          => oldgoal
      end

(* [Print_goal]
 * printet die Goals
 *)
  fun print_goal (oldgoal) = 
      let 
	  val (envs, pters, _,_ ) = oldgoal
	  val nrgoals = List.length(pters)
          (* val _ = print ("> ") *)
	  fun print_it env pter n = Visual.print_goal (pter, Env.freemapi2s(env), Env.bndmapi2s(env) ) (n, nrgoals)

	  fun cyclic_print nil nil         n = ()
	    | cyclic_print (x::xs) (y::ys) n = ( print_it x y n; cyclic_print xs ys (n+1) )
	    | cyclic_print _ _             n = ()
             
      in
	  cyclic_print envs pters 1
      end

(* [latex_goal]
 * gibt latex Strings der angegebnen Term Liste zurueck
 *)
  fun latex_goal (oldgoal, pter) = 
      let 
	  val (envs, _ , _,_ ) = oldgoal
	  val env = hd(envs)
      in
	  Visual.latex_goals (pter,(Env.freemapi2s(env)), (Env.bndmapi2s(env)))
      end


(* --------------------------------------------------------------------*)
(* |                     ENDE Zusatzmethoden                          |*)
(* --------------------------------------------------------------------*)

end (* of struct *)
(* ==========================================================================================
                             Position.sml
   ========================================================================================== *)

structure Position= 
struct
(* Position.sml 
 *)

local open Data Process in

exception internalerror

val initvmap = fn x:int => raise Error (" INIT VMAP ERROR !!(POSITION.SML) \n ")
fun cons(f,s) = fn x:int => if x = 0 then s else f(x-1)

fun javainput (C c)                l (fmap,bmap) vmap  = "C " ^ c ^" "
  | javainput (V x)                l (fmap,bmap) vmap  = "V " ^ (if ( x >= l ) then fmap (x-l) else vmap x) ^ " "
  | javainput (g as (GAMMA (_,i))) l (fmap,bmap) vmap  = "GAMMA " ^ Int.toString(i)  ^ " " 
  | javainput (A(m,m'))            l (fmap,bmap) vmap  = "APP {" ^ (javainput m  l (fmap,bmap) vmap) ^ "}" ^
                                                             "{" ^ (javainput m' l (fmap,bmap) vmap) ^ "}"
  | javainput (L((t,i),m))         l (fmap,bmap) vmap  = "LAM " ^ (bmap i ) ^ "{"
							 ^ ( javainput m (l+1) (fmap,bmap) (cons(vmap,bmap i)) ) ^"}" 
  | javainput (EX((t,i),m))        l (fmap,bmap) vmap  = "EX " ^ (bmap i ) ^ "{"
							 ^ ( javainput m (l+1) (fmap,bmap) (cons(vmap,bmap i)) ) ^"}"
  | javainput (ALL((t,i),m))       l (fmap,bmap) vmap  = "ALL " ^ (bmap i ) ^ "{"
							 ^ ( javainput m (l+1) (fmap,bmap) (cons(vmap,bmap i)) ) ^"}"
  | javainput (AND(m,m'))          l (fmap,bmap) vmap  = "AND {" ^ (javainput m  l (fmap,bmap) vmap) ^ "}" ^
                                                             "{" ^ (javainput m' l (fmap,bmap) vmap) ^ "}"
  | javainput (OR(m,m'))           l (fmap,bmap) vmap  = "OR {" ^ (javainput m  l (fmap,bmap) vmap) ^ "}" ^
                                                             "{" ^ (javainput m' l (fmap,bmap) vmap) ^ "}"
  | javainput (IMP(m,m'))          l (fmap,bmap) vmap  = "IMP {" ^ (javainput m  l (fmap,bmap) vmap) ^ "}" ^
                                                             "{" ^ (javainput m' l (fmap,bmap) vmap) ^ "}"
  | javainput (EQ(m,m'))           l (fmap,bmap) vmap  = "EQ {" ^ (javainput m  l (fmap,bmap) vmap)  ^ "}" ^
                                                             "{" ^ (javainput m' l (fmap,bmap) vmap) ^ "}"
  | javainput (NOT(m))             l (fmap,bmap) vmap  = "NOT {" ^ (javainput m  l (fmap,bmap) vmap) ^ "}" 

fun get_premisse (AND(a,b)) i = if i <= 1 then a else get_premisse b (i-1)
  | get_premisse t i = if i = 1 then t else raise internalerror

(* Prefix fuer Programm *)
fun calc' 0 = ""
  | calc' i = ",2" ^ (calc' (i-1))
fun calculate_prefix 0 _ = "2"
  | calculate_prefix i ende = "1" ^ (calc' (i-1)) ^ (if ende then "" else ",1")

fun main(goal, i) =
    let 
	val _ = if (i < 0) then raise Error("Premise Nr. " ^ Int.toString(i) ^ " not found!\n") else ()
	val pter = Goal.get_first_goal(goal)
	val (left,right) = ( case pter of
			    IMP(a,b) => (a,b)
			  | _ => raise Error("Term not in implicative form") )
				

	val maps = ( Goal.freemapi2s(goal) , Goal.bndmapi2s(goal) )
   
	val subterm = ( if (i > 0) then (get_premisse left i) else right )
	    handle internalerror => 
		raise Error ("Premise Nr. " ^ Int.toString(i) ^ " not found!\n")
		    
	val ende = (get_premisse left (i+1) ; false) handle internalerror => true
	    
	val prefix = ( case left of
		      (C "TRUE") => "0"
		    | _ => calculate_prefix i ende   )

	val s    = javainput (subterm) 0 maps initvmap
	    
	(* val _ = print ("Term : \n" ^ s ^ "\n" ^ "Prefix :" ^ prefix ^ "\n")  *)
	val _ = print ("Rufe externes Programm JavaPos auf\n")

	val _ = system ("java -jar " ^ Paths.position_path ^ "Position.jar \""^s^"\"" ^ " " ^ prefix);
    in
	()
    end
end
end (* of structure Position.sml *)
(* ==========================================================================================
                             LatexOutput.sml
   ========================================================================================== *)

structure LatexOutput= 
struct
local open Data in


(* split wird in List.partition benutzt, um die goalxs 
 * in die Liste der Definitionen und der Kommandos zu teilen 
 *)
val split = fn (g,c) =>  
    ( case c of
	  LetVarS _    => true
        | LetConstS _  => true
	| DefS _       => true
	| TypS _       => true
	| EmptyS       => true
	| GoalS _      => false
        | SaveS _      => false
	| CmdS _       => false
	| CmdEqS _     => false
	| ForS _       => false
        | ErrorS _     => false
        | FileEndS     => false 
	      ) 

(* Returns a list of ( Goal and commando)  lists  *)
fun splitgoals' nil                       temp result = (temp::result)
  | splitgoals' ((t as (g,GoalS(_)))::xs) nil  result = splitgoals' xs [t] (result)
  | splitgoals' ((t as (g,GoalS(_)))::xs) temp result = splitgoals' xs [t] (temp::result)
  | splitgoals' (x::xs)                   temp result = splitgoals' xs (temp@[x]) result
fun splitgoals xs = List.rev (splitgoals' xs nil nil)

(* Hardcoded Header, falls Datei Header.tex nicht gefunden wird,
   nehme Standart Header 
 *)
val headerxs =
["%This is one possible proof output header\n",
 "% you can modify it\n",
 "\\documentclass[a4paper,10pt]{article}\n",
 "\\newtheorem{theorem}{Theorem}[section]\n",
 "\\newtheorem{lemma}[theorem]{Lemma}\n",
 "\\newtheorem{proposition}[theorem]{Proposition}\n",
 "\\newtheorem{corollary}[theorem]{Corollary}\n",
 "\\newenvironment{proof}[1][Proof]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newenvironment{definition}[1][Definitionen]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newenvironment{example}[1][Example]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newenvironment{remark}[1][Remark]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newcommand{\\qed}{\\nobreak \\ifvmode \\relax \\else\n",
 "     \\ifdim\\lastskip<1.5em \\hskip-\\lastskip\n",
 "     \\hskip1.5em plus0em minus0.5em \\fi \\nobreak\n",
 "     \\vrule height0.75em width0.5em depth0.25em\\fi}\n",
 "\n",
 "\\title{Proof Assistant NED Result}\n",
 "\\usepackage{amsmath,amssymb}\n",
 "\\begin{document}\n",
 "\\maketitle\n",
 "% END OF HEADER\n"] 

(* Hardcoded End, falls Datei End.tex nicht gefunden wird,
   nehme Standart Header 
 *)
val endxs = 
[ "% End Notes\n",
  "\\end{document}\n" ]

(* Kopiert den Inhalt der Header.tex / End.tex files in Output File *)
fun copy instream outstream =
    case TextIO.inputLine instream of
	"" => ()
      | s  => (TextIO.output(outstream,s); 
	       copy instream outstream )

fun copy' sxs outstream = foldl ( fn (s,_) => TextIO.output(outstream, s) ) () sxs



(* Zustaending fuer den Header der NEDOutput.tex *)
fun HeaderOutput outstream title =
( let 
      val instream = TextIO.openIn("../LatexOut/Header.tex") 
  in 
      ( copy instream outstream ;     
       TextIO.output(outstream, ("\\section{"^title^"}\n"));
       TextIO.closeIn instream 
       )
  end
) handle IO.Io _ => ( copy' headerxs outstream ; TextIO.output(outstream, ("\\section{"^title^"}\n")) )
    

(* Zustaending fuer den Bottom der NEDOutput.tex *)
fun EndOutput outstream = 
( let 
      val instream = TextIO.openIn("../LatexOut/End.tex")
  in 
      ( copy instream outstream ; 
       TextIO.closeIn instream  )
  end
) handle IO.Io _ => copy' endxs outstream


fun skipgoals g ((goal,CmdS(REMOVECMD _ ))::cx) = skipgoals goal cx
  | skipgoals g ((goal,CmdS(SHIFTCMD _ ))::cx)  = skipgoals goal cx
  | skipgoals g ((goal,SaveS(_))::cx) = skipgoals goal cx
  | skipgoals g cx = (g,cx)


fun DefinitionOutput _ nil = ()
 |  DefinitionOutput output defxs = 
    let 
	val _ = output ("\\begin{definition}:\\\\\n\\begin{itemize}")
	val _ = DefOut output (defxs)
	val _ = output ("\\end{itemize} \n\\end{definition}")
    in 
	()
    end
and DefOut output nil = ()
  | DefOut output (d::dx) = 
    let
	val (goal,cmd) = d
	val itemout = fn s => output( "\\item "^s )
    in
	(
	 ( case cmd of
	    LetVarS (s,t) => itemout ("Sei Konstante "^Visual.filter(s)^" : "^(Visual.ty2latex t) )
          | LetConstS (s,t) => itemout ("Sei Variable "^Visual.filter(s)^" : "^(Visual.ty2latex t) )
	  | DefS (s,_,k) => let
				val term = Goal.get_definition(goal,s)
				val prefix = ( case k of
					       CLASSIC => "Def "
					     | LEMMA   => "Lemma "
					     | Axiom   => "Axiom " )
			    in
				itemout (prefix ^ Visual.filter(s) ^ " = " ^(Goal.latex_goal(goal,term)))
			    end
	  | TypS (s)   => itemout ( "Neuer Typ : " ^ Visual.filter(s) )
	  | _ => ()
			  
	 );  (* end of case *)
	       DefOut output dx
	       )
    end



fun TheoremOutput output goalspez = 
    let
	val (goal,cmd) = goalspez 
	val _ = ( case cmd of 
	          (GoalS _ ) => ()
		 | _ => raise Error "No goal specification was found !\n"
  	        )
	     
	val term' = Goal.get_first_goal(goal)
	val term = case term' of
	             IMP(C "TRUE" , right ) => right
		   | t => t
        val _ = output("\\newpage")
	val _ = output("\n\\begin{theorem}:\\\\")
	val _ = output(Goal.latex_goal(goal,term))
	val _ = output("\\end{theorem}")
    in ()
    end



fun ProofOutput output nil = ()
 |  ProofOutput output cmdxs = 
    let 
	val _ = output ("\\begin{proof}:\\\\\\\\")
	val _ = ProofOut output (cmdxs)
	val _ = output ("\\hspace*{5mm} $\\top$ \\\\\n\\qed\n\\end{proof}")
    in 
	()
    end
and ProofOut output nil = ()
  | ProofOut output (c::cx) = 
    let
	val (goal,cmd) = c
	val _ =    
	 ( case cmd of
	       CmdS (t)   =>  output ("$\\dashv $ {\\tiny "^Visual.comm2string(t)^"}\\\\\\\\")
	     | CmdEqS (t) =>  output ("$\\dashv $ {\\tiny "^Visual.commeq2string(t)^"}\\\\\\\\")
	     | _ => ()
	  ) (*end of case*)
	val (goal', cx') = skipgoals goal cx 
    in
	( 
	 ( let
	      val actives = (Goal.get_nr_of_goals(goal'))
	   in  
	       if (actives > 0 ) then 
		   if ( Goal.get_first_goal(goal') <> (C "PROOF DONE") )
		       then ( ProofOutSubgoals output goal' (1, actives) ) else ()
		   else ()
	   end
	  );   (* end of case *)
	 
 	  ProofOut output cx'
       )
    end

and ProofOutSubgoals output goal (nr, actives) = 
    let 
	val term = Goal.get_first_goal(goal)  
	(* val _ = if (nr > 1) then (output (Int.toString(nr)^ " Subgoals remaining \\\\")) else () *)
	val _ = output (Goal.latex_goal(goal, term) ^ "\\\\")
    in
	if (* (nr < actives ) *)
	   ( nr < actives )
	    then
		(output (Int.toString(actives - nr) ^ " Subgoal" ^ 
			 (if (actives-nr) > 1 then "s" else "") ^ 
			      " remaining \\\\"))
		(* ProofOutSubgoals output (Goal.remove_first_goal(goal)) ( nr+1 , actives) *)
	else ()
    end

	 

fun GoalOutput output nil = ()
 |  GoalOutput output (x::xs) =
      ( TheoremOutput output (hd(x)) ;
        ProofOutput   output (tl(x)) ;
        GoalOutput output xs          )
      
(* Start here *)
fun main(goalxs,title) = 
let
    val outstream = TextIO.openOut("NEDOutput.tex")
    (* output is a function that prints out s and a carriage return *)
    val output = fn s => TextIO.output(outstream, s^"\n")  
    (* splittet die goalxs in 
       - Preprocessing Commandos ( like Definitions, typedeclarations , var def
       - Goal Spezifikation und Kommandos zur Bearbeitung
     *)

    (* Reversiere deswegen, weil das erste Element der Liste auch
     * das erste Kommando sein soll 
     *)
    val (prexs, cmdxs) = List.partition split ( List.rev (goalxs) )
    val (goalxs) = splitgoals (cmdxs)

in 
    ( HeaderOutput outstream title;
      DefinitionOutput output prexs;
      GoalOutput output goalxs;
      EndOutput outstream ;
      TextIO.closeOut outstream )
end


end (* of local open *)				    
end (* of structure LatexOutput.sml *)
(* ==========================================================================================
                             Equality.sml
   ========================================================================================== *)

structure Equality= 
struct
(* Equality.sml
 * implements the Equality Extension Rules :
 * @ Leibniz Extension :        A(:i) = B(:i)  --->  \/p:i->bool. p A => p B
 * @ Boolean Extensionality :   A(:bool) = B(:bool) ---> A => B (1/2)
 *                                                       B => A (2/2)
 * @ Functional Extensionality: 
 *                    (1)        A(:a->b) = B(:a->b)  ---> \/x:a. A x = B x 
 *		      (2)   (\x:a. A ) = (\x:a. B )   ---> \/x:a. A = B
 *
 * ----------------------------------------------------
 * | !! Equality.sml returns ( newterm , newgoal ) !! |
 * ----------------------------------------------------
 * 
*)                                 


local open Data Goal in

fun main (term, goal) = process term goal

and process (EQ(eqleft,eqright)) goal = 
    let

	 val cmap = Goal.typc(goal)
	 val fmap = Goal.typi(goal)    (* Free Var Map *)
	 val tmap = Goal.typemap(goal) (* Typlisten Map *)

	 (* Hole Typ von subterm a *)
	 val typ_a = Typecheck.get_type eqleft (cmap,fmap,tmap)
	 (* Hole Typ von subterm b *)
	 val typ_b = Typecheck.get_type eqright (cmap,fmap,tmap)

	 val _ = ( if (typ_a <> typ_b) 
		     then raise Error("Different Types in Equality !\n")
		   else () )


	(* [leibniz]
	 * takes basetype i and returns the expanded LeibnizEquality 
	 *        ... => A:i = B:i
	 * ---->  ... => \/p:i->bool. p A => p B
	 *)	     
	 fun leibniz(i) = 
	     let 
		 (* Fuege Geb.Variable <p> der Umgebung hinzu *)
		 val goal' = Goal.update_bndvar(goal, "p") 
		 val nextbndnr = Goal.nr_bndvars(goal') - 1
		 val newterm = ( ALL((TA(i, TC "bool"), nextbndnr), 
				      IMP(A(V 0 , (ETT.up 0 eqleft 1)),
					  A(V 0 , (ETT.up 0 eqright 1) ))) 
				 )

	     in
		  (newterm, goal')
	     end

	 (* [funext]
	 * takes type alpha and returns the expanded Functional
	 * extended Term of "right"
	 *          ... => A(:alpha->beta) = B(:alpha->beta)
	 * ---->    ... => \/z:alpha. A z = B z
	 *)	     
	 fun funext(alpha) = 
	     let 
	 	 (* Fuege Geb.Variable <z> der Umgebung hinzu *)
		 val goal' = Goal.update_bndvar(goal, "z") 
		 val nextbndnr = Goal.nr_bndvars(goal') - 1
		 val newterm = ( ALL((alpha, nextbndnr), 
				      EQ(A( (ETT.up 0 eqleft 1), V 0),
					 A( (ETT.up 0 eqright 1), V 0)))
				 )
     
		
	     in
		 (newterm, goal')
	     end

	 (* [lamext]
	 * substitutes two lambda terms in equality by an ALL expression
	 *          ... => (\x:alpha. A) = (\x:alpha. B)
	 * ---->    ... => \/x:alpha. A  =  B 
	 *)	     
	 fun lamext() = 
	     let 
		 val (alpha, nr) = ( case (eqleft) of
				    (L((t,i),_)) => (t,i)
				  | _ => raise Error("Equality.sml Internal Nr. 4 \n") )
		 val (l, r) = ( case (eqleft,eqright) of
			       ( L(_,l) , L(_,r)) => (l,r)
			     | _ => raise Error("Equality.sml Internal Nr. 5 \n") )

		 val newterm = ( ALL((alpha, nr), EQ(l,r) ))
     
	     in
		 (newterm, goal)
	     end


	 fun funextORlam(alpha) =
	     case (eqleft,eqright) of
		 (L(_) , L(_)) => lamext()
	       | _             => funext(alpha) 
		      

	 (* [boolext]
	 * performs the Boolean Extensionality Rule
	 * Bsp.:
	 *          C => A:bool = B:bool
	 * ---->    C => (A => B) & (B => A )
	 *)	
	 fun boolext() = 
	     let
		val term1  = IMP(eqleft, eqright)
		val term2  = IMP(eqright, eqleft)
		val newterm = AND(term1, term2)
	     in
		 (newterm, goal)
	     end



	 (* Waehle wie die Gleichheit ersetzt werden soll *)
	 val (newterm, newgoal) = 
	     (  case typ_a of 
		    (TC "bool")      => boolext()
		  | (i as (TC _ ))   => leibniz(i)
		  | (TA(alpha,beta)) => funextORlam(alpha)
	     )

     in
	 (newterm, newgoal)
     end

| process t goal = (t,goal)

end (* of local open *)
end (* of structure Equality.sml *)
(* ==========================================================================================
                             Intro.sml
   ========================================================================================== *)

structure Intro= 
struct
(* Intro.sml implementiert alle Introduction-Regeln des Natural Deduction
 * calculus:
 *
 *  @  A => \/x . B[x]   -->  A => B[x]                 with x as a new parameter
 *  @  A => E x . B[x]   -->  A => B[? i]               (? i) is the i-th gamma variable
 *  @  A => B & C        -->  (1) A => B   (2) A => C   splits the goal in two subgoals
 *  @  A => ~B           -->  B & A => FALSE            let us assume B and show false (contradiction)
 *  @  A => B + C        -->  A => B  or  A => C        decide what to show ( <intro l> for B and <intro r> for C )
 *  @  A => ( B => C )   -->  A & B => C                extend the assumptions
 *  @  A => ( B = C )    -->  Equality substitutions vgl. "Equality.sml"
 *)

   
local open Data Goal in

fun main(g,modus) = let 
		val formel = Goal.get_first_goal(g)
		val new_goal = process formel g modus
	      in
		  (Goal.print_goal(new_goal);new_goal)
	      end

and process (IMP(left, ALL( (t,i), m))) goal _ =                          (* ALL RECHTS *)
     let 
	 (* holt den String namen der geb. Var i *)
	 val newname    = Goal.freeVarIntro(goal,i) 
	 (* hier werden ebenfalls die geb. Var ersetzt *)
	 val tempgoal = Goal.update_ftypeenv(goal, newname , t) 
	 val freemaps2i  = Goal.freemaps2i(tempgoal)

	 val tempgoal' = Goal.add_delta(tempgoal, freemaps2i newname)
	 val newright = ETT.down 0 (ETT.sub m 0 (V ((freemaps2i newname) + 1))) 1

	 val tempgoal'' = Goal.update_first_goal(tempgoal',IMP(left, newright ))
	 val newgoal = Goal.kaper_check(tempgoal'')     
     in
	 newgoal
     end

  | process (IMP(left, OR(ALL( (t,i), m), B))) goal _ =                    (* ALL RECHTS *)
     let 
	 (* holt den String namen der geb. Var i *)
	 val newname    = Goal.freeVarIntro(goal,i) 
	 (* hier werden ebenfalls die geb. Var ersetzt *)
	 val tempgoal = Goal.update_ftypeenv(goal, newname , t) 
	 val freemaps2i  = Goal.freemaps2i(tempgoal)

	 val tempgoal' = Goal.add_delta(tempgoal, freemaps2i newname)
	 val newright = ETT.down 0 (ETT.sub m 0 (V ((freemaps2i newname) + 1))) 1
	 val tempgoal''= Goal.update_first_goal(tempgoal',IMP(left, OR( newright , B)))
	 val newgoal = Goal.kaper_check(tempgoal'')
     in
	 newgoal
     end



  | process (IMP(left, oldright as (EX( (t,i), m)) )) goal modus =                              (* EXISTIERT RECHTS *)
     let
	 val nr_next_gamma = Goal.next_gamma_nr(goal)
	 val instance = GAMMA (t, nr_next_gamma)
	 val newgoal' = Goal.add_gamma(goal) 
	 val newright'= ETT.down 0 (ETT.sub m 0 (instance)) 1
	 val newright =( case modus of
			STANDART => newright'
		      | MORE => OR( newright', oldright ))
			     
	 val newgoal = Goal.update_first_goal(newgoal', IMP(left, newright) )
     in
	 newgoal
     end
  | process (IMP(left, oldright as (OR ( EX( (t,i), m) , B) ) )) goal modus =            (* EXISTIERT RECHTS *)
     let
	 val nr_next_gamma = Goal.next_gamma_nr(goal)
	 val instance = GAMMA (t, nr_next_gamma)
	 val newgoal' = Goal.add_gamma(goal)
	 val newright'= ETT.down 0 (ETT.sub m 0 (instance)) 1
	 val newright =( case modus of
			STANDART => OR(newright', B)
		      | MORE => OR( newright', oldright ))

	 val newgoal = Goal.update_first_goal(newgoal', IMP(left, newright) )
     in
	 newgoal
     end

  |process (IMP(left,AND(a,b))) goal _ =                                 (* AND Intro *)
     let
	 val g1 = Goal.update_first_goal(goal,IMP(left,a))
	 val g2 = Goal.split_goal(g1,IMP(left,b))
     in g2 
     end
  |process (IMP(left,OR(AND(a,b), B))) goal _ = 
     let
	 val g1 = Goal.update_first_goal(goal,IMP(left,OR(a,B)))
	 val g2 = Goal.split_goal(g1,IMP(left,OR(b,B)))
     in g2 
     end


  |process (IMP(left,NOT(a))) goal _ = 
     let 
	 val contradiction = 
	     (  case left of
		    (C "TRUE") => IMP(a, C "FALSE")
		  | _         => IMP(AND(a,left), C "FALSE" )
					     )
	 val g1 = Goal.update_first_goal(goal,contradiction)
     in g1 
     end
  |process (IMP(left,OR(NOT(a), B))) goal _ = 
     let 
	 val contradiction = 
	     (  case left of
		    (C "TRUE") => IMP(a, B)
		  | _         => IMP(AND(a,left), B )
					     )
	 val g1 = Goal.update_first_goal(goal,contradiction)
     in g1 
     end
 
 
  |process (IMP(left,IMP(a,b))) goal _ = 
     let 
	 val assume = 
	     (  case left of
		    (C "TRUE") => IMP(a, b)
		  | _         => IMP(AND(a,left), b)
			)
	 val g1 = Goal.update_first_goal(goal, assume )
     in g1 
     end
  |process (IMP(left,OR(IMP(a,b), B))) goal _ = 
     let 
	 val assume = 
	     (  case left of
		    (C "TRUE") => IMP(a, OR(b,B))
		  | _         => IMP(AND(a,left), OR(b,B))
			)
	 val g1 = Goal.update_first_goal(goal, assume )
     in g1 
     end
 
  |process (IMP(left,OR(OR(a,b), B))) goal _ =       (* Glaettet den OR Tree *)
     Goal.update_first_goal(goal,IMP(left,OR(b,OR(a, B))))

 
 

  |process (IMP(left, t as (EQ(a,b)) )) goal _ =
     let
	 (* Rufe Prozedur Equality.main auf
	  * gib den ersetzten Subterm zurueck
	  *)
	 val (newright, newgoal) = Equality.main(t, goal)  
     in
	 Goal.update_first_goal(newgoal, IMP(left, newright))
     end
	 
 |process (IMP(left, OR(t as (EQ(a,b)),B) )) goal _ =
     let
	 (* Rufe Prozedur Equality.main auf
	  * gib den ersetzten Subterm zurueck
	  *)
	 val (newright, newgoal) = Equality.main(t, goal)  
     in
	 Goal.update_first_goal(newgoal, IMP(left, OR(newright,B) ))
     end



(* TRANSLATE DEFINITION CHECK 
 *
 * Handelt es sich bei der Conklusion um eine Definition des users,
 * wird versucht, den Ausdruck der Definition einzusetzen: 
 * A => B[D]  and B contains a Definition (D=C) --> A => B[D:=C] 
 *)
  |process (IMP(left, right )) goal modus =   
     let
	 val (newright, manipulated) = 
	     case right of
		 OR(right',B) =>   
		     if (Goal.check_def(goal, right')) then (OR(Goal.translate_def(goal,right'),B), true)
		     else (right,false)
	       | _ => 
			 if (Goal.check_def(goal, right)) then (Goal.translate_def(goal,right), true)
			 else (right,false)

	 val newgoal = if manipulated then Goal.update_first_goal(goal, IMP(left, newright)) 
                       else goal
     in 
	 (* Damit kein Zyklus entsteht *)
	 if manipulated then 
	     let
		 val newfirst = Goal.get_first_goal(newgoal)
	     in 
		 process newfirst newgoal modus
	     end
	 else newgoal   
     end

  |process _ goal _ = goal

end (* of local open *)





end (* of structure Intro.sml *)
(* ==========================================================================================
                             Parser.sml
   ========================================================================================== *)

structure Parser= 
struct
local
in
datatype token =
    TAEQUI
  | TALL
  | TAND
  | TAPPLY
  | TAXIOM
  | TBETA
  | TCOMMA
  | TCON
  | TCONST of string
  | TCUT
  | TDASHARROW
  | TDEF
  | TDOT
  | TDOUBLEPT
  | TELIM
  | TEOF
  | TEOL
  | TEQ
  | TETA
  | TEXISTS
  | TEXIT
  | TGOAL
  | THELP
  | TIMP
  | TINSERT
  | TINSTANCE
  | TINTRO
  | TJUNK
  | TLAM
  | TLAMBDA
  | TLATEX
  | TLEMMA
  | TLPAR
  | TMOVE
  | TNAME of string
  | TNOT
  | TNR of int
  | TOR
  | TPOS
  | TPRINT
  | TREF
  | TREMOVE
  | TRESET
  | TREW
  | TRPAR
  | TSAVE
  | TSEMI
  | TSHIFT
  | TSHOWLABELS
  | TSIMPLIFY
  | TSYM
  | TTRANS
  | TTYPE
  | TUNDO
  | TVAR
  | TWITH
end;

open Obj Parsing;
prim_val vector_ : int -> 'a -> 'a Vector.vector = 2 "make_vect";
prim_val update_ : 'a Vector.vector -> int -> 'a -> unit = 3 "set_vect_item";

open Data;

fun elim' ("more") = CmdS(ELIMCMD(MORE))
  | elim' s = raise Error "Parser Error: <elim [more]>\n"

fun intro' ("more") = CmdS(INTROCMD(MORE))
  | intro' s = raise Error "Parser Error: <intro [more]>\n"

fun move' ("r") = CmdS(MOVECMD (RIGHT))
  | move' ("l") = CmdS(MOVECMD (LEFT ))
  | move' s     = raise Error "Parser Error: <move r/l > to move from Right/Left to Left/Right \n" 

val shiftmsg =  ( "Parser Error: <shift (l/r) (u/d) [nr] > \n "
				   ^ "             <shift goal [nr]>\n" 
				   ^ "to cyclic shift premises/conclusions or \n"
				   ^ "goals by [nr] positions up/down \n" )

fun shift' ("l") (("u"), n) = CmdS(SHIFTCMD(LEFT,UP,n))
 |  shift' ("l") (("d"), n) = CmdS(SHIFTCMD(LEFT,DOWN,n))
 |  shift' ("r") (("u"), n) = CmdS(SHIFTCMD(RIGHT,UP,n))
 |  shift' ("r") (("d"), n) = CmdS(SHIFTCMD(RIGHT,DOWN,n))
 |  shift' s     n = raise Error shiftmsg

fun goalshift' ("goals") n  = CmdS(GOALSHIFTCMD(n))
 |  goalshift' s n = raise Error shiftmsg

fun remove' ("l" , n ) = CmdS(REMOVECMD(LEFT,n))
 |  remove' ("r" , n ) = CmdS(REMOVECMD(RIGHT,n))
 |  remove' s          = raise Error "Parser Error: <remove [l/r] (nr)> to remove nr on left/right hand side\n"

(*
fun apply' (nr, "l") = CmdS(APPLYCMD(REW(nr, LEFT)))
 |  apply' (nr, "r") = CmdS(APPLYCMD(REW(nr, RIGHT)))
 |  apply' _ = raise Error "Parser Error: <apply (NR|NAME) [NR|(LEFT|RIGHT)] > to apply some premisse \n"
*)

(* Rewriting *
 * Wenn rew [..] with <name> (l/r)  ==> wende das externe Lemma/Axiom <name>  *
 *                                        zum Rewriting an.                  *
 * Wenn rew [..] with <nr>   (l/r)  ==> wende die Praemisse <nr> zur Rew an 
 *
 * Wenn rew [..] with (A=B)         ==> generiere Lemma (A=B) und versuche zu rewriten ! 
 *)

fun rew_axiom' xs (name, "l") = CmdEqS(REWEQ(xs,(LAXIOM(name,LEFT))))
 |  rew_axiom' xs (name, "r") = CmdEqS(REWEQ(xs,(LAXIOM(name,RIGHT))))
 |  rew_axiom' _  _ = raise Error "Parser Error : <rew (1,2,1,...) with name (l/r)> to apply rewriting"	 

fun rew_prem' xs (nr, "l") = CmdEqS(REWEQ(xs,(PREMISSE(nr,LEFT))))
 |  rew_prem' xs (nr, "r") = CmdEqS(REWEQ(xs,(PREMISSE(nr,RIGHT))))
 |  rew_prem' _ _ = raise Error "Parser Error : <rew (1,2,1,...) with name (l/r)> to apply rewriting"	 

fun symmetric' ("l") = CmdEqS(SYMEQ(LEFT))
 |  symmetric' ("r") = CmdEqS(SYMEQ(RIGHT))
 |  symmetric' _ = raise Error "Parser Error : <sym [l/r]> to apply symmetric rule"

fun insert' ("l") (n,s) = CmdEqS(INSERTEQ(LEFT,(n,s)))
 |  insert' ("r") (n,s) = CmdEqS(INSERTEQ(RIGHT,(n,s)))
 |  insert' _ _ = raise Error "Parser Error: <insert name = formel [l/r]> to insert a formula for variable name"


(* checkt die Positionsliste *)
fun checkpos nil = nil
 |  checkpos (x::xs) = x::(checkpos' xs)
and checkpos' nil = nil
 |  checkpos' ((LA _)::xs) = raise Error "Please use Positions [Label](1|2)*"
 |  checkpos' (x::xs) = x::(checkpos' xs)
   

fun undo' n = CmdS(UNDOCMD n)


(* Line 78, file Parser.sml *)
val yytransl = #[
  257 (* TAEQUI *),
  258 (* TALL *),
  259 (* TAND *),
  260 (* TAPPLY *),
  261 (* TAXIOM *),
  262 (* TBETA *),
  263 (* TCOMMA *),
  264 (* TCON *),
  265 (* TCONST *),
  266 (* TCUT *),
  267 (* TDASHARROW *),
  268 (* TDEF *),
  269 (* TDOT *),
  270 (* TDOUBLEPT *),
  271 (* TELIM *),
  272 (* TEOF *),
  273 (* TEOL *),
  274 (* TEQ *),
  275 (* TETA *),
  276 (* TEXISTS *),
  277 (* TEXIT *),
  278 (* TGOAL *),
  279 (* THELP *),
  280 (* TIMP *),
  281 (* TINSERT *),
  282 (* TINSTANCE *),
  283 (* TINTRO *),
  284 (* TJUNK *),
  285 (* TLAM *),
  286 (* TLAMBDA *),
  287 (* TLATEX *),
  288 (* TLEMMA *),
  289 (* TLPAR *),
  290 (* TMOVE *),
  291 (* TNAME *),
  292 (* TNOT *),
  293 (* TNR *),
  294 (* TOR *),
  295 (* TPOS *),
  296 (* TPRINT *),
  297 (* TREF *),
  298 (* TREMOVE *),
  299 (* TRESET *),
  300 (* TREW *),
  301 (* TRPAR *),
  302 (* TSAVE *),
  303 (* TSEMI *),
  304 (* TSHIFT *),
  305 (* TSHOWLABELS *),
  306 (* TSIMPLIFY *),
  307 (* TSYM *),
  308 (* TTRANS *),
  309 (* TTYPE *),
  310 (* TUNDO *),
  311 (* TVAR *),
  312 (* TWITH *),
    0];

val yylhs = "\255\255\
\\001\000\001\000\001\000\001\000\001\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
\\003\000\003\000\003\000\003\000\003\000\003\000\003\000\004\000\
\\004\000\005\000\005\000\006\000\006\000\007\000\007\000\008\000\
\\008\000\009\000\009\000\010\000\010\000\011\000\011\000\011\000\
\\012\000\012\000\013\000\013\000\014\000\014\000\014\000\014\000\
\\014\000\000\000";

val yylen = "\002\000\
\\003\000\002\000\002\000\001\000\001\000\001\000\002\000\004\000\
\\004\000\002\000\005\000\005\000\005\000\004\000\003\000\003\000\
\\002\000\002\000\002\000\001\000\002\000\004\000\001\000\001\000\
\\002\000\002\000\004\000\004\000\004\000\005\000\003\000\003\000\
\\006\000\006\000\007\000\005\000\005\000\002\000\001\000\002\000\
\\003\000\001\000\002\000\002\000\003\000\001\000\002\000\001\000\
\\001\000\001\000\001\000\001\000\002\000\001\000\002\000\001\000\
\\001\000\004\000\006\000\004\000\004\000\006\000\006\000\001\000\
\\003\000\001\000\003\000\001\000\003\000\001\000\003\000\001\000\
\\003\000\001\000\002\000\001\000\002\000\001\000\001\000\003\000\
\\001\000\003\000\001\000\003\000\003\000\003\000\002\000\002\000\
\\001\000\002\000";

val yydefred = "\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\079\000\000\000\
\\000\000\000\000\005\000\004\000\000\000\056\000\000\000\050\000\
\\000\000\000\000\000\000\048\000\000\000\000\000\000\000\000\000\
\\000\000\078\000\000\000\000\000\051\000\023\000\000\000\054\000\
\\000\000\000\000\000\000\052\000\049\000\000\000\000\000\000\000\
\\000\000\000\000\090\000\000\000\006\000\057\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\076\000\000\000\040\000\000\000\
\\000\000\000\000\021\000\000\000\047\000\000\000\007\000\000\000\
\\000\000\043\000\000\000\053\000\000\000\000\000\018\000\075\000\
\\038\000\000\000\044\000\000\000\000\000\055\000\000\000\025\000\
\\026\000\010\000\019\000\000\000\003\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\077\000\000\000\000\000\041\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\080\000\045\000\000\000\000\000\089\000\000\000\031\000\
\\032\000\000\000\000\000\000\000\000\000\016\000\000\000\001\000\
\\065\000\067\000\069\000\071\000\073\000\060\000\000\000\083\000\
\\000\000\000\000\000\000\009\000\000\000\061\000\000\000\027\000\
\\022\000\058\000\000\000\000\000\000\000\088\000\000\000\087\000\
\\000\000\000\000\028\000\029\000\014\000\008\000\000\000\000\000\
\\000\000\013\000\011\000\000\000\000\000\012\000\086\000\085\000\
\\036\000\037\000\000\000\000\000\000\000\030\000\084\000\062\000\
\\082\000\063\000\059\000\000\000\033\000\034\000\035\000";

val yydgoto = "\002\000\
\\043\000\044\000\045\000\046\000\047\000\048\000\049\000\050\000\
\\051\000\052\000\053\000\129\000\130\000\111\000";

val yysindex = "\005\000\
\\013\255\000\000\222\254\229\254\235\254\247\254\000\000\131\255\
\\002\255\025\255\000\000\000\000\036\255\000\000\131\255\000\000\
\\042\255\043\255\063\255\000\000\068\255\092\255\096\255\131\255\
\\102\255\000\000\144\255\106\255\000\000\000\000\232\254\000\000\
\\227\254\112\255\127\255\000\000\000\000\139\255\131\255\141\255\
\\148\255\143\255\000\000\034\255\000\000\000\000\169\255\185\255\
\\164\255\151\255\187\255\130\255\000\000\006\255\000\000\154\255\
\\178\255\179\255\000\000\180\255\000\000\168\255\000\000\177\255\
\\181\255\000\000\170\255\000\000\182\255\152\255\000\000\000\000\
\\000\000\163\255\000\000\089\255\123\255\000\000\041\255\000\000\
\\000\000\000\000\000\000\184\255\000\000\186\255\144\255\144\255\
\\144\255\144\255\144\255\000\000\131\255\124\255\000\000\183\255\
\\124\255\188\255\131\255\124\255\131\255\131\255\131\255\124\255\
\\189\255\000\000\000\000\252\254\005\255\000\000\146\255\000\000\
\\000\000\131\255\173\255\174\255\167\255\000\000\124\255\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\124\255\000\000\
\\192\255\199\255\131\255\000\000\131\255\000\000\198\255\000\000\
\\000\000\000\000\200\255\131\255\089\255\000\000\089\255\000\000\
\\135\255\171\255\000\000\000\000\000\000\000\000\172\255\131\255\
\\124\255\000\000\000\000\131\255\131\255\000\000\000\000\000\000\
\\000\000\000\000\131\255\190\255\191\255\000\000\000\000\000\000\
\\000\000\000\000\000\000\175\255\000\000\000\000\000\000";

val yyrindex = "\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\055\255\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\057\255\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\065\255\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\071\255\000\000\000\000\
\\088\255\000\000\000\000\000\000\000\000\000\000\052\255\128\255\
\\069\255\091\255\083\255\072\255\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\103\255\000\000\
\\000\000\000\000\000\000\000\000\000\000\212\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\105\255\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\066\255\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000";

val yygindex = "\000\000\
\\000\000\000\000\248\255\127\000\130\000\126\000\129\000\131\000\
\\194\000\000\000\171\000\161\255\000\000\028\000";

val YYTABLESIZE = 226;
val yytable = "\059\000\
\\054\000\132\000\141\000\076\000\135\000\001\000\063\000\055\000\
\\139\000\056\000\074\000\143\000\075\000\057\000\003\000\070\000\
\\004\000\005\000\093\000\094\000\006\000\007\000\008\000\150\000\
\\009\000\058\000\077\000\010\000\011\000\012\000\081\000\151\000\
\\013\000\014\000\015\000\016\000\060\000\017\000\018\000\019\000\
\\142\000\020\000\021\000\022\000\023\000\024\000\025\000\026\000\
\\027\000\144\000\085\000\028\000\029\000\030\000\031\000\032\000\
\\033\000\169\000\034\000\061\000\035\000\036\000\037\000\038\000\
\\039\000\040\000\041\000\042\000\064\000\068\000\062\000\046\000\
\\074\000\042\000\074\000\117\000\064\000\118\000\081\000\065\000\
\\086\000\039\000\081\000\072\000\126\000\068\000\068\000\024\000\
\\074\000\074\000\134\000\070\000\136\000\137\000\138\000\074\000\
\\064\000\066\000\064\000\072\000\072\000\046\000\067\000\042\000\
\\020\000\146\000\072\000\070\000\070\000\074\000\081\000\039\000\
\\081\000\068\000\070\000\068\000\074\000\024\000\074\000\017\000\
\\072\000\015\000\154\000\108\000\155\000\109\000\068\000\072\000\
\\112\000\072\000\069\000\158\000\003\000\110\000\020\000\070\000\
\\071\000\070\000\007\000\007\000\161\000\113\000\073\000\168\000\
\\066\000\066\000\078\000\170\000\171\000\017\000\013\000\015\000\
\\007\000\162\000\172\000\114\000\127\000\115\000\128\000\116\000\
\\021\000\079\000\024\000\024\000\026\000\026\000\027\000\163\000\
\\159\000\164\000\160\000\165\000\066\000\080\000\066\000\082\000\
\\024\000\084\000\026\000\027\000\099\000\100\000\103\000\104\000\
\\083\000\088\000\087\000\089\000\090\000\091\000\095\000\096\000\
\\097\000\098\000\101\000\105\000\106\000\119\000\102\000\107\000\
\\131\000\145\000\120\000\149\000\152\000\133\000\140\000\147\000\
\\148\000\153\000\156\000\002\000\157\000\121\000\123\000\166\000\
\\167\000\122\000\124\000\175\000\072\000\125\000\092\000\000\000\
\\173\000\174\000";

val yycheck = "\008\000\
\\035\001\097\000\007\001\033\001\100\000\001\000\015\000\035\001\
\\104\000\037\001\035\001\007\001\037\001\035\001\002\001\024\000\
\\004\001\005\001\013\001\014\001\008\001\009\001\010\001\119\000\
\\012\001\035\001\056\001\015\001\016\001\017\001\039\000\127\000\
\\020\001\021\001\022\001\023\001\035\001\025\001\026\001\027\001\
\\045\001\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\\036\001\045\001\017\001\039\001\040\001\041\001\042\001\043\001\
\\044\001\153\000\046\001\035\001\048\001\049\001\050\001\051\001\
\\052\001\053\001\054\001\055\001\017\001\001\001\035\001\017\001\
\\001\001\017\001\003\001\035\001\035\001\037\001\013\001\037\001\
\\047\001\017\001\017\001\001\001\093\000\017\001\018\001\017\001\
\\017\001\018\001\099\000\001\001\101\000\102\000\103\000\024\001\
\\045\001\035\001\047\001\017\001\018\001\047\001\035\001\047\001\
\\017\001\114\000\024\001\017\001\018\001\038\001\045\001\047\001\
\\047\001\045\001\024\001\047\001\045\001\047\001\047\001\017\001\
\\038\001\017\001\131\000\035\001\133\000\037\001\035\001\045\001\
\\006\001\047\001\035\001\140\000\002\001\045\001\047\001\045\001\
\\035\001\047\001\009\001\009\001\006\001\019\001\037\001\152\000\
\\017\001\018\001\035\001\156\000\157\000\047\001\020\001\047\001\
\\009\001\019\001\163\000\033\001\033\001\035\001\035\001\037\001\
\\030\001\035\001\033\001\033\001\035\001\035\001\036\001\033\001\
\\141\000\035\001\143\000\037\001\045\001\035\001\047\001\035\001\
\\033\001\035\001\035\001\036\001\013\001\014\001\013\001\014\001\
\\037\001\001\001\018\001\024\001\038\001\003\001\037\001\014\001\
\\014\001\014\001\018\001\014\001\045\001\014\001\018\001\037\001\
\\018\001\056\001\017\001\037\001\013\001\018\001\018\001\035\001\
\\035\001\011\001\013\001\000\000\013\001\087\000\089\000\045\001\
\\045\001\088\000\090\000\045\001\027\000\091\000\052\000\255\255\
\\035\001\035\001";

val yyact = vector_ 91 (fn () => ((raise Fail "parser") : obj));
(* Rule 1, file Parser.grm, line 107 *)
val _ = update_ yyact 1
(fn () => repr(let
val d__1__ = peekVal 2 : Data.parseexpr
in
( (d__1__) ) end : Data.parseexpr))
;
(* Rule 2, file Parser.grm, line 109 *)
val _ = update_ yyact 2
(fn () => repr(let
val d__1__ = peekVal 1 : Data.parseexpr
in
( (d__1__) ) end : Data.parseexpr))
;
(* Rule 3, file Parser.grm, line 110 *)
val _ = update_ yyact 3
(fn () => repr(let
val d__1__ = peekVal 1 : Data.parseexpr
in
( (d__1__) ) end : Data.parseexpr))
;
(* Rule 4, file Parser.grm, line 111 *)
val _ = update_ yyact 4
(fn () => repr(let
in
( EmptyS ) end : Data.parseexpr))
;
(* Rule 5, file Parser.grm, line 112 *)
val _ = update_ yyact 5
(fn () => repr(let
in
( FileEndS ) end : Data.parseexpr))
;
(* Rule 6, file Parser.grm, line 116 *)
val _ = update_ yyact 6
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( ForS((d__1__)) ) end : Data.parseexpr))
;
(* Rule 7, file Parser.grm, line 117 *)
val _ = update_ yyact 7
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( GoalS((d__2__)) ) end : Data.parseexpr))
;
(* Rule 8, file Parser.grm, line 118 *)
val _ = update_ yyact 8
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.ty
in
( LetVarS((d__2__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 9, file Parser.grm, line 119 *)
val _ = update_ yyact 9
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.ty
in
( LetConstS((d__2__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 10, file Parser.grm, line 120 *)
val _ = update_ yyact 10
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( TypS((d__2__)) ) end : Data.parseexpr))
;
(* Rule 11, file Parser.grm, line 121 *)
val _ = update_ yyact 11
(fn () => repr(let
val d__2__ = peekVal 3 : string
val d__5__ = peekVal 0 : Data.sourceexpr
in
( DefS((d__2__), (d__5__), CLASSIC) ) end : Data.parseexpr))
;
(* Rule 12, file Parser.grm, line 122 *)
val _ = update_ yyact 12
(fn () => repr(let
val d__2__ = peekVal 3 : string
val d__5__ = peekVal 0 : Data.sourceexpr
in
( DefS((d__2__), (d__5__), LEMMA) ) end : Data.parseexpr))
;
(* Rule 13, file Parser.grm, line 123 *)
val _ = update_ yyact 13
(fn () => repr(let
val d__2__ = peekVal 3 : string
val d__5__ = peekVal 0 : Data.sourceexpr
in
( DefS((d__2__), (d__5__), AXIOM) ) end : Data.parseexpr))
;
(* Rule 14, file Parser.grm, line 125 *)
val _ = update_ yyact 14
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__3__ = peekVal 1 : string
val d__4__ = peekVal 0 : int
in
( shift' (d__2__) ((d__3__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 15, file Parser.grm, line 126 *)
val _ = update_ yyact 15
(fn () => repr(let
val d__2__ = peekVal 1 : string
val d__3__ = peekVal 0 : string
in
( shift' (d__2__) ((d__3__), 1) ) end : Data.parseexpr))
;
(* Rule 16, file Parser.grm, line 127 *)
val _ = update_ yyact 16
(fn () => repr(let
val d__2__ = peekVal 1 : string
val d__3__ = peekVal 0 : int
in
( goalshift' (d__2__) (d__3__)  ) end : Data.parseexpr))
;
(* Rule 17, file Parser.grm, line 128 *)
val _ = update_ yyact 17
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( goalshift' (d__2__) 1   ) end : Data.parseexpr))
;
(* Rule 18, file Parser.grm, line 129 *)
val _ = update_ yyact 18
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( move' (d__2__) ) end : Data.parseexpr))
;
(* Rule 19, file Parser.grm, line 130 *)
val _ = update_ yyact 19
(fn () => repr(let
val d__2__ = peekVal 0 : int
in
( undo' (d__2__) ) end : Data.parseexpr))
;
(* Rule 20, file Parser.grm, line 131 *)
val _ = update_ yyact 20
(fn () => repr(let
in
( undo' 1  ) end : Data.parseexpr))
;
(* Rule 21, file Parser.grm, line 133 *)
val _ = update_ yyact 21
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( CmdS(CUTCMD((d__2__))) ) end : Data.parseexpr))
;
(* Rule 22, file Parser.grm, line 134 *)
val _ = update_ yyact 22
(fn () => repr(let
val d__2__ = peekVal 2 : int
val d__4__ = peekVal 0 : Data.sourceexpr
in
( CmdS(INSTANCECMD((d__2__),(d__4__))) ) end : Data.parseexpr))
;
(* Rule 23, file Parser.grm, line 138 *)
val _ = update_ yyact 23
(fn () => repr(let
in
( CmdEqS(REFEQ) ) end : Data.parseexpr))
;
(* Rule 24, file Parser.grm, line 139 *)
val _ = update_ yyact 24
(fn () => repr(let
in
( CmdEqS(SYMEQ(RIGHT)) ) end : Data.parseexpr))
;
(* Rule 25, file Parser.grm, line 140 *)
val _ = update_ yyact 25
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( symmetric'((d__2__)) ) end : Data.parseexpr))
;
(* Rule 26, file Parser.grm, line 141 *)
val _ = update_ yyact 26
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( CmdEqS(TRANSEQ((d__2__))) ) end : Data.parseexpr))
;
(* Rule 27, file Parser.grm, line 143 *)
val _ = update_ yyact 27
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( CmdEqS(INSERTEQ(RIGHT, ((d__2__),(d__4__)))) ) end : Data.parseexpr))
;
(* Rule 28, file Parser.grm, line 144 *)
val _ = update_ yyact 28
(fn () => repr(let
val d__3__ = peekVal 1 : string
val d__4__ = peekVal 0 : string
in
( rew_axiom' nil ((d__3__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 29, file Parser.grm, line 145 *)
val _ = update_ yyact 29
(fn () => repr(let
val d__3__ = peekVal 1 : int
val d__4__ = peekVal 0 : string
in
( rew_prem'  nil ((d__3__),(d__4__)) ) end : Data.parseexpr))
;
(* Rule 30, file Parser.grm, line 146 *)
val _ = update_ yyact 30
(fn () => repr(let
val d__4__ = peekVal 1 : Data.sourceexpr
in
( CmdEqS(REWEQ(nil, EQUALITY((d__4__)))) ) end : Data.parseexpr))
;
(* Rule 31, file Parser.grm, line 147 *)
val _ = update_ yyact 31
(fn () => repr(let
in
( CmdEqS(REWEQ(nil, BETA)) ) end : Data.parseexpr))
;
(* Rule 32, file Parser.grm, line 148 *)
val _ = update_ yyact 32
(fn () => repr(let
in
( CmdEqS(REWEQ(nil, ETA)) ) end : Data.parseexpr))
;
(* Rule 33, file Parser.grm, line 149 *)
val _ = update_ yyact 33
(fn () => repr(let
val d__3__ = peekVal 3 : Data.pos list
val d__5__ = peekVal 1 : string
val d__6__ = peekVal 0 : string
in
( rew_axiom' (checkpos((d__3__)))  ((d__5__),(d__6__)) ) end : Data.parseexpr))
;
(* Rule 34, file Parser.grm, line 150 *)
val _ = update_ yyact 34
(fn () => repr(let
val d__3__ = peekVal 3 : Data.pos list
val d__5__ = peekVal 1 : int
val d__6__ = peekVal 0 : string
in
( rew_prem'  (checkpos((d__3__)))  ((d__5__),(d__6__)) ) end : Data.parseexpr))
;
(* Rule 35, file Parser.grm, line 151 *)
val _ = update_ yyact 35
(fn () => repr(let
val d__3__ = peekVal 4 : Data.pos list
val d__6__ = peekVal 1 : Data.sourceexpr
in
( CmdEqS(REWEQ((checkpos((d__3__))), EQUALITY((d__6__)))) ) end : Data.parseexpr))
;
(* Rule 36, file Parser.grm, line 152 *)
val _ = update_ yyact 36
(fn () => repr(let
val d__3__ = peekVal 2 : Data.pos list
in
( CmdEqS(REWEQ((checkpos((d__3__))), BETA)) ) end : Data.parseexpr))
;
(* Rule 37, file Parser.grm, line 153 *)
val _ = update_ yyact 37
(fn () => repr(let
val d__3__ = peekVal 2 : Data.pos list
in
( CmdEqS(REWEQ((checkpos((d__3__))), ETA)) ) end : Data.parseexpr))
;
(* Rule 38, file Parser.grm, line 155 *)
val _ = update_ yyact 38
(fn () => repr(let
val d__2__ = peekVal 0 : int
in
( CmdS(POSCMD (d__2__)) ) end : Data.parseexpr))
;
(* Rule 39, file Parser.grm, line 156 *)
val _ = update_ yyact 39
(fn () => repr(let
in
( CmdS(POSCMD 0)  ) end : Data.parseexpr))
;
(* Rule 40, file Parser.grm, line 158 *)
val _ = update_ yyact 40
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( CmdS(APPLYCMD(DEFINITION (d__2__))) ) end : Data.parseexpr))
;
(* Rule 41, file Parser.grm, line 160 *)
val _ = update_ yyact 41
(fn () => repr(let
val d__2__ = peekVal 1 : int
val d__3__ = peekVal 0 : int
in
( CmdS(APPLYCMD(MODUSPONENS((d__2__),(d__3__)))) ) end : Data.parseexpr))
;
(* Rule 42, file Parser.grm, line 161 *)
val _ = update_ yyact 42
(fn () => repr(let
in
( CmdS(INTROCMD(STANDART)) ) end : Data.parseexpr))
;
(* Rule 43, file Parser.grm, line 162 *)
val _ = update_ yyact 43
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( intro' (d__2__) ) end : Data.parseexpr))
;
(* Rule 44, file Parser.grm, line 163 *)
val _ = update_ yyact 44
(fn () => repr(let
val d__2__ = peekVal 0 : int
in
( CmdS(REMOVECMD (LEFT,(d__2__)))) end : Data.parseexpr))
;
(* Rule 45, file Parser.grm, line 164 *)
val _ = update_ yyact 45
(fn () => repr(let
val d__2__ = peekVal 1 : string
val d__3__ = peekVal 0 : int
in
( remove' ((d__2__), (d__3__)) ) end : Data.parseexpr))
;
(* Rule 46, file Parser.grm, line 165 *)
val _ = update_ yyact 46
(fn () => repr(let
in
( CmdS(ELIMCMD(STANDART)) ) end : Data.parseexpr))
;
(* Rule 47, file Parser.grm, line 166 *)
val _ = update_ yyact 47
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( elim' (d__2__) ) end : Data.parseexpr))
;
(* Rule 48, file Parser.grm, line 167 *)
val _ = update_ yyact 48
(fn () => repr(let
in
( CmdS(LAMCMD) ) end : Data.parseexpr))
;
(* Rule 49, file Parser.grm, line 168 *)
val _ = update_ yyact 49
(fn () => repr(let
in
( CmdS(SIMPLIFYCMD) ) end : Data.parseexpr))
;
(* Rule 50, file Parser.grm, line 169 *)
val _ = update_ yyact 50
(fn () => repr(let
in
( CmdS(HELPCMD) ) end : Data.parseexpr))
;
(* Rule 51, file Parser.grm, line 170 *)
val _ = update_ yyact 51
(fn () => repr(let
in
( CmdS(PRINTCMD) ) end : Data.parseexpr))
;
(* Rule 52, file Parser.grm, line 171 *)
val _ = update_ yyact 52
(fn () => repr(let
in
( CmdS(SHOWLABELSCMD) ) end : Data.parseexpr))
;
(* Rule 53, file Parser.grm, line 172 *)
val _ = update_ yyact 53
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( CmdS(LATEXCMD (d__2__)) ) end : Data.parseexpr))
;
(* Rule 54, file Parser.grm, line 173 *)
val _ = update_ yyact 54
(fn () => repr(let
in
( CmdS(RESETCMD) ) end : Data.parseexpr))
;
(* Rule 55, file Parser.grm, line 174 *)
val _ = update_ yyact 55
(fn () => repr(let
val d__2__ = peekVal 0 : string
in
( SaveS (d__2__) ) end : Data.parseexpr))
;
(* Rule 56, file Parser.grm, line 175 *)
val _ = update_ yyact 56
(fn () => repr(let
in
( CmdS(EXITCMD) ) end : Data.parseexpr))
;
(* Rule 57, file Parser.grm, line 179 *)
val _ = update_ yyact 57
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 58, file Parser.grm, line 180 *)
val _ = update_ yyact 58
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( LamS(((d__2__), TC "not_specified"), (d__4__)) ) end : Data.sourceexpr))
;
(* Rule 59, file Parser.grm, line 181 *)
val _ = update_ yyact 59
(fn () => repr(let
val d__2__ = peekVal 4 : string
val d__4__ = peekVal 2 : Data.ty
val d__6__ = peekVal 0 : Data.sourceexpr
in
( LamS(((d__2__),(d__4__)),(d__6__)) ) end : Data.sourceexpr))
;
(* Rule 60, file Parser.grm, line 182 *)
val _ = update_ yyact 60
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( AllS(((d__2__), TC "not_specified"), (d__4__)) ) end : Data.sourceexpr))
;
(* Rule 61, file Parser.grm, line 183 *)
val _ = update_ yyact 61
(fn () => repr(let
val d__2__ = peekVal 2 : string
val d__4__ = peekVal 0 : Data.sourceexpr
in
( ExS (((d__2__), TC "not_specified"), (d__4__)) ) end : Data.sourceexpr))
;
(* Rule 62, file Parser.grm, line 184 *)
val _ = update_ yyact 62
(fn () => repr(let
val d__2__ = peekVal 4 : string
val d__4__ = peekVal 2 : Data.ty
val d__6__ = peekVal 0 : Data.sourceexpr
in
( AllS(((d__2__),(d__4__)),(d__6__)) ) end : Data.sourceexpr))
;
(* Rule 63, file Parser.grm, line 185 *)
val _ = update_ yyact 63
(fn () => repr(let
val d__2__ = peekVal 4 : string
val d__4__ = peekVal 2 : Data.ty
val d__6__ = peekVal 0 : Data.sourceexpr
in
( ExS (((d__2__),(d__4__)),(d__6__)) ) end : Data.sourceexpr))
;
(* Rule 64, file Parser.grm, line 189 *)
val _ = update_ yyact 64
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 65, file Parser.grm, line 190 *)
val _ = update_ yyact 65
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( EqS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 66, file Parser.grm, line 193 *)
val _ = update_ yyact 66
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 67, file Parser.grm, line 194 *)
val _ = update_ yyact 67
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( AndS(ImpS((d__1__),(d__3__)),ImpS((d__3__),(d__1__))) ) end : Data.sourceexpr))
;
(* Rule 68, file Parser.grm, line 197 *)
val _ = update_ yyact 68
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 69, file Parser.grm, line 198 *)
val _ = update_ yyact 69
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( ImpS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 70, file Parser.grm, line 201 *)
val _ = update_ yyact 70
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 71, file Parser.grm, line 202 *)
val _ = update_ yyact 71
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( OrS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 72, file Parser.grm, line 205 *)
val _ = update_ yyact 72
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 73, file Parser.grm, line 206 *)
val _ = update_ yyact 73
(fn () => repr(let
val d__1__ = peekVal 2 : Data.sourceexpr
val d__3__ = peekVal 0 : Data.sourceexpr
in
( AndS((d__1__),(d__3__)) ) end : Data.sourceexpr))
;
(* Rule 74, file Parser.grm, line 209 *)
val _ = update_ yyact 74
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 75, file Parser.grm, line 210 *)
val _ = update_ yyact 75
(fn () => repr(let
val d__2__ = peekVal 0 : Data.sourceexpr
in
( NotS((d__2__)) ) end : Data.sourceexpr))
;
(* Rule 76, file Parser.grm, line 213 *)
val _ = update_ yyact 76
(fn () => repr(let
val d__1__ = peekVal 0 : Data.sourceexpr
in
( (d__1__) ) end : Data.sourceexpr))
;
(* Rule 77, file Parser.grm, line 214 *)
val _ = update_ yyact 77
(fn () => repr(let
val d__1__ = peekVal 1 : Data.sourceexpr
val d__2__ = peekVal 0 : Data.sourceexpr
in
( AppS((d__1__), (d__2__)) ) end : Data.sourceexpr))
;
(* Rule 78, file Parser.grm, line 217 *)
val _ = update_ yyact 78
(fn () => repr(let
val d__1__ = peekVal 0 : string
in
( VarS (d__1__) ) end : Data.sourceexpr))
;
(* Rule 79, file Parser.grm, line 218 *)
val _ = update_ yyact 79
(fn () => repr(let
val d__1__ = peekVal 0 : string
in
( CstS (d__1__) ) end : Data.sourceexpr))
;
(* Rule 80, file Parser.grm, line 219 *)
val _ = update_ yyact 80
(fn () => repr(let
val d__2__ = peekVal 1 : Data.sourceexpr
in
( (d__2__) ) end : Data.sourceexpr))
;
(* Rule 81, file Parser.grm, line 222 *)
val _ = update_ yyact 81
(fn () => repr(let
val d__1__ = peekVal 0 : Data.ty
in
( (d__1__) ) end : Data.ty))
;
(* Rule 82, file Parser.grm, line 223 *)
val _ = update_ yyact 82
(fn () => repr(let
val d__1__ = peekVal 2 : Data.ty
val d__3__ = peekVal 0 : Data.ty
in
( TA((d__1__),(d__3__)) ) end : Data.ty))
;
(* Rule 83, file Parser.grm, line 226 *)
val _ = update_ yyact 83
(fn () => repr(let
val d__1__ = peekVal 0 : string
in
( TC((d__1__)) ) end : Data.ty))
;
(* Rule 84, file Parser.grm, line 227 *)
val _ = update_ yyact 84
(fn () => repr(let
val d__2__ = peekVal 1 : Data.ty
in
( (d__2__) ) end : Data.ty))
;
(* Rule 85, file Parser.grm, line 231 *)
val _ = update_ yyact 85
(fn () => repr(let
val d__1__ = peekVal 2 : int
val d__3__ = peekVal 0 : Data.pos list
in
( (PO (d__1__))::(d__3__) ) end : Data.pos list))
;
(* Rule 86, file Parser.grm, line 232 *)
val _ = update_ yyact 86
(fn () => repr(let
val d__1__ = peekVal 2 : string
val d__3__ = peekVal 0 : Data.pos list
in
( (LA (d__1__))::(d__3__) ) end : Data.pos list))
;
(* Rule 87, file Parser.grm, line 233 *)
val _ = update_ yyact 87
(fn () => repr(let
val d__1__ = peekVal 1 : int
in
( (PO (d__1__))::nil ) end : Data.pos list))
;
(* Rule 88, file Parser.grm, line 234 *)
val _ = update_ yyact 88
(fn () => repr(let
val d__1__ = peekVal 1 : string
in
( (LA (d__1__))::nil ) end : Data.pos list))
;
(* Rule 89, file Parser.grm, line 235 *)
val _ = update_ yyact 89
(fn () => repr(let
in
( nil ) end : Data.pos list))
;
(* Entry Main *)
val _ = update_ yyact 90 (fn () => raise yyexit (peekVal 0));
val yytables : parseTables =
  ( yyact,
    yytransl,
    yylhs,
    yylen,
    yydefred,
    yydgoto,
    yysindex,
    yyrindex,
    yygindex,
    YYTABLESIZE,
    yytable,
    yycheck );
fun Main lexer lexbuf = yyparse yytables 1 lexer lexbuf;
end (* of structure Parser.sml *)
(* ==========================================================================================
                             Lexer.sml
   ========================================================================================== *)

structure Lexer= 
struct
local open Obj Lexing in


 open Lexing Parser;

 exception LexicalError of string * int * int (* (message, loc1, loc2) *)

 fun lexerError lexbuf s = 
     raise LexicalError (s, getLexemeStart lexbuf, getLexemeEnd lexbuf);

 val commentStart = ref 0;  (* Start of outermost comment being scanned *)
 
 fun commentNotClosed lexbuf =
     raise LexicalError ("Comment not terminated", 
                         !commentStart, getLexemeEnd lexbuf);
     
 val commentDepth = ref 0;  (* Current comment nesting *)

 (* Scan keywords as identifiers and use this function to distinguish them. *)
 (* If the set of keywords is large, use an auxiliary hashtable.            *)

fun keyword s = (* Keywords *)
     case s of
         "with"             => TWITH
 (* Hilfstoken *)
       | "type"             => TTYPE
       | "const"            => TCON 
       | "var"              => TVAR
       | "def"              => TDEF
       | "lemma"            => TLEMMA
       | "axiom"            => TAXIOM
       | "apply"            => TAPPLY
 (* Gleichheitsregeln *)
       | "rew"              => TREW
       | "pos"              => TPOS
       | "subst"            => TINSERT
       | "ref"              => TREF
       | "sym"              => TSYM
       | "trans"            => TTRANS
       | "beta"             => TBETA
       | "eta"              => TETA
       | "lam"              => TLAM
 (* ** Kommandos ** *)
       | "goal"             => TGOAL
       | "elim"             => TELIM
       | "remove"           => TREMOVE
       | "instance"         => TINSTANCE
       | "move"             => TMOVE
       | "shift"            => TSHIFT
       | "intro"            => TINTRO
       | "cut"              => TCUT
       | "simplify"         => TSIMPLIFY
       | "save"             => TSAVE
       | "help"             => THELP
       | "print"            => TPRINT
       | "labels"       => TSHOWLABELS
       | "latex"            => TLATEX
       | "undo"             => TUNDO
       | "reset"            => TRESET
       | "exit"             => TEXIT
       | "FALSE"            => TCONST "FALSE"
       | "TRUE"             => TCONST "TRUE"
       | "false"            => TCONST "FALSE"
       | "true"             => TCONST "TRUE"
       | _                  => TNAME s;
 
 
fun action_26 lexbuf = (
 lexerError lexbuf "Illegal symbol in input" )
and action_25 lexbuf = (
 TEOF )
and action_24 lexbuf = (
 TEOL )
and action_23 lexbuf = (
 TDOT )
and action_22 lexbuf = (
 TCOMMA )
and action_21 lexbuf = (
 TDOUBLEPT )
and action_20 lexbuf = (
 TSEMI )
and action_19 lexbuf = (
 TRPAR )
and action_18 lexbuf = (
 TLPAR )
and action_17 lexbuf = (
 TEXISTS )
and action_16 lexbuf = (
 TALL )
and action_15 lexbuf = (
 TLAMBDA )
and action_14 lexbuf = (
 TEQ )
and action_13 lexbuf = (
 TAEQUI )
and action_12 lexbuf = (
 TIMP )
and action_11 lexbuf = (
 TNOT )
and action_10 lexbuf = (
 TAND )
and action_9 lexbuf = (
 TOR )
and action_8 lexbuf = (
 TDASHARROW )
and action_7 lexbuf = (
 commentStart := getLexemeStart lexbuf;
                          commentDepth := 1; 
                          SkipComment lexbuf; Token lexbuf )
and action_6 lexbuf = (
 keyword (getLexeme lexbuf) )
and action_5 lexbuf = (
 case Int.fromString (getLexeme lexbuf) of
                               NONE   => lexerError lexbuf "internal error"
               		     | SOME i => TNR i
                        )
and action_4 lexbuf = (
 Token lexbuf )
and action_3 lexbuf = (
 SkipComment lexbuf )
and action_2 lexbuf = (
 commentNotClosed lexbuf )
and action_1 lexbuf = (
 commentDepth := !commentDepth + 1; 
                          SkipComment lexbuf )
and action_0 lexbuf = (
 commentDepth := !commentDepth - 1;  
                          if !commentDepth = 0 then ()
                          else SkipComment lexbuf 
                        )
and state_0 lexbuf = (
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"*" => state_35 lexbuf
 |  #"(" => state_34 lexbuf
 |  #"\^Z" => action_2 lexbuf
 |  #"\^@" => action_2 lexbuf
 |  _ => action_3 lexbuf
 end)
and state_1 lexbuf = (
 let val currChar = getNextChar lexbuf in
 if currChar >= #"A" andalso currChar <= #"Z" then  state_19 lexbuf
 else if currChar >= #"a" andalso currChar <= #"z" then  state_19 lexbuf
 else if currChar >= #"0" andalso currChar <= #"9" then  state_14 lexbuf
 else case currChar of
    #"\t" => action_4 lexbuf
 |  #"\r" => action_4 lexbuf
 |  #" " => action_4 lexbuf
 |  #"~" => state_21 lexbuf
 |  #"\\" => state_20 lexbuf
 |  #"=" => state_18 lexbuf
 |  #"<" => state_17 lexbuf
 |  #";" => action_20 lexbuf
 |  #":" => action_21 lexbuf
 |  #"/" => state_13 lexbuf
 |  #"." => action_23 lexbuf
 |  #"-" => state_11 lexbuf
 |  #"," => action_22 lexbuf
 |  #"+" => action_9 lexbuf
 |  #")" => action_19 lexbuf
 |  #"(" => state_7 lexbuf
 |  #"&" => action_10 lexbuf
 |  #"\n" => action_24 lexbuf
 |  #"\^@" => action_25 lexbuf
 |  _ => action_26 lexbuf
 end)
and state_7 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_18);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"*" => action_7 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_11 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_26);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #">" => action_8 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_13 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_26);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"\\" => action_17 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_14 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_5);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_22 lexbuf
 else backtrack lexbuf
 end)
and state_17 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_26);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"=" => state_26 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_18 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_14);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #">" => action_12 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_19 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_6);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_24 lexbuf
 else if currChar >= #"A" andalso currChar <= #"Z" then  state_24 lexbuf
 else if currChar >= #"a" andalso currChar <= #"z" then  state_24 lexbuf
 else case currChar of
    #"'" => state_24 lexbuf
 |  #"_" => state_24 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_20 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_15);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"/" => action_16 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_21 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_11);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_22 lexbuf
 else backtrack lexbuf
 end)
and state_22 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_5);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_22 lexbuf
 else backtrack lexbuf
 end)
and state_24 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_6);
 let val currChar = getNextChar lexbuf in
 if currChar >= #"0" andalso currChar <= #"9" then  state_24 lexbuf
 else if currChar >= #"A" andalso currChar <= #"Z" then  state_24 lexbuf
 else if currChar >= #"a" andalso currChar <= #"z" then  state_24 lexbuf
 else case currChar of
    #"'" => state_24 lexbuf
 |  #"_" => state_24 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_26 lexbuf = (
 let val currChar = getNextChar lexbuf in
 case currChar of
    #">" => action_13 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_34 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_3);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #"*" => action_1 lexbuf
 |  _ => backtrack lexbuf
 end)
and state_35 lexbuf = (
 setLexLastPos lexbuf (getLexCurrPos lexbuf);
 setLexLastAction lexbuf (magic action_3);
 let val currChar = getNextChar lexbuf in
 case currChar of
    #")" => action_0 lexbuf
 |  _ => backtrack lexbuf
 end)
and Token lexbuf =
  (setLexLastAction lexbuf (magic dummyAction);
   setLexStartPos lexbuf (getLexCurrPos lexbuf);
   state_1 lexbuf)

and SkipComment lexbuf =
  (setLexLastAction lexbuf (magic dummyAction);
   setLexStartPos lexbuf (getLexCurrPos lexbuf);
   state_0 lexbuf)

(* The following checks type consistency of actions *)
val _ = fn _ => [action_26, action_25, action_24, action_23, action_22, action_21, action_20, action_19, action_18, action_17, action_16, action_15, action_14, action_13, action_12, action_11, action_10, action_9, action_8, action_7, action_6, action_5, action_4];
val _ = fn _ => [action_3, action_2, action_1, action_0];

end
end (* of structure Lexer.sml *)
(* ==========================================================================================
                             Elim.sml
   ========================================================================================== *)

structure Elim= 
struct
(* Elim.sml implementiert alle Eliminierungs-Regeln des Natural Deduction
 * calculus:
 *
 *  @  (E x . A[x]) => B   -->  A[x] => B                 with x as a new parameter
 *  @  (\/ x. A[x]  => B   -->  A[? i] => B               (? i) is the i-th gamma variable
 *  @  A + B => C          -->  (1) A => B   (2) A => C   splits the goal in two subgoals
 *  @  (A & B) & C  => D   -->  B & ( A & C )             smoothes the and-tree
 *  @  ~A => B             -->  ~A => A                   if ~A is valid, then it's enough to show A
 *)



local open Data Goal in

fun main(goal,modus) = 
    let 
	val pter = Goal.get_first_goal(goal)
	val newgoal = elim_left pter goal modus
    in (Goal.print_goal(newgoal); newgoal)
    end


and elim_left (IMP(oldleft as EX((t,i),m), right)) goal _ =                              (* EXISTIERT LEFT *)
    let 
	val newname    = Goal.freeVarIntro(goal,i) (* holt den String namen der geb. Var i *)
	val tempgoal = Goal.update_ftypeenv(goal, newname , t) (* hier werden ebenfalls die geb. Var ersetzt *)
	val freemaps2i  = Goal.freemaps2i(tempgoal)
	val tempgoal' = Goal.add_delta(tempgoal, freemaps2i newname)
	val newleft  = ETT.down 0 (ETT.sub m 0 (V ((freemaps2i newname) + 1))) 1 
	val tempgoal''= Goal.update_first_goal(tempgoal',IMP( newleft, right ))
	val newgoal = Goal.kaper_check(tempgoal'')
    in
	newgoal
    end
  | elim_left (IMP(oldleft as AND(EX((t,i),m),B), right)) goal _ =                       (* EXISTIERT LEFT IM AND CASE*)
    let 
	val newname    = Goal.freeVarIntro(goal,i) (* holt den String namen der geb. Var i *)
	val tempgoal   = Goal.update_ftypeenv(goal, newname , t) (* hier werden ebenfalls die geb. Var ersetzt *)
	val freemaps2i  = Goal.freemaps2i(tempgoal)
	val tempgoal' = Goal.add_delta(tempgoal, freemaps2i newname)
	val newleft' = ETT.down 0 (ETT.sub m 0 (V ((freemaps2i newname) + 1))) 1
	val newleft  = AND(newleft', B)
	val tempgoal''= Goal.update_first_goal(tempgoal',IMP(newleft,right ))
	val newgoal = Goal.kaper_check(tempgoal'')
    in
	newgoal
    end

  | elim_left (IMP(oldleft as ALL((t,i), m), right)) goal modus =                            (* ALL LEFT *)
    let
	val nr_next_gamma = Goal.next_gamma_nr(goal)
	val instance = GAMMA (t, nr_next_gamma)
	val newgoal' = Goal.add_gamma(goal)
	val newleft' = ETT.down 0 (ETT.sub m 0 (instance)) 1
	val newleft = ( case modus of
		       STANDART => newleft'
		     | MORE => AND(newleft', oldleft) )
	val newgoal = Goal.update_first_goal(newgoal',IMP( newleft , right))
    in
	newgoal
    end

  | elim_left (IMP(oldleft as AND(ALL((t,i), m), B), right)) goal modus =                     (* ALL LEFT IM AND CASE *)
    let
	val nr_next_gamma = Goal.next_gamma_nr(goal)
	val instance = GAMMA (t, nr_next_gamma)
	val newgoal' = Goal.add_gamma(goal)
	val newleft' = ETT.down 0 (ETT.sub m 0 (instance)) 1
	val newleft  = ( case modus of 
			STANDART => AND( newleft', B )
		      | MORE => AND(newleft', oldleft) )
	val newgoal = Goal.update_first_goal(newgoal',IMP(newleft , right))
    in
	newgoal
    end

  | elim_left (IMP(OR(a,b), right)) goal _ =                              (*  A + B => C  wird zu  A => C und B => C *)
    let
	val newgoal1 = Goal.update_first_goal(goal, IMP(a,right))
	val newgoal2 = Goal.split_goal(newgoal1, IMP(b,right))
    in
	newgoal2
    end

  | elim_left (IMP(AND(OR(a,b),c), right)) goal _ =     
    let
	val newgoal1 = Goal.update_first_goal(goal, IMP(AND(a,c),right))
	val newgoal2 = Goal.split_goal(newgoal1, IMP(AND(b,c),right))
    in
	newgoal2
    end

  | elim_left (IMP(IMP(a,b), right)) goal _ =                     (* ( A => B ) => C wird zu B => C und (~C) => A *)
    let
	val newgoal1 = Goal.update_first_goal(goal, IMP(b, right))
	val newgoal2 = Goal.split_goal(newgoal1, IMP(NOT(right), a))
    in 
	newgoal2
    end
 
  | elim_left (IMP(AND(IMP(a,b),c), right)) goal _ = 
    let
	val newgoal1 = Goal.update_first_goal(goal, IMP(AND(b,c), right))
	val newgoal2 = Goal.split_goal(newgoal1, IMP(AND(NOT(right),c), a))
    in 
	newgoal2
    end





  | elim_left (IMP(left as (NOT(a)), right )) goal _ =             (* Wenn wir ~A haben, genuegt es A zu zeigen *)
     let 
	 val newright = a
	 val newgoal = Goal.update_first_goal(goal,IMP( left , newright ))
     in 
	 newgoal
     end

  | elim_left (IMP(left as (AND(NOT(a),b)), right )) goal _ =
     let
	 val newright = a
	 val newgoal = Goal.update_first_goal(goal, IMP(left,newright))
     in
	 newgoal
     end
  

  | elim_left (IMP(AND(AND(a,b), c), right)) goal _ =        (* glaettet den AND Tree (A * B) * C => D *)
     let                                                   (* wird zu                 B * A * C => D *)
	val newleft = AND(b,AND(a,c))
	val newgoal = Goal.update_first_goal(goal, IMP(newleft,right))
     in 
	 newgoal
     end

	 
  | elim_left (IMP(t as (EQ(a,b)), right )) goal _ =
     let
	 (* 
	  * Rufe Prozedur Equality.main auf
	  * gib den ersetzten Subterm zurueck
	  *)
	 val (newleft, newgoal) = Equality.main(t, goal)  
     in
	 Goal.update_first_goal(newgoal, IMP(newleft, right))
     end
	 
  | elim_left (IMP(AND(t as (EQ(a,b)), B), right )) goal _ =
     let
	 (* 
	  * Rufe Prozedur Equality.main auf
	  * gib den ersetzten Subterm zurueck
	  *)
	 val (newleft, newgoal) = Equality.main(t, goal)  
     in
	 Goal.update_first_goal (newgoal, IMP(AND(newleft, B), right ))
     end

     
(* TRANSLATE DEFINITION CHECK 
 *
 * Handelt es sich bei der obersten Praemisse um eine Definition des users,
 * wird versucht, den Ausdruck der Definition einzusetzen: 
 * A[D] => B  and A contains a Definition (D=C) --> A[D:=C] => B
 *)

  | elim_left (IMP(left, right )) goal modus =
     let
	 val (newleft, manipulated) = 
	     case left of
		 AND(a,b) => if (Goal.check_def(goal, a)) then (AND(Goal.translate_def(goal,a),b) , true)
			     else (left, false)
	       | a => if (Goal.check_def(goal, a)) then (Goal.translate_def(goal,a), true)
		      else (left, false)
	 val newgoal = if manipulated then Goal.update_first_goal(goal, IMP(newleft, right)) else goal
     in 
     (* Damit kein Zyklus entsteht *)
	 if manipulated then 
	     let val newfirst = Goal.get_first_goal(newgoal)
	     in elim_left newfirst newgoal modus
	     end 
	 else newgoal   

     end


  | elim_left _ goal _ = goal
	     
end (* of local open *)
end (* of structure Elim.sml *)
(* ==========================================================================================
                             Instance.sml
   ========================================================================================== *)

structure Instance= 
struct
(* Instance.sml implementiert die Instanziierung von Gammavariablen
 * mit Subtermen:
 * instance <i> = <expr>    substituiert innerhalb des aktiven Goals 
 *                          alle Vorkommen der Gammavariablen i durch
 *                          <expr>
 *
 * Gamma Variablen sind genauso wie Deltavaribalen innerhalb eines
 * Graphens gespeichert, bei jeder Instanziierung entscheided ein Zyklustest
 * ueber die Gueltigkeit der Substituierung
 *) 

local open Data Goal Env in


val var_order = fn (GAMMA(_,i) , GAMMA(_,j)) => if (i < j) then LESS else if ( i > j ) then GREATER else EQUAL
                  |(GAMMA(_,i) , V (j) )     => LESS (* Gammas immer kleiner als V *)
                  |(V(i) , GAMMA(j) )        => GREATER 
                  |(V(i) , V(j))             => if (i < j) then LESS else if ( i > j ) then GREATER else EQUAL
                  | (_,_) => LESS
     
(* remove_dbl loescht das Doppelauftreten der Variablen in der Liste *)
fun remove_dbl xs = foldr ( fn (x,nil) => [x] 
                               | (x,ys) => if x = hd(ys) then ys else x::ys ) 
                      nil (Listsort.sort var_order xs)

(* Checkt mittels Typechecker, ob Term <pter> den Typ <typ> besitzt *)
fun checktype (pter, typ) goal = 
    let 
	val cmap = Goal.typc(goal)
	val fmap = Goal.typi(goal)  (* Gets a int->Typ mapping function *)
	val tmap = Goal.typemap(goal)  (* Gets a string->bool mapping function *)
    in (( Typecheck.get_type pter (cmap,fmap,tmap)) = typ ) 
	handle Typecheck.illtyped _ => false
    end
			     
(* Ersetzt Gamma Variable i durch Term n, nur dann, wenn Typen uebereinstimmen *)
fun gammasub (g as GAMMA (t,i) ) n nr tycheck = 
    if (i=nr) then 
	if (tycheck t) 
	then n 
	else raise Error("Gamma Var. "^Int.toString(i)^" and Instanciation hava different Types\n")
    else g
  | gammasub (A(m,m'))           n nr tycheck = A(gammasub m n nr tycheck, gammasub m' n nr tycheck)
  | gammasub (L((t,i),m))        n nr tycheck = L((t,i), gammasub m (ETT.up 0 n 1) nr tycheck)
  | gammasub (EX((t,i),m))       n nr tycheck = EX((t,i), gammasub m (ETT.up 0 n 1) nr tycheck)
  | gammasub (ALL((t,i),m))      n nr tycheck = ALL((t,i), gammasub m (ETT.up 0 n 1) nr tycheck)
  | gammasub (AND(m,m'))         n nr tycheck = AND( gammasub m n nr tycheck, gammasub m' n nr tycheck)
  | gammasub (OR(m,m'))          n nr tycheck = OR( gammasub m n nr tycheck , gammasub m' n nr tycheck )
  | gammasub (IMP(m,m'))         n nr tycheck = IMP( gammasub m n nr tycheck , gammasub m' n nr tycheck )
  | gammasub (EQ(m,m'))          n nr tycheck = EQ( gammasub m n nr tycheck, gammasub m' n nr tycheck )
  | gammasub (NOT(m))            n nr tycheck = NOT( gammasub m n nr tycheck)
  | gammasub a                   n nr tycheck = a  (* Var und Const bleiben erhalten *)



fun subst_gamma'(goal, _,_) 0 = goal
  | subst_gamma'(oldgoal, gammanr, f) n =
 let
    val pter = Goal.get_first_goal(oldgoal)
    val instgoal = Goal.first_goal( Goal.init_goal_env(oldgoal), f )

    val inst = 
	case (Goal.get_first_goal (instgoal)) of
	    IMP(_,t) => t
	  | _ => raise Error ("NEVER HAPPEND\n")
	
    val freevarlist = remove_dbl(ETT.getfreevars(inst))
   (* DEBUG  val _ = List.app( fn a => print (Visual.pter2string a)) freevarlist *)
	
    val typcheck = fn t => checktype (inst, t) oldgoal

    val mergegoal = Goal.merge_envs(oldgoal,instgoal)
   (* wenn es erlaubt ist zu substituieren, dann tue es ;) *)
    val newgoal = if ( Goal.substitute_gamma oldgoal (gammanr, freevarlist) )
		  then 
		      let 
			  val tempgoal = Goal.update_first_goal(mergegoal, gammasub pter inst gammanr typcheck)
			  (* Ueberpruefe, ob freie Variablen gekapert wurden *)
			  val kaperfreegoal = Goal.kaper_check(tempgoal)
		      in
			  kaperfreegoal
		      end
		  else raise Error ("There are free variables in instanciation, not known at gamma creation\n")
in
    subst_gamma'(Goal.cyclic_shift(newgoal), gammanr, f) (n-1)
end



(* Hauptroutine zum Substituieren *)
(* ruft fuer jedes Subgoal subst_gamma' auf *)

fun subst_gamma(oldgoal, gammanr, f) = 
let
    val n = Goal.get_nr_of_goals(oldgoal)
in
    subst_gamma'(oldgoal, gammanr, f) n
end

	 

fun main(goal,i,formel) = let
			      val newgoal = subst_gamma(goal,i,formel)
			  in
			      (Goal.print_goal(newgoal) ; newgoal )
			  end

end (* of local open *)
end (* of structure Instance.sml *)
(* ==========================================================================================
                             Cut.sml
   ========================================================================================== *)

structure Cut= 
struct
(* This Cut.sml File implements the Cut Rule
 i.e. :  A => C   is the same as:    A => B und B => C
 *)

local open Data Goal in

fun add formel goal = 
    let
	val pter = Goal.get_first_goal(goal)
	val (left,right) = 
	   ( case pter of
		IMP(l,r) => (l,r)
	      |_     => raise Error("Fehler in Cut Rule 1\n")
	   )
		
	val newpter = IMP(left, formel)
	val newgoal' = Goal.update_first_goal(goal,newpter)
	val newgoal = Goal.split_goal(newgoal',
				      IMP( ( case left of 
					    (C "TRUE") => formel 
					    | _ => AND(formel, left))  , right))
    in 
	newgoal 
    end




fun main(goal,f) = let 
		    val instinit = Goal.init_goal_env(goal)
		    val instgoal = Goal.first_goal(instinit,f)
		   
		    val formel' = Goal.get_first_goal(instgoal)
		    val formel  =( case formel' of
			          (IMP(C "TRUE", m)) => m
				    | _ => raise Error("Fehler Cut Rule\n")
				 )
		    val mergegoal = Goal.merge_envs(goal,instgoal)
		    val new_goal = add formel mergegoal
	      in
		  (Goal.print_goal(new_goal); new_goal)
	      end

end (* of local open *)

end (* of structure Cut.sml *)
(* ==========================================================================================
                             Move.sml
   ========================================================================================== *)

structure Move= 
struct
(*  The Move.sml Modul implements the Equivalence of :
 *  B => ~A + (C + D)    <-->   A & B => C + D   <-->   ~C & ( A & B ) => D
 *)

local open Data Goal in

fun not (NOT(t)) = t
  | not t        = NOT(t)
    
    
fun smooth (t as (OR(a,b))) = 
    (case b of
	 (C("FALSE")) => a      (* a + FALSE = a *)
       | _            => t      (* gesamter Teilausdruck *)
	     )
	 
  | smooth ( t as AND(a,b)) = 
	 (case b of
	      (C("TRUE")) => a   
	    | _           => t )
  | smooth t = t

	      
fun mover2l (IMP(left, OR(a,b))) goal = 
    Goal.update_first_goal(goal, IMP(smooth(AND(not(a),left)), b) )

  | mover2l (IMP(left, a)) goal = 
    Goal.update_first_goal(goal, IMP(smooth(AND(not(a),left)), C "FALSE"))
  | mover2l _ goal = goal



fun movel2r ( IMP(AND(a,b),right) ) goal = 
	Goal.update_first_goal(goal, IMP(b,smooth(OR(not(a),right))))
  
  | movel2r ( IMP(a, right) ) goal =
	Goal.update_first_goal(goal, IMP(C "TRUE", smooth(OR(not(a),right))))

  | movel2r _ goal = goal
    


fun main(goal, d) = 
    let
	val pter = Goal.get_first_goal(goal)
	val newgoal = case d of 
	    LEFT  => movel2r pter goal
	  | RIGHT => mover2l pter goal
    in
	(Goal.print_goal(newgoal); newgoal)
    end

end



(* 
val _ = print ("(l)eft or (r)ight to move? :")
	val s = TextIO.inputLine TextIO.stdIn                      
	val (l,r) = 
	    case (String.sub (s,0)) of
*)
end (* of structure Move.sml *)
(* ==========================================================================================
                             Remove.sml
   ========================================================================================== *)

structure Remove= 
struct
(* This Remove.sml Modul implements the Removing Procedure of 
 * an Hypothesis/Conclusion :
 * <remove (l/r) i> - removes the i-th Hypothesis in the Conjunctiontree 
 *                                    /Conclusion in the Disjunctiontree
 *)

local open Data Goal in
exception norem


(* Removes the n.th component in an AND Tree *)
fun removeit_and' (AND(a,b)) n l = if n=l then b 
			       else( case b of
					AND(x,y) => AND(a,removeit_and' b n (l+1))
				      | t => if ( n=(l+1) ) then a else AND(a,b) )
					
  | removeit_and' _ _ _ = raise norem
fun removeit_and p n = removeit_and' p n 1

(* Removes the n.th component in an OR Tree *)
fun removeit_or' (OR(a,b)) n l = if n=l then b 
			       else ( case b of
					OR(x,y) => OR(a,removeit_or' b n (l+1))
				      | t => if ( n=(l+1) ) then a else OR(a,b) )
					
  | removeit_or' _ _ _ = raise norem
fun removeit_or p n = removeit_or' p n 1



(* **********  Main Processes *********** *)

fun process_left (IMP(left as AND(a,B), right)) nr goal = 
    let
	val newleft = (removeit_and left nr) handle norem => left
	val newgoal = Goal.update_first_goal(goal,IMP(newleft ,right) )
    in newgoal 
    end
  |process_left (IMP(a, right)) 1 goal = 
    let
	val newgoal = Goal.update_first_goal(goal, IMP(C "TRUE" ,right) )
    in newgoal 
    end
  |process_left t _ goal = goal

fun process_right (IMP(left, right as OR(a,B))) nr goal = 
    let
	val newright = (removeit_or right nr) handle norem => right
	val newgoal = Goal.update_first_goal(goal,IMP(left , newright) )
    in newgoal 
    end
  |process_right (IMP(left, a)) 1 goal = 
    let
	val newgoal = Goal.update_first_goal(goal,IMP(left , C "FALSE" ) )
    in newgoal 
    end
  |process_right t _ goal = goal



fun main(g,(d,i)) = 
    let 
	val formel = Goal.get_first_goal(g)
	val new_goal = case d of 
	    LEFT  => process_left formel i g
	  | RIGHT => process_right formel i g
    in
	(Goal.print_goal(new_goal);new_goal)
    end

end (* of local open *)

end (* of structure Remove.sml *)
(* ==========================================================================================
                             Shift.sml
   ========================================================================================== *)

structure Shift= 
struct
(* Shifter Routinen der Praemissen :
 * <shift l u [nr] > - shiftet die Praemissen zyklisch um eine Position nach oben 
 *                   [ optional um Anzahl <nr> Positionen ] 
 * <shift l d [nr] > - analog nach unten
 *
 * <shift r u [nr] > - shiftet die Conclusionen zyklisch um eine Position nach oben 
 *                   [ optional um Anzahl <nr> Positionen ] 
 * <shift r d [nr] > - analog nach unten
 *
 *
 * <shift goals [nr] > - shiftet die Subgoals cyclisch um eine Position nach oben 
 *                    [ optional um Anzahl <nr> Positionen ] 
 *)

local open Data Goal in
exception never
(* Go to the right most leaf in AND Tree and put m there *)
fun putlast_and (AND(x,y)) m = AND(x,putlast_and y m)
  | putlast_and t m = AND(t,m)

(* Returns the right most leaf in Front of an AND Tree *)
fun getlast_and (AND(x,t as AND(y,z))) = let
					     val (a,b) = ( case (getlast_and t) of
							       AND(a,b) => (a,b)
							      | _ => raise never
							 )
					 in
					     AND(a,AND(x,b))
					 end
  | getlast_and (AND(x, leaf ))        = AND(leaf,x)
  | getlast_and t = t

(* Go to the right most leaf in OR Tree and put m there *)
fun putlast_or (OR(x,y)) m = OR(x,putlast_or y m)
  | putlast_or t m = OR(t,m)

(* Returns the right most leaf in Front of an OR Tree *)
fun getlast_or (OR(x,t as OR(y,z))) = let
                                       val (a,b) = ( case (getlast_or t) of
							       OR(a,b) => (a,b)
							      | _ => raise never
						   )
				     in
					 OR(a,OR(x,b))
				     end
  | getlast_or (OR(x, leaf ))        = OR(leaf,x)
  | getlast_or t = t

(* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)

(* Force left a Cyclic Left Shift        1 & 2 & 3   ->   2 & 3 & 1      *)
fun shift_left_up (IMP(AND(a,B),right)) goal =  
    let
	val newleft = putlast_and B a
    in
	Goal.update_first_goal(goal, IMP(newleft,right) )
    end
  | shift_left_up _ goal = goal

    
(* Force left a Cyclic Right Shift        1 & 2 & 3   ->   3 & 1 & 2      *)
fun shift_left_down (IMP(AND(a,B),right)) goal = 
    let
	val newleft = getlast_and (AND(a,B))
    in
	Goal.update_first_goal(goal, IMP(newleft,right) )
    end
  | shift_left_down _ goal = goal


(* ********************  Rechts Shifting ********************** *)

(* Force right a Cyclic Left Shift        1 & 2 & 3   ->   2 & 3 & 1      *)
fun shift_right_up (IMP(left,OR(a,B))) goal =  
    let
	val newright = putlast_or B a
    in
	Goal.update_first_goal(goal, IMP(left,newright) )
    end
  | shift_right_up _ goal = goal

    
(* Force right a Cyclic Right Shift        1 & 2 & 3   ->   3 & 1 & 2      *)
fun shift_right_down (IMP(left,OR(a,B))) goal = 
    let
	val newright = getlast_or (OR(a,B))
    in
	Goal.update_first_goal(goal, IMP(left,newright) )
    end
  | shift_right_down _ goal = goal


(* fuer shifings um mehrere Positionen *)
fun loopshift cmd goal 0 = goal
  | loopshift cmd goal i = 
    let val pter = Goal.get_first_goal(goal)
    in loopshift cmd (cmd pter goal) (i-1)
    end

fun goalloopshift goal 0 = goal
  | goalloopshift goal i = goalloopshift (Goal.cyclic_shift(goal)) (i-1)


fun main(goal, x, (y, i) ) = let
			val pter = Goal.get_first_goal(goal)
			val newgoal = case x of  (* X-Richtung, LEFT or RIGHT *)
			    LEFT => ( case y of   (* Y-Richtung also up and down *)
				     UP   => loopshift (shift_left_up)   goal i
				   | DOWN => loopshift (shift_left_down) goal i
					 )
			  |RIGHT =>  ( case y of   (* Y-Richtung also up and down *)
				     UP   => loopshift (shift_right_up)   goal i
				   | DOWN => loopshift (shift_right_down) goal i
					 )
		    in
			(Goal.print_goal(newgoal); newgoal)
		    end



fun goalmain(goal, n) = 
    let
	val newgoal = goalloopshift goal n
    in
	(Goal.print_goal(newgoal); newgoal)
    end

end (* of local open *)
end (* of structure Shift.sml *)
(* ==========================================================================================
                             Bool.sml
   ========================================================================================== *)

structure Bool= 
struct
(* Bool.sml implementiert die Regel Bool des Natural Deduction
 * calculus; Formeln werden uebersetzt in Primbaeume und so weit
 * wie moeglich vereinfacht.
 * Applikation werden dabei temporaer als Variablen representiert
 * ebenso Teilausdreucke bestehend aus Existenz-oder Allquantoren
 *)

local open Data in

exception primerror
datatype dt = F | T | D of var * dt * dt


(* Implementiert Axiom  < E x. u = u  >  *)
fun redquan (EX((t,s),f))  = if ( ETT.free 0 f ) then (EX((t,s),redquan f))  else redquan (ETT.down 0 f 1)
  | redquan (ALL((t,s),f)) = if ( ETT.free 0 f ) then (ALL((t,s),redquan f)) else redquan (ETT.down 0 f 1)
  | redquan t = t

(* Basisalgorithmen fuer Primbaeume *)
fun red (t as (_, a, b)) = if a=b then a else D(t)

fun var (x: var) = D(x, F, T)

fun neg F = T
  | neg T = F
  | neg (D(x, a, b)) = D(x, neg a, neg b)


fun And (F, _) = F
  | And (T, b) = b
  | And (_, F) = F
  | And (a, T) = a
  | And (a as D(x, a0, a1), b as D(y, b0, b1)) =
    if x=y then
      red(x, And(a0, b0), And(a1, b1))
    else if x<y then red(x, And(a0, b), And(a1, b))
                else red(y, And(a, b0), And(a, b1))

fun or (a,b) = neg(And(neg a, neg b))


(* bcom wandelt Decision Trees back zu pter *)

fun bcom F           v2a = C "FALSE"
  | bcom T           v2a = C "TRUE"
  | bcom (D(x,a,b))  v2a = let val p = (v2a(x)) handle primerror => (V x)   (* Rueckmapping Var to App *)
			     in
				 case (a,b) of 
				     (F,T)   => p
				   | (T,F)   => NOT(p)
				   | (F,b')  => AND(p, bcom b v2a)
				   | (T,b')  => OR(NOT(p), AND(p, bcom b v2a))
				   | (a',T)  => OR(AND(NOT(p), bcom a' v2a), p)
				   | (a',F)  => AND(NOT(p), bcom a' v2a)
				   | (a',b') => OR(AND(NOT(p), bcom a' v2a ), AND(p , bcom b' v2a))
			     end


(* com wandelt pter-Ausdruecke in Primbaeume um *)
(* Mappe Gamma Variablen des Typs Bool ebenfalls zu Boolvars *)

fun com (C "FALSE")               p2v = F
  | com (C "TRUE")                p2v = T
  | com (t as (C s))              p2v = var ( p2v t )
  | com (V(x))                    p2v = var x
  | com (NOT(a))                  p2v = neg(com a p2v)
  | com (AND(a,b))                p2v = And(com a p2v , com b p2v)
  | com (OR(a,b))                 p2v = or (com a p2v , com b p2v)
  | com (IMP(a,b))                p2v = or(neg(com a p2v), com b p2v)
  | com (t as (EQ(a,b)))          p2v = if(ETT.equal(a,b)) then T else var ( p2v t )
  | com (t as (A(_,_)))           p2v = var ( p2v t )
  | com (t as (ALL(_,_)))         p2v = var ( p2v t )
  | com (t as (EX(_,_)))          p2v = var ( p2v t )
  | com (t as GAMMA(TC "bool",_)) p2v = var ( p2v t )  
  | com (L(_,_))                  p2v = raise Error ("Ein Lambda kommt selten allein\n")
  | com t                         p2v = raise Error ("Error while translating pter->primtree\n")
                                                  



(* pterprocess' sammelt Applikationen, Gammavariablen, 
 * Existenz- und Allquantoren in einer Liste rs 
 *)
fun pterprocess' (t as (C("FALSE")) )     rs   = rs
  | pterprocess' (t as (C("TRUE"))  )     rs   = rs
  (* Constanten werden temp. zu Variablen umgetauft *)
  | pterprocess' (t as (C s) )            rs   = (t::rs)
  | pterprocess' (t as (A(a,b)))          rs   = (t::rs)
  | pterprocess' (t as (GAMMA(_)))        rs   = (t::rs)
  | pterprocess' (NOT(a))                 rs   = pterprocess' a rs 
  | pterprocess' (AND(a,b))               rs   = (pterprocess' a rs ) @ (pterprocess' b rs )
  | pterprocess' (OR(a,b))                rs   = (pterprocess' a rs ) @ (pterprocess' b rs )
  | pterprocess' (IMP(a,b))               rs   = (pterprocess' a rs ) @ (pterprocess' b rs )
  | pterprocess' (t as (EQ(_,_)))         rs   = (t::rs)
  | pterprocess' (t as (ALL(_,f)))        rs   = (t::rs)
  | pterprocess' (t as (EX(_,f)))         rs   = (t::rs)
 
  | pterprocess' (L(_,f))                 rs   = rs (* wende BOOL nur auf unterster Stufe an *)
  | pterprocess' _                        rs   = rs

fun pterprocess pter max = let 
                              (* ls beinhaltet alle Subterme des Typs bool, die auf
			         Variablen gemappt werden 
			       *)
			      val ls = (pterprocess' pter nil)
			      val initpter2var = fn pter => raise primerror
			      val initvar2pter = fn i    => raise primerror
				  
			      val (pter2var, var2pter, _ ) = 
				  (      
				   (* Lege Temporaere Mappings fuer die Subterme an *)
				   foldl ( fn (x, (f,g,n) ) => (f x ; (f,g,n)) 
					  handle primerror => ( fn a => if ETT.equal(a,x) then n else f a , 
					               	        fn v => if v=n then x else g v ,
							        n+1 
							      )
					 ) 
				   (initpter2var, initvar2pter, max) ls 
				   )
			  in
			      (* pter2var : pter -> var 
			       * var2pter : var  -> pter 
			       *)
			      (pter2var , var2pter)
			  end
		      


		      
fun main(goal) = let 
		     val firstgoal = Goal.get_first_goal(goal)
		     val n = Goal.nr_freevars(goal)
		     val (pter2var, var2pter) = pterprocess firstgoal n   (* Bekomme Subterm-Mappings *)

		     (* Generiere Primbaum aus firstgoal und mappings *)
		     val tree = com firstgoal pter2var                (* pter -> dec tree *)
		     (* Rueckuebersetzung Primbaum nach Term *)
		     val simplpter' = (bcom tree var2pter)            (* dec tree -> pter *)
		     val simplpter  = case simplpter' of
			                (t as (C "TRUE" )) => t
				      | (t as (C "FALSE")) => t
				      | t                  => IMP (C "TRUE" , t)
			              
		     val newgoal = Goal.update_first_goal(goal, simplpter)
		 in
		     (Goal.print_goal(newgoal);newgoal)

		 end
end (* of local open *)
end (* of structure Bool.sml *)
(* ==========================================================================================
                             Lambda.sml
   ========================================================================================== *)

structure Lambda= 
struct
(* Lambda.sml implementiert beta/eta Reduktionen 
 * innerhalb eines Ausdrucks 
 *)

local open Data Goal ETT in

fun red (C c) = C c
  | red (V x) = V x
  | red (GAMMA a) = GAMMA a
  | red (A(m,m')) = beta (red m) (red m')
  | red (L(t,m)) = eta t (red m)
  | red (EX(t,m))  = EX(t, red m)
  | red (ALL(t,m)) = ALL(t, red m)
  | red (AND(m,m')) = AND( red m, red m')
  | red (OR(m,m')) = OR(red m , red m')
  | red (IMP(m,m')) = IMP(red m , red m')
  | red (EQ(m,m')) = EQ(red m, red m')
  | red (NOT(m)) = NOT(red m)
and beta (L(t,m)) n = red (down 1 (sub m 0 (up 0 n 1)) 1)
  | beta m n = A(m,n)
and eta t (A(m,V 0)) = if not (free 0 m) then down 0 m 1 else L(t, A(m, V 0))
  | eta t m = L(t,m)


fun justbeta (A( L(t,m), n)) = down 1 (sub m 0 (up 0 n 1)) 1
  | justbeta p = p

fun justeta (p as (L(t, A(m,V 0)))) = if not (free 0 m) then (down 0 m 1) else p
  | justeta p = p

fun main(goal) = let 
		     val pter = Goal.get_first_goal(goal)
		     val redpter = red(pter)
		     val newgoal = Goal.update_first_goal(goal,redpter)
		 in 
		     (Goal.print_goal(newgoal) ; newgoal )
		 end

end (* of local open *)
end (* of structure Lambda.sml *)
(* ==========================================================================================
                             Apply.sml
   ========================================================================================== *)

structure Apply= 
struct
(* This Apply.sml File implements 
 * @ the Rule ModusPonens 
 *     [1] A                                  [1] A 
 *     [2] (A => B)                           [2] (A&C&D ...) => B)
 *      =>  C                                  => C
 *     <apply 1 2> --->             bzw.      <apply 1 2> ---> 
 *     [1] B                                  [2] (C&D ...) => B)
 *     ...                                    ...
 *      => C                                   => C
 *
 * @ Adding of axioms and lemmas to the Hypothesis 
 *   of the goal !
 *      <apply (name)> adds the axiom/lemma (name) 
 *   In case of LEMMA, the lemma has to be shown after the
 *     Main goal proof.
 *   In case of AXIOM not!
 *
 *)


local open Data Goal in
exception nomodusponens
exception noequality


fun appModusPonens_rekursiv (AND(a, r as (AND(b,c)) )) match = 
    if (ETT.equal (a,match)) then r 
    else (AND(a, appModusPonens_rekursiv r match))

  | appModusPonens_rekursiv (AND(a, b)) match = 
	if (ETT.equal(a,match)) then b 
	else if (ETT.equal(b,match)) then a 
	     else raise nomodusponens
  | appModusPonens_rekursiv _ _ = raise nomodusponens

fun appModusPonens (IMP( left as (AND(a,b)), c )) match = IMP( (appModusPonens_rekursiv left match) , c )
  | appModusPonens (IMP(a, b)) match = if (ETT.equal(a,match)) then b else raise nomodusponens
  | appModusPonens _ _ = raise nomodusponens


fun get_subterm (AND(a,b)) i = if i <= 1 then a else get_subterm b (i-1)
  | get_subterm t i = if i = 1 then t else raise Error ("No subterm found !\n")


(* [addLemma]
 * fuegt die Lemmabehauptung als Subgoal hinzu 
 *)
fun addLemma (goal, name) =
    let
	val term = Goal.get_definition(goal, name)
	(* speichere die korrekte Umgebung *)
	val backup_env = Goal.get_env(goal) 
	(* hole die alte Umgebung *)
	val envref = !(Goal.get_envref(goal, name))
	(* setze diese als Hauptumgebung *)
	val goal2 = Goal.set_env(goal, envref)
	(* adde nun Lemma Beweis, hierbei wird Hauptumgebung
	 * dupliziert !! 
	 *)
	val goal3 = Goal.split_goal(goal2, IMP( C "TRUE" ,term ))
	(* schreibe Backup Umgebung zurueck *)
	val goal4 = Goal.set_env(goal3, backup_env)
	(* Aendere diese Definition zu AXIOM *)
	val _ = Goal.change_definitions_kind(goal4, name, AXIOM )
    in
	goal4
    end




(* [process_mp]
 * realisiert Modus Ponens rule 
 *)

fun process_mp (src,dest) goal = 
    let 
	val (left,right) = ( case (Goal.get_first_goal(goal)) of
			    IMP(left,right) => (left,right)
			  | _ => raise Error("Interner Fehler Apply 3 \n" ) )

	(* Bestimme zuerst die src. Voraussetzung *)
	val subsource = get_subterm left src
	(* Dann die Zielvoraussetzung *)
	val subdest   = get_subterm left dest


	val newgoal = ( 
		       let
			   val newprem = appModusPonens subdest subsource
		       in
			   Goal.update_first_goal(goal, IMP(AND(newprem,left), right))
		       end ) handle nomodusponens => goal
    in
	newgoal
    end

(* Zustaendig fuer die Definitionen *)

fun process_defs d goal = 
    let 
	val kind = Goal.get_definitions_kind(goal, d)
	val term = Goal.get_definition(goal, d)
	val newgoal = 	
	    ( case kind of
		  CLASSIC => raise Error ("Standart definitions cannot be applied!\n")

		(* Im Falle eines AXIOMS, fuege es einfach der Praemissen Liste hinzu *)

		| AXIOM   => let 
				 val firstterm = Goal.get_first_goal(goal)
				 val newterm = ( case firstterm of
						(IMP(C "TRUE" ,right)) => (IMP(term,right))
					      | (IMP ( left, right ))  => (IMP(AND(term,left),right))
					      | _ => raise Error("Interner Fehler 1 at Unit Apply")
					        )
				 val goal' = Goal.update_first_goal(goal,newterm)
			     in
				 goal'
			     end

		(* Ein Lemma kann als Voraussetzung hinzugefuegt werden, wenn es im Anschluss
		 * an den Hauptbeweis ebenfalls bewiesen wird 
		 *) 
	     
		| LEMMA   => let 
				 val firstterm = Goal.get_first_goal(goal)
				 val newterm = ( case firstterm of
                                                (IMP(C "TRUE", right)) => (IMP(term,right))
					      |	(IMP(left,right)) => (IMP(AND(term,left),right))
					      | _ => raise Error("Interner Fehler 2 at Unit Apply")
					        )
				 val goal1 = Goal.update_first_goal(goal, newterm)
				 val newgoal = addLemma(goal1, d)
				 
			     in
				 newgoal
			     end
	   ) (* end of case *)
    in 
	     newgoal
    end (* end of process_defs *)


fun main(goal, arg ) = 
    let 
	val new_goal =( case arg of
 	                (DEFINITION d) => process_defs d goal
		      | (MODUSPONENS (src,dest) ) => process_mp (src,dest) goal )

    in
	(Goal.print_goal(new_goal);new_goal)
    end

end (* of local open *)

end (* of structure Apply.sml *)
(* ==========================================================================================
                             Unify.sml
   ========================================================================================== *)

structure Unify =
struct 

    open Data

    (* Der mgu(t1,t2) ist eine Liste von Tupeln,
     * wobei die erste Komponente Variablen enthalten,
     * die zweite Terme 
     *)

    exception notunifiable
    type unifier = (int * pter) list


    fun union nil ys = ys
     |  union (x::xs) ys = if not(List.exists( fn t => t = x ) ys ) 
			       then union xs (x::ys) 
			       else union xs ys

    fun sub b ts = foldl ( fn ((i,t), newb ) => ( ETT.sub newb i t ) ) b ts

    (* ------------------------------------------------- *)
    (* |            BEGIN Hauptmethoden                | *)
    (* ------------------------------------------------- *)

    fun show_unifier uni = 
	foldl ( fn ((i, n),_)=> print (" (V "^Int.toString(i)^") -> (" ^ Visual.pter2string(n) ^ ")\n") ) () uni


    fun getUnifier' (A(a,b)) (A(c,d)) us t = 
	let 
	    val ts = getUnifier' a c us t
	 (* val _ = print ("------------------ DEBUG ------------------\n")
	    val _ = print ("A(a,b)   b = " ^ Visual.pter2string(b) ^ "\n")
            val _ = print ("A(c,d)   d = " ^ Visual.pter2string(d) ^ "\n")
	    val _ = print ("TEMP UNIFIER : \n")
	    val _ = show_unifier ts                                             *)
(*	    val substb = sub b ts               *)
	    val substd = sub d ts 
	 (* val _ = print ("A(a,b)   b = " ^ Visual.pter2string(substb) ^ "\n")
            val _ = print ("A(c,d)   d = " ^ Visual.pter2string(substd) ^ "\n")
            val _ = print ("---------------- DEBUG ENDE ---------------\n")     *)
	in getUnifier' b substd (union ts us ) t
	end


      | getUnifier' (AND(a,b)) (AND(c,d)) us t = 
	let 
	    val ts = getUnifier' a c us t
(*	    val substb = sub b ts          *)
	    val substd = sub d ts
	in getUnifier' b substd (union ts us ) t
	end
    
      | getUnifier' (OR(a,b)) (OR(c,d)) us t = 
	let 
	    val ts = getUnifier' a c us t
(*	    val substb = sub b ts          *)
	    val substd = sub d ts
	in getUnifier' b substd (union ts us ) t
	end

      | getUnifier' (IMP(a,b)) (IMP(c,d)) us t = 
	let 
	    val ts = getUnifier' a c us t
(*	    val substb = sub b ts          *)
	    val substd = sub d ts
	in getUnifier' b substd (union ts us ) t
	end

      | getUnifier' (EQ(a,b)) (EQ(c,d)) us t = 
	let 
	    val ts = getUnifier' a c us t
(*	    val substb = sub b ts          *)
	    val substd = sub d ts
	in getUnifier' b substd (union ts us ) t
	end

      | getUnifier' (NOT(a)) (NOT(b)) us t= 
	getUnifier' a b us t

      | getUnifier' (ALL((t1,i1),m)) (ALL((t2,i2),n)) us t =
	let
	    (* Pruefe, ob die Typen der Quantoren Basistypen sind und uebereinstimmen *)
	    val _ = if ((t1 = t2)) then ()  (* andalso ((Typecheck.type_ordnung t1) = 0) ) then () *)
		    else raise notunifiable
	in
	    getUnifier' (ETT.down 1 m 1) (ETT.down 1 n 1) us t
	end
		
      | getUnifier' (EX((t1,i1),m)) (EX((t2,i2),n)) us t =
	let
	    (* Pruefe, ob die Typen der Quantoren Basistypen sind und uebereinstimmen *)
	    val _ = if ((t1 = t2)) then () (* andalso ((Typecheck.type_ordnung t1) = 0) ) then () *)
		    else raise notunifiable
	in
	    getUnifier' (ETT.down 1 m 1) (ETT.down 1 n 1) us t
	end

      | getUnifier' (L((t1,i1),m)) (L((t2,i2),n)) us t = 
	let
	    (* Pruefe, ob die Typen der Quantoren Basistypen sind und uebereinstimmen *)
	    val _ = if ((t1 = t2)) then ()  (* andalso ((Typecheck.type_ordnung t1) = 0) ) then () *) 
		    else raise notunifiable
	in
	    getUnifier' (ETT.down 1 m 1) (ETT.down 1 n 1) us t
	end

      | getUnifier'  (C s) (C t) us _ = if (s=t) then us else
	                              raise notunifiable

      | getUnifier'  (V i) (V j) us _ = if (i <> j) then ( union [(j, V i)] us ) else us

      | getUnifier'  n     (V i) us t = if (i >= t) then
					    if not(ETT.free i n) then ( union [(i, n)] us )
					    else raise notunifiable
				      else raise notunifiable

      (* Normalerweise sollte der naechste Fall unifizieren, aber
       * Vereinbarung : Unifiziere nur den freeterm ( Axiom / Lemma ) und nicht
       *                den fixen Goal Subterm !! 
       *)
   (*   | getUnifier'  (V i) n     us = if not(ETT.free i n) then ( union [(i, n)] us )
				      else raise notunifiable  *)

      | getUnifier' _ _ _ _ = raise notunifiable



    fun getUnifier (fixterm, freeterm) threshold  = 
	( let
	      val unifier  = ( getUnifier' fixterm freeterm nil threshold )
	  in
	      unifier
	  end
	 )
	      handle notunifiable => 
		  raise Error ("Axiom and Subterm not FO-Unifiable \n")
		      

    fun subst' freeterm unifs 0 = freeterm
      | subst' freeterm unifs n = subst' (sub freeterm unifs) unifs (n-1)

    fun subst_wrt_unifier (freeterm, unifs ) =
	(* Muss die Substitutionsliste n mal durchlaufen lassen :
	 *  Worst case : 
	 *                   [ V 0  -->  C 0 ,
	 *		       V 1  -->  V 0 ,
	 *		       V 2  -->  V 1 ] 
	 * ==>    Bei einmaliger Substitution : Aus V 2 -> V 1 ( genuegt nicht ) 
	 *)
	subst' freeterm unifs (List.length(unifs))

end (* of struct *)



(* ==========================================================================================
                             Rewriting.sml
   ========================================================================================== *)

structure Rewriting= 
struct
local open Data Goal in

(* -------------------------------- *)
(* |    Gets the Subterm          | *)
(* -------------------------------- *)

fun getSubTerm (t as (C c)) xs lams = 
    if null(xs) then (t,lams) else raise Error ("Subterm wrong position (End reached - Constant)\n") 
  | getSubTerm (t as (V x)) xs lams = 
	if null(xs) then (t,lams) else raise Error ("Subterm wrong position (End reached - Variable)\n")
  | getSubTerm (t as ((GAMMA _))) xs lams = 
	    if null(xs) then (t,lams) else raise Error ("Subterm wrong position (End reached - Gamma Var) \n")
  | getSubTerm (t as (A(m,m'))) xs lams = if (null(xs)) then (t,lams)
				  else ( case hd(xs) of
					1 => getSubTerm m (tl(xs)) lams
				      | 2 => getSubTerm m'(tl(xs)) lams
				      | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (App)\n") )
					    
  | getSubTerm (t as (L((typ,i),m))) xs lams = if (null(xs)) then (t,lams)
				       else(  case hd(xs) of
					    1 => getSubTerm m (tl(xs)) ((typ,i)::lams)
					  | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (L)\n"))
  | getSubTerm (t as (EX((typ,i),m))) xs lams = if (null(xs)) then (t,lams) 
				       else ( case hd(xs) of
					     1 => getSubTerm m (tl(xs)) ((typ,i)::lams)
					   | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (EX)\n"))
  | getSubTerm (t as (ALL((typ,i),m))) xs lams = if (null(xs)) then (t,lams)
				       else ( case hd(xs) of
					     1 => getSubTerm m (tl(xs)) ((typ,i)::lams)
					   | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (ALL)\n"))

  | getSubTerm (t as (AND(m,m'))) xs lams = if (null(xs)) then (t,lams)
				  else ( case hd(xs) of
					1 => getSubTerm m (tl(xs)) lams
				      | 2 => getSubTerm m'(tl(xs)) lams
				      | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (AND)\n"))

  | getSubTerm (t as (OR(m,m'))) xs lams = if (null(xs)) then (t,lams)
				  else ( case hd(xs) of
					1 => getSubTerm m (tl(xs)) lams
				      | 2 => getSubTerm m'(tl(xs)) lams
				      | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (OR)\n"))
  | getSubTerm (t as (IMP(m,m'))) xs lams = if (null(xs)) then (t,lams)
				  else ( case hd(xs) of
					1 => getSubTerm m (tl(xs)) lams
				      | 2 => getSubTerm m'(tl(xs)) lams
				      | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (IMP)\n"))
  | getSubTerm (t as (EQ(m,m'))) xs lams = if (null(xs)) then (t,lams)
				  else ( case hd(xs) of
					1 => getSubTerm m (tl(xs)) lams
				      | 2 => getSubTerm m'(tl(xs)) lams
				      | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (EQ)\n"))
 | getSubTerm (t as (NOT(m))) xs lams = if (null(xs)) then (t,lams) 
				  else ( case hd(xs) of
					1 => getSubTerm m (tl(xs)) lams
				      | i => raise Error ("Subterm wrong position  " ^ Int.toString(i) ^ " (NOT)\n"))




(* -------------------------------- *)
(* |    Sets the new Subterm      | *)
(* -------------------------------- *)

fun setSubTerm (t as (C c)) xs n = if null(xs) then n else t
  | setSubTerm (t as (V x)) xs n = if null(xs) then n else t
  | setSubTerm (t as (GAMMA _)) xs n = if null(xs) then n else t
  | setSubTerm (t as (A(m,m'))) xs n = if (null(xs)) then n
				  else ( case hd(xs) of
					1 => A(setSubTerm m (tl(xs)) n, m')
				      | 2 => A(m, setSubTerm m'(tl(xs)) n )
				      | i => t )
					    
  | setSubTerm (t as (L((typ,i), m))) xs n = if (null(xs)) then n
					     else ( case hd(xs) of
					    1 => L((typ,i), setSubTerm m (tl(xs)) n )
					  | i => t )

  | setSubTerm (t as (EX((typ,i), m))) xs n = if (null(xs)) then n
				       else ( case hd(xs) of
					    1 => EX((typ,i), setSubTerm m (tl(xs)) n )
					  | i => t )

  | setSubTerm (t as (ALL((typ,i), m))) xs n = if (null(xs)) then n
				       else ( case hd(xs) of
					    1 => ALL((typ,i), setSubTerm m (tl(xs)) n )
					  | i => t )

 
  | setSubTerm (t as (AND(m,m'))) xs n = if (null(xs)) then n
				  else ( case hd(xs) of
					1 => AND(setSubTerm m (tl(xs)) n, m')
				      | 2 => AND(m, setSubTerm m'(tl(xs)) n )
				      | i => t )

  | setSubTerm (t as (OR(m,m'))) xs n = if (null(xs)) then n
				  else ( case hd(xs) of
					1 => OR(setSubTerm m (tl(xs)) n, m')
				      | 2 => OR(m, setSubTerm m'(tl(xs)) n )
				      | i => t )

  | setSubTerm (t as (IMP(m,m'))) xs n = if (null(xs)) then n
				  else ( case hd(xs) of
					1 => IMP(setSubTerm m (tl(xs)) n, m')
				      | 2 => IMP(m, setSubTerm m'(tl(xs)) n )
				      | i => t ) 

  | setSubTerm (t as (EQ(m,m'))) xs n = if (null(xs)) then n
				  else ( case hd(xs) of
					1 => EQ(setSubTerm m (tl(xs)) n, m')
				      | 2 => EQ(m, setSubTerm m'(tl(xs)) n )
				      | i => t )
  | setSubTerm (t as (NOT(m))) xs n = if (null(xs)) then n 
				  else ( case hd(xs) of
					1 => NOT(setSubTerm m (tl(xs)) n)
				      | i => t )


fun get_premisse (AND(a,b)) i = if i <= 1 then a else get_premisse b (i-1)
  | get_premisse t i = if i = 1 then t else raise Error ("No Premisse found !\n")



(* returns the max. Free Variable of the fixterm *)       
val HighestFreeVar = foldl ( fn (x,n)=> case x of (V i) => if i > n then i else n | _ => n ) 0 

(* Umrechnung der Position *)
(* Passe Position der aktuellen Ausgabe an !! *)
(* Spezialfall 0 fuer Formeldeduktion *)
fun getPosSequenz (0::nil) _ = nil
  | getPosSequenz xs (IMP(C "TRUE", _)) = (2::xs)
  | getPosSequenz xs _ = xs
 

(* returns Tupel (match,destination) in Abhaengigkeit 
 * von LEFT / RIGHT 
 *)
fun MatchDest  (EQ(l,r)) d  = if (d = LEFT) then (l,r) else (r,l)
  | MatchDest (IMP(l,r)) d  = if (d = RIGHT) then (r,l) else 
    raise Error ("In case of => Lemmata, please choose <r>")
  | MatchDest  _ _ =  raise Error ("No Equational Axiom/Lemma/Premisse \n") 
	      

(* Translates the Positions List [LA "A", PO 1, PO 2 , ... ]
 * in IntegerList [2,1,1,2,... ]
 *)

fun translabel pter ord =
let
    val label = ref 65 (* initial auf #"A" *)

    fun exp (L((t,i),e)) xs = 
	let val clabel = !label
	in ( label := !label + 1 ; (* erhoehe label counter *)
	    (if (clabel = ord) then xs else exp e (xs @ [1])) )
	end

      | exp (EX((t,i),e)) xs = 
	let val clabel = !label
	in ( label := !label + 1 ; 
	     (if (clabel = ord) then xs else exp e (xs @ [1])) )
	end

      | exp (ALL((t,i),e)) xs = 
	let val clabel = !label
	in ( label := !label + 1 ; 
	    (if (clabel = ord) then xs else exp e (xs @ [1])) )
	end

      | exp t xs = eqexp t xs
	    
    and eqexp (EQ(a,b)) xs =
	let val clabel = !label
	in ( label := !label + 1 ; 
	    (if (clabel = ord) then xs 
	   (* works, because the false side has list nil 
	    * and impexp will be evaluated first !! 
	    *)
	     else ( impexp a (xs @ [1])) @ ( eqexp b (xs @ [2])) ))
	end
						 
      | eqexp t xs = impexp t xs
	    
    and impexp (IMP(a,b)) xs = 
	let val clabel = !label
	in ( label := !label + 1 ;
	    ( if (clabel = ord) then xs 
	      else ( orexp a (xs @ [1])) @ ( impexp b (xs @ [2])) ))
	end

      | impexp t xs = orexp t xs
	    
    and orexp (OR(a,b)) xs =
	let val clabel = !label
	in ( label := !label + 1 ;
	     (if (clabel = ord) then xs 
	      else ( andexp a (xs @ [1]) ) @ ( orexp b (xs @ [2])) ))
	end
	
      | orexp t xs = andexp t xs
	    
    and andexp (AND(a,b)) xs =
	let val clabel = !label
	in ( label := !label + 1 ;
	     ( if (clabel = ord) then xs
	       else ( negexp a (xs @ [1]) ) @ ( andexp b (xs @ [2]) )))
	end
	 
      | andexp t xs = negexp t xs
	    
    and negexp (NOT(a)) xs =
	let val clabel = !label
	in ( label := !label + 1 ;
	    (if (clabel = ord) then xs else negexp a (xs @ [1]) ))
	end
	    
      | negexp t xs = appexp t xs
	    
    and appexp (A(a,b)) xs = 
	( appexp a (xs @ [1])) @ ( atexp b (xs @ [2]) )
	
      | appexp t xs = atexp t xs
	    
    and atexp (V(n)) xs = 
	let val clabel = !label
	in ( label := !label + 1 ;
	    ( if (clabel = ord ) then xs else nil )) 
	end
	
      | atexp (GAMMA(_,i)) xs = nil
	
      | atexp (C(s)) xs = 
	let val clabel = !label
	in ( label := !label + 1 ;
	    ( if (clabel = ord ) then xs else nil )) 
	end

      | atexp t xs = exp t xs
	   
in
    exp pter nil
end (* of translate *)

(* Translates Pos Liste into int liste *)
fun pos2intlist ((LA s)::xs) = raise Error "Parser filters Positionlist - Rewriting Internal #1\n"
  | pos2intlist ((PO i)::xs ) = i::(pos2intlist xs)
  | pos2intlist  nil = nil

(*    
fun debuglist' (i::is) = ( print (Int.toString(i) ^ " "); debuglist'(is))
  | debuglist' nil = ()
fun debuglist is = ( print( "\n[" ) ;  debuglist' is ; print ("]\n") )
*)

fun translate _ (liste as ((PO i)::xs))   = pos2intlist liste
  | translate pter (liste as ((LA s)::xs)) = 
    let val c = ( case (Char.fromString s ) of 
		 (SOME c) => c 
	       | (NONE) => raise Error "Position List Error, please use [LABEL](1|2)* \n" )
	
	val newlist = ( case pter of 
		       (IMP (C"TRUE", p)) => translabel p (Char.ord(c))
		     | _ => raise Error "Labels in ND-Mode not available\n" )   
    in
	(newlist @ (pos2intlist xs))
    end
 | translate _ nil = nil
(* | translate _ _ = raise Error "Translation Process Pos2Int List Error - Rewriting #2 \n" *)
    
	    
(* -------------------------------- *)
(* |    Main Procedures           | *)
(* -------------------------------- *)

fun BetaEtaRed goal (xs,mode) = 
    let val pter = Goal.get_first_goal(goal)
	val position = getPosSequenz xs pter
	(* Hole betreffenden Subterm *)
        val (subterm,_) = getSubTerm pter position nil
	(* ruft den Beta/Eta-Reduktions Algorithmus in Lambda.sml auf *)
	val red_subterm = 
	    case mode of
		BETA => Lambda.justbeta(subterm)
	      | ETA => Lambda.justeta(subterm)
	      | _ => raise Error ("in BetaEtaRed Rewriting Module \n")
	(* Ersetze den alten subterm durch den beta/etareduzierten *)
	val newpter = setSubTerm pter position red_subterm
	val newgoal = Goal.update_first_goal(goal, newpter)
    in
	(Goal.print_goal(newgoal); newgoal )
    end


(* Nimmt den subterm(fixterm), das entaprechende
 * Lemma/Axiom in Form einer Gleichung (A=B) und den lamlevel
 *
 * Der Matchingalgorithmus versucht A mit dem fixterm zu matchen
 * und gibt B ( entsprechend gematcht aus 
 *)
fun startMatching (fixterm, d) (freeterm) lamlevel =
let
	(* Passe freeterm an den Lambda Level des Fixterms an *)
	val def  = ETT.up 0 freeterm lamlevel 

	(* Muessen dafuer sorgen, dass die Menge der subterm/fixterm Freien Variablen und
	 * die Menge der def Variablen disjunct sind !! 
	 * => Hebe alle freien Variablen in def um die max. Variable + 1 des subterm an
	 *)
	val disjunctup    =((HighestFreeVar (ETT.getfreevars fixterm)) + 1)
	val tempdef       = ETT.up 0 def disjunctup
	
	                                                            

	(* axiom name = ( A = B ) 
	 * 
	 *  If   d = LEFT  ==> (match, destination) = ( A, B )
                 d = RIGHT ==>                      = ( B, A )
	 *)

	val (match, destination) = MatchDest tempdef d
	  
	(* DEBUG 
	 val _ = print ("AXIOM/PREMISSE FREE (LEFT) : " ^ (Visual.pter2string match) ^ "\n" )
         val _ = print ("AXIOM/PREMISSE FREE (RIGHT): " ^ (Visual.pter2string destination) ^ "\n\n" )
	 *)

	(* Starte Unifikation *)
	val unifier = Unify.getUnifier (fixterm, match) (lamlevel + disjunctup)
	(* DEBUG val _ = ( print ("Unifier :\n") ; Unify.show_unifier(unifier) ) *)
	
	(* noch temporaer angehobene destination *)
	val newdest_up = Unify.subst_wrt_unifier (destination, unifier)
	(* mache temp anhebung an den ausstehenden Variablen des freeterms wieder rueckgaengig *)
	val newdest    = ETT.down disjunctup newdest_up disjunctup
	(* DEBUG val _ = print (("NEUER SUBTERM : " ^ Visual.pter2string newdest) ^ "\n") *)
in
    newdest
end



(* ---------------------------------------------------------------------
 *                              [ MAIN ]
 * ---------------------------------------------------------------------
 *)


(* -------------------------------------------------------------- *)
(*                     Beta Eta - Fall                         *)
(* -------------------------------------------------------------- *)

fun main'(goal, (xs, BETA)) = BetaEtaRed goal (xs,BETA)
  | main'(goal, (xs, ETA))  = BetaEtaRed goal (xs,ETA)


(* -------------------------------------------------------------- *)
(*                   Equality  - Fall                             *)
(* -------------------------------------------------------------- *)

  | main'(goal, (xs, EQUALITY(f))) = 
    let 
	val pter = Goal.get_first_goal(goal)
	val position = getPosSequenz xs pter
	val (subterm, lambdas) = getSubTerm pter position nil
	val lamlevel = List.length(lambdas)

	(* Hole neue Formel und Uebersetze sie in PTER *)
	val instinit = Goal.init_goal_env(goal)
	val instgoal = Goal.first_goal(instinit, f)
	val formel'  = Goal.get_first_goal(instgoal)
	val formel = ( case formel' of
		      (IMP(C "TRUE", m )) => m
			  | _ => raise Error ("Internal Error in Rewriting Module (1)\n") )

	(* Formel must be an equality !! *)
	val newdest = startMatching (subterm, LEFT) (formel) lamlevel
	(* Setze den gematchten Term *)
	val newpter = setSubTerm pter position newdest
	
	val mergegoal = Goal.merge_envs(goal, instgoal)
	val newgoal'  = Goal.update_first_goal(mergegoal, newpter)
	val newgoal'' = Goal.split_goal(newgoal', formel') (* formel' wegen IMP(C"TRUE",formel) Invarianz *)
	val newgoal = Goal.kaper_check(newgoal'')
    in
	(Goal.print_goal(newgoal); newgoal )
    end

(* -------------------------------------------------------------- *)
(*                     Praemissen - Fall                          *)
(* -------------------------------------------------------------- *)


  | main'(goal, (xs, (PREMISSE(nr,d)) ) ) = 
    let 
	(* remove_dbl loescht das Doppelauftreten der Variablen in der Liste *)
	fun remove_dbl xs = foldr ( fn (x,nil) => [x] | (x,ys) => if x = hd(ys) then ys else x::ys ) 
	    nil (Listsort.sort String.compare xs)

	(* Vergleicht zwei sortierte (!) Listen ohne doppelauftreten
	 * und gibt die Unterschiede in einer Liste aus *)
	fun intersection xs nil = nil
	  | intersection nil ys = nil
          | intersection (x::xs) (y::ys) = if (x=y) then (x::(intersection xs ys))
					 else if (x < y) then (intersection xs (y::ys))
					     else (intersection (x::xs) ys) 

	(* Gibt eine Liste von allen allquantifizierten gebundenen Variablen 
	 * in einer Praemisse zurueck *)
	fun getPremisseBnds (EQ(_,_)) = nil 
          | getPremisseBnds (ALL((t,i), m)) = (t,i)::(getPremisseBnds m)
          | getPremisseBnds _ = raise Error ("You can't apply Rewriting Rule on this premise \n")

	(* Gibt ein Goal zurueck, dass nun alle freien Variablen 
	 * enthaelt, die vorher gebunden waren *)
	fun Bnd2FreeVars nil goal = goal
          | Bnd2FreeVars ((t,s)::xs) goal = 
	    let
		val freemaps2i = Goal.freemaps2i(goal)
		val known = (freemaps2i s; true) handle Error _ => false
		val newgoal = if known then goal else 
		    Goal.update_ftypeenv(goal, s, t)
	    in Bnd2FreeVars xs newgoal end

	(* Ersetze alle gebundenen Variablen durch die entsprechend
	 * neu engefuehrten freien Variablen *)
	fun translateBnd2Free (t as (EQ(m,m'))) _ = t
          | translateBnd2Free (ALL((t,i), m))  (bmap,fmap) = 
	    let val bndname =  bmap i
		val freenr  =  fmap bndname 
	        val m'      =  (ETT.down 0 (ETT.sub m 0 (ETT.up 0 (V freenr) 1)) 1)
	    in translateBnd2Free m' (bmap,fmap) end
	  | translateBnd2Free _ _ = raise Error ("You can't apply Rewriting Rule on this premise \n")


	val pter = Goal.get_first_goal(goal)
	val bndmapi2s = Goal.bndmapi2s(goal)

	val position = getPosSequenz xs pter
	(* getSubTerm returns the wished subterm and the respect bnd Variables of the fixterm *)
	val (subterm, lambdas) = getSubTerm pter position nil
	val lamlevel = List.length(lambdas)
	
	(* DEBUG val _ = print ("Alter SUBTERM(FIX) : " ^ (Visual.pter2string subterm) ^ "\n") *)
	val def' =  let
			val left = ( case pter of IMP(l,_) => l | t => t )
		    in (* (print(Visual.pter2string(get_premisse left nr) ^ "\n\n")) *)
			get_premisse left nr 
		    end

	(* --------------------------- *)
	(* Sonderfall Kaper Routinen   *)
	(* --------------------------- *)
	(* FreetermBndvarsNr = Liste aller allquantifizierten BndVariables (Zahl)  im Freeterm *)
        val FreetermBndvarsNr    = getPremisseBnds (def')
	(* FreetermBndvarsNames = uebersetzt die Zifferndarstellung in Namen *) 
	val FreetermBndvarsNames' = (map (fn (t,i) => (t, bndmapi2s i)) FreetermBndvarsNr)
	val FreetermBndvarsNames  = remove_dbl (map (fn (t,s) => s ) FreetermBndvarsNames')
	(* FixtermBndvarsNames = Liste aller im Fixterm gebundenen Variablen *)
	val FixtermBndvarsNames  = remove_dbl (map (fn (_,i) =>  bndmapi2s i) lambdas)
	(* macht die neuen freien Variablen im Goal bekannt *)
	val newvargoal   = Bnd2FreeVars FreetermBndvarsNames' goal
	val freemaps2i   = Goal.freemaps2i(newvargoal)
	val freemapi2s   = Goal.freemapi2s(newvargoal)
	(* Neuer Freeterm jetzt nicht mehr gebunden !!! *)
	val freedef      = translateBnd2Free def' (bndmapi2s, freemaps2i)
	

	(* Berechne neues unifiziertes N ===> > N' < *)
	val newdest = startMatching (subterm, d) freedef lamlevel
	val newdestFreeVarsNr = ETT.getfreevars newdest 
	val newdestFreeVarsNames = 
	    remove_dbl ( map ( fn (V i) => freemapi2s i 
	  | _ => raise Error ("Internal Error in Rewriting Module (2)"))
			newdestFreeVarsNr)
	    
	(* allegekaperten Variablen ! *)
	val intersect = intersection newdestFreeVarsNames FixtermBndvarsNames
	(* pruefe nun, ob kapern erlaubt war !! *)
	val _ = foldl ( fn (s,_) => 
		       if (List.exists (fn z => z = s) FreetermBndvarsNames) then () 
		       else raise Error ("Free Variable "^s^" was captured --> Missing Allquantifier !\n"))
	    () intersect
	    

	val newpter = setSubTerm pter position newdest
	
	val newgoal' = Goal.update_first_goal(newvargoal, newpter)
	val newgoal  = Goal.kaper_check(newgoal')
			     
    in
	(Goal.print_goal(newgoal); newgoal )
    end


(* -------------------------------------------------------------- *)
(*                     Axiom/Lemma - Fall                         *)
(* -------------------------------------------------------------- *)


 | main'(goal, (xs, (LAXIOM(name,d)) ) ) = 
    let 
	val pter = Goal.get_first_goal(goal)
	(* val _ = (print ("DEBUG XS: ");  foldl ( fn(x,_) => print (Int.toString(x) ^ " ") ) () xs ) *)
	val position = getPosSequenz xs pter
	(* getSubTerm returns the wished subterm and the respect lambda level *)
	val (subterm, lambdas) = getSubTerm pter position nil
	val lamlevel = List.length(lambdas)
	(* DEBUG val _ = print ("Alter SUBTERM(FIX) : " ^ (Visual.pter2string subterm) ^ "\n") *)
	val def' = Goal.get_definition(goal, name)
		
	val newdest = startMatching (subterm, d) def' lamlevel
	val newpter = setSubTerm pter position newdest
	
	val workgoal = 
	    case ( Goal.get_definitions_kind(goal, name) ) of
		LEMMA => Apply.addLemma(goal, name)
	      | _     => goal                                 
	   
	val newgoal' = Goal.update_first_goal(workgoal, newpter)
	val newgoal  = Goal.kaper_check(newgoal')
			     
    in
	(Goal.print_goal(newgoal); newgoal )
    end


(* In case of Labeling, we have to translate the pos list first *)
fun main (goal, (ps, mode)) = 
let 
    val pter = Goal.get_first_goal(goal)
in
    main' (goal, ((translate pter ps), mode))
end


end (* of local open *)

end (* of structure Rewriting.sml *)
(* ==========================================================================================
                             RefSymTrans.sml
   ========================================================================================== *)

structure RefSymTrans= 
struct
local open Data Goal in

(* Wende Reflexivitaet, Symmetrie und Transitivitaet
 * nur auf erste Conclusion an !!!
 *)


(* ******************************** *)
(* Gleichheitsregel : REFLEXIVITAET *)
(* ******************************** *)

fun reflexiv (goal) = 
    let
	val pter = Goal.get_first_goal(goal)
	val (conclusion) = 
	    ( case pter of
		  (IMP(l,r)) => r 
		| _ => raise Error ("Internal Error in RefSymTrans\n") )

	val (left, right) = ( case conclusion of
			    (EQ (l,r))       => (l,r) 
                          | (OR (EQ(l,r),_)) => (l,r)
			  | _         => raise Error ("You can only apply Reflexivity-Rule to equalities"))

	(* Wenn sie wirklich gleich sind, dann genuegt nur noch die Conclusion *)
	val newpter = if (ETT.equal(left,right)) then (C "TRUE") else pter
	val newgoal = Goal.update_first_goal(goal, newpter)
    in
	(Goal.print_goal(newgoal) ; newgoal)
    end


(* ******************************** *)
(* Gleichheitsregel : SYMMETRIE     *)
(* ******************************** *)

fun symmetrie (goal,d) = 
    let
	val pter = Goal.get_first_goal(goal)
	val (premise, conclusion) = 
	    ( case pter of
		  (IMP(l,r)) => (l,r)
		| _ => raise Error ("Internal Error in RefSymTrans\n") )

	val newpter = ( case d of 
                        RIGHT => ( case conclusion of
				  (EQ (l,r))          => (IMP(premise, EQ(r,l)))
				| (OR (EQ(l,r),rest)) => (IMP(premise, OR( EQ(r,l), rest)))
				| _         => raise Error ("You can only apply Symmetric-Rule to equalities\n"))

		      | LEFT  => ( case premise of
				  (EQ (l,r))          => (IMP(EQ(r,l), conclusion))
				| (AND (EQ(l,r),rest))=> (IMP(AND( EQ(r,l), rest), conclusion))
				| _         => raise Error ("You can only apply Symmetric-Rule to equalities\n")) )
			    
	val newgoal = Goal.update_first_goal(goal, newpter)
    in
	(Goal.print_goal(newgoal) ; newgoal)
    end


(* *********************************** *)
(* Gleichheitsregel : TRANSITIVITAET   *)
(* *********************************** *)

val err = ("You can apply transitivity only on equalities\n") 
fun trans_conclusion' formel goal = 
    let
	val pter = Goal.get_first_goal(goal)
	val (left,right) = 
	   ( case pter of
		IMP(l,r) => (l,r)
	      |_     => raise Error("Interner Fehler in RefSymTrans 1\n"))

	(* Update formel *)
	val updatepter = IMP( left, ( case right of 
				     EQ(l,_) => (EQ(l, formel))
				   | OR(EQ(l,_),B) => (OR(EQ(l, formel), B))
				   | _ => raise Error err ) )
		 
    	val newgoal' = Goal.update_first_goal(goal,updatepter)

	(* neues goal *)
	val newpter = IMP( left, ( case right of 
				  EQ (_,r) => (EQ(formel, r))
				| OR(EQ(_,r), B) => (OR(EQ(formel, r), B))
				| _ => raise Error err ) )
	
	val newgoal = Goal.split_goal(newgoal', newpter) 
    in 
	newgoal 
    end

fun trans (goal, f) = 
    let
	val transinit = Goal.init_goal_env(goal)
	val transgoal = Goal.first_goal(transinit, f)

	val formel' = Goal.get_first_goal(transgoal)
	val formel  = ( case (formel') of
		       IMP(C"TRUE", p) => p
		     | _ => raise Error ("Internal Error: Transitivity\n") )

	val mergegoal = Goal.merge_envs(goal, transgoal)
	val newgoal   = trans_conclusion' formel mergegoal
    in
	(Goal.print_goal(newgoal) ; newgoal)
    end


end (* of local open *)

end (* of structure RefSymTrans.sml *)
(* ==========================================================================================
                             Insert.sml
   ========================================================================================== *)

structure Insert= 
struct
(* Insert.sml implements the Equality Rule : Insertion 
 *    insert [l/r] name formula 
 * if M=N andalso type(name) = type(formula) 
 *   then M[name:=formula] = N[name:=formula] else Error
 *)


local open Data Goal in


fun main (goal, (d:xdirection, (name:string, f:sourceexpr)) ) = 
    let
	val pter = Goal.get_first_goal(goal)
	val (premise, conclusion) = 
	    (  case pter of 
		   IMP(l,r) => (l,r) 
		 | _ => raise Error ("Internal Error 1 Insert.sml\n") )

	(* Hole die betreffende Gleichung *)	    
	val (M,N) = ( case d of 
		      LEFT => ( case premise of 
			       EQ(a,b) => (a,b)
			     | AND(EQ(a,b),rest) => (a,b) 
			     | _ => raise Error ("There must be an equality, in which you can substitute "
						  ^ "Variable " ^ name ^ "\n")) 
		    | RIGHT => ( case conclusion of
				EQ(a,b) => (a,b)
			      | OR(EQ(a,b),rest) => (a,b) 
			      | _ => raise Error ("There must be an equality, in which you can substitute "
						  ^ "Variable " ^ name ^ "\n")) )

	 (* Hole Variablen Nr von name und Typ *)
	val varnr = ( Goal.freemaps2i(goal) ) name 
	val vartype = (Goal.typi(goal)) varnr

	val _ = if (ETT.free varnr M) andalso (ETT.free varnr N)
		    then () 
		else raise Error ("Equality does not contain free variable ["
				  ^ name ^ "] on both sides \n")

	(* Generiere Formel pter *)
	val insertinit = Goal.init_goal_env(goal)
	val insertgoal = Goal.first_goal(insertinit, f)
	val formel = ( case (Goal.get_first_goal(insertgoal)) of
		      IMP(C "TRUE",r) => r
		    | _ => raise Error ("Internal Error 4 Insert.sml") )

	(* Muss goals wieder mergen *)
	val mergegoal = Goal.merge_envs(goal, insertgoal)

	
	(* Checke jetzt die Types von name und formel *)
	val _ = if (Instance.checktype(formel, vartype) mergegoal) then () 
		else raise Error ("Sorry, types of name and insertion are different \n")

	(* Nehme nun Einfuegung vor *)
	val (M',N') = (ETT.sub M varnr formel, ETT.sub N varnr formel)

	val newpter = IMP ( case d of 
			   LEFT => ( (case premise of 
				      EQ _ => EQ(M',N')
				    | AND(EQ _ ,rest) => AND(EQ(M',N'),rest) 
				    | _ => raise Error "Internal Error 5 Insert.sml" ), conclusion)

			 | RIGHT =>(premise, ( case conclusion of
					      EQ _  => EQ(M',N')
					    | OR(EQ _ ,rest) => OR(EQ(M',N'),rest) 
					    | _ => raise Error ("Internal Error 6 Insert.sml"))) )
			       
	val newgoal' = Goal.update_first_goal(mergegoal, newpter)
	val newgoal =  Goal.kaper_check(newgoal')
    in
	(Goal.print_goal(newgoal); newgoal)
    end



end (* of local open *)

end (* of structure Insert.sml *)
(* ==========================================================================================
                             Main.sml
   ========================================================================================== *)

structure Main= 
struct
(* Main.sml - Das Herz des Proof Assistenten
 *
 * In einer Hauptschleife werden die Standart Eingabe abgefragt, 
 * Ausdruecke geparst und durch Aufrufen der entsprechenden 
 * Subroutinen goals modifiziert.
 *)

local open BasicIO Nonstdio Data in
exception fileEnd 
exception exitProgram

fun welcome() = print ("NED Version 1.3\nType \"exit\" to quit " ^
		       "the program\n\"help\" gives more information\n\n"); 

fun errorpos i = "" (* (implode (List.tabulate (i+3, (fn n => #" ")))) ^ "^" *)

fun createLexerStream (is : instream) =
  Lexing.createLexer (fn buff => fn n => Nonstdio.buff_input is buff 0 n)

(* counts the Lines and chars*)
val currline = ref 1
val currchar = ref 0
val prompt   = ref true

(* Initiales,Backup und Save Goal *)
val initgoal = Goal.init_goal()
val backup   = ref (Goal.init_goal())  

(* returns (goal,savegoal) *)
(* Take current goal in history, if the last proof has finished, set the backup goal *)
fun getGoal(goalxs) =
    let val (headgoal, _) = hd(goalxs) 
	val nr = Goal.get_nr_of_goals(headgoal)
    in  
	if ( (nr > 0) andalso (Goal.get_first_goal(headgoal) = (C "PROOF DONE")))
	    then (Goal.init_goal_env(!backup), (!backup) ) 
	else (headgoal, initgoal)
    end


(* ------------------------------------
 *            Action Handler 
 * ------------------------------------ *)

fun ActionHandler (result as (CmdS(c))) goalxs = 
 let 
     val (goal,_) = getGoal(goalxs)
 (* ND-KOMMANDOS *)
 in   ( case c of 
       POSCMD i       => (Position.main(goal,i); goalxs )
     | HELPCMD        => (Hilfe.main() ; goalxs )
     | PRINTCMD       => (Goal.print_goal(goal); goalxs )
     | SHOWLABELSCMD  => (Visual.set_showlabels(); goalxs)
     | LATEXCMD s     => (LatexOutput.main((goalxs), s); goalxs) 
     | INTROCMD(m)    => (( (Intro.main(goal,m)), result)       :: goalxs )
     | ELIMCMD(m)     => (( (Elim.main (goal,m)), result)       :: goalxs )
     | REMOVECMD(d,i) => (( (Remove.main(goal,(d,i))), result)  :: goalxs )
     | MOVECMD(d)     => (( (Move.main (goal, d )), result)     :: goalxs )
     | SHIFTCMD(xd,yd,i) =>(( (Shift.main(goal, xd, (yd, i) )), result) ::goalxs ) 
     | GOALSHIFTCMD(i)=> (( Shift.goalmain(goal,i), result )    :: goalxs )
     | SIMPLIFYCMD    => (( (Bool.main(goal) ), result)         :: goalxs )
     | LAMCMD         => (( (Lambda.main(goal) ), result)       :: goalxs )
     | APPLYCMD(args) => (( (Apply.main(goal, args )), result)  :: goalxs )
     (* Wende die Cut-Regel an *)
     | CUTCMD (f)     =>  ( ( (Cut.main(goal, f)), result) :: goalxs )
     (* Instanziierung der Gammavariablen i mit Ausdruck f - instance i = f *)
     | INSTANCECMD(i,f) =>( (( Instance.main(goal,i,f) ), result) :: goalxs )
     | RESETCMD       => (print("> RESET DONE\n"); 
			  currline := 1; currchar := 0;
			  backup := (initgoal);
			  ((initgoal , EmptyS)::nil) ) 
     | UNDOCMD(i)     => if (List.length(goalxs) > i ) 
			     then let
				      val newgoalxs = List.drop(goalxs,i)
				      val (newfirst,_) = hd(newgoalxs)
				  in
				      (print "> UNDO successful...\n";
				       Goal.print_goal(newfirst); 
				       newgoalxs )
				  end
			 else raise Error ("Nothing to UNdo\n")
     | EXITCMD     => (print ("Bye\n") ; raise exitProgram)
			     ) (* end of case c *)
 end (* of let *)

(* Gleichheitskommandos *)
 | ActionHandler (result as (CmdEqS(c))) goalxs  = 
 let 
     val (goal,_) = getGoal(goalxs)
 in	  ( case c of
		REFEQ          => ( (RefSymTrans.reflexiv(goal), result)    ::goalxs) 
	      | SYMEQ(d)       => ( (RefSymTrans.symmetrie(goal,d), result) ::goalxs)
	      | TRANSEQ (f)    => ( (RefSymTrans.trans(goal,f), result)     ::goalxs)
	      | REWEQ(args)    => (((Rewriting.main(goal, args)), result)   ::goalxs)
	      | INSERTEQ(args) => ( ((Insert.main(goal,args)), result)      ::goalxs)
		    ) (* end of case *)
 end

(* Erweitere Variablenumgebung - var <name>:typ *)
  | ActionHandler (result as (LetVarS(s,t))) goalxs = 
 let                    
     val (goal,_) = getGoal(goalxs)
     val nr_active_goals = Goal.get_nr_of_goals(goal)
     val newgoal = 
	 if (nr_active_goals > 0 ) 
	     then raise Error ("You can't add new variables within a proof\n")
	 else Goal.update_ftypeenv (goal, s, t)
 in
     (print ("> var "^ s ^ ": " ^ Visual.ty2string(t) ^"\n" );
      ((newgoal, result)::goalxs) )
 end 

  (* Erweitere Konstantenumgebung - const <name>:typ *)
  | ActionHandler (result as (LetConstS(s,t))) goalxs =
 let     
     val (goal,_) = getGoal(goalxs)
     val nr_active_goals = Goal.get_nr_of_goals(goal)
     val newgoal = 
	 if (nr_active_goals > 0 ) 
	     then raise Error ("You can't add new constants within a proof\n")
	 else Goal.update_ctypeenv (goal, s, t)
 in
     (print ("> const "^ s ^ ":" ^ Visual.ty2string(t) ^ "\n");
      ((newgoal, result)::goalxs) )
 end

  (* Deklaration von Definitionen - def <name>:<ausdruck>    hier: mode=CLASSIC
   *                              lemma <name>:<ausdruck>          mode=LEMMA
   *                              axiom <name>:<ausdruck>          mode=AXIOM
   *)	       
  | ActionHandler (result as (DefS(s,f,mode))) goalxs = 
 let
     val (goal,_) = getGoal(goalxs)
     val nr_active_goals = Goal.get_nr_of_goals(goal)
     val name = case mode of CLASSIC => "def" | LEMMA => "lemma" | AXIOM => "axiom"
     val newgoal =
	 if (nr_active_goals > 0 ) 
	     then 
		 raise Error ("You can't add new definitions within a proof\n")
	 else 
	     Goal.new_definition(goal, s, f, mode ) 
 in
     (print ("> " ^ name ^ " " ^ s ^ " saved \n");
      ( (newgoal, result)::goalxs) )
 end

  (* Typneudeklaration - type <name> *)
  | ActionHandler (result as (TypS(s))) goalxs   =
 let
     val (goal,_) = getGoal(goalxs)
     val nr_active_goals = Goal.get_nr_of_goals(goal)
     val newgoal = 
	 if ( nr_active_goals > 0 ) 
	     then raise Error ("You can't add new types within a proof\n")
	 else Goal.new_type(goal, s)
 in
     (print ("> type " ^ s ^ "\n") ; 
      ((newgoal, result)::goalxs) )
 end

  (* Neues goal - goal <ausdruck> *)
  | ActionHandler (result as (GoalS(f))) goalxs  = 
 let 
     val (goal,_) = getGoal(goalxs)
     val nr_goals = Goal.get_nr_of_goals(goal)
 in 
     if ( nr_goals = 0 ) then (* Das erste Goal wurde deklariert *)
	 let val newgoal = Goal.first_goal(goal, f)
	 in
	     if Goal.typecheck(newgoal) then
		 (
		  (* speichere das Goal, das alle Var./Const/Ax/Lemma
		   * Deklarationen enthaelt, ohne Goalspezifikation 
		   *)
		  backup := newgoal;						  
		  Goal.print_goal(newgoal) ; 
		  ((newgoal, result)::goalxs))
	     else raise Error ("Ill-Typed Formula ( must have base type bool )!\n") 
	 end
     else ( Goal.print_goal(goal) ; 
	   raise Error ("There are already active goals \n"))
 end
	       
  (* Das bewiesene Goal soll unter <s> gespeichert werden *)
  | ActionHandler (result as (SaveS(s))) goalxs  = 
 let
     val (goal, savegoal) = getGoal(goalxs)
     val check = Goal.get_nr_of_goals(savegoal)
     val newgoal = 
	 if (check > 0) 
	     then let val axpter = 
		 ( case (Goal.get_first_goal(savegoal)) of
		       IMP(C "TRUE", p) => p
		     | _ => raise Error ("Save Main Internal Error \n") )
		  in
		      (print ("Lemma <" ^ s ^ "> saved \n") ;
		       Goal.save_goal(goal, s, axpter)      )
		  end
	 else raise Error ("There is no goal to save \n")
 in 
     ((newgoal, result)::goalxs)
 end

  (* fuer direkte Formeleingabe *)
  | ActionHandler (ForS(f)) _  = raise Error ("Please use <goal formula> for input \n")
		   
  (* fuer leere inputs *)
  | ActionHandler (EmptyS) goalxs =  (prompt:=false; goalxs)
		   
  (* Parser/Lexer Errors *)
  | ActionHandler (result as (ErrorS(msg))) goalxs = 
    let
	val goal =( case goalxs of 
		 ((g,_)::xs) => g 
	       | _ => raise Error ("Internal Error Main.sml \n"))
    in  ( print msg; ((goal, result)::goalxs) )
    end
		   
  | ActionHandler (FileEndS) _ = raise fileEnd
		   
(* end of ActionHandler *)
		   
				     

(* Entry Point of NED *)
fun start(args) = 
    let 
	val _ = welcome()
	datatype input_mode = FILESTREAM | STDINSTREAM 
	val (stream, mode) = 
	    ( case args of 
		  (s::nil) => ( (( Nonstdio.open_in_bin s ), FILESTREAM ) 
			       handle SysErr _ => 
				   (print ("-- Error: File <"^s^"> not found --\n\n"); 
				    (std_in, STDINSTREAM )) )
		    | _ => (std_in,STDINSTREAM)                                         ) 
        
        val buffer  = createLexerStream (stream)

	val currbuf  = ref (buffer)
	val currmode = ref (mode)

	val initentry = (initgoal, EmptyS)

	fun switch_buffer() = if ((!currmode) = FILESTREAM) then 
	                        ( BasicIO.close_in stream;
				  currbuf := createLexerStream (std_in);
				  currline := 1 ;
				  currmode := STDINSTREAM )
			       else () 
					
	 
	(* Haupt Schleife mit Argument goalxs Liste (History) *)
	fun mainloop goalxs : unit = 
	    ( let 
		  val lexbuf = (!currbuf)
		  val _ = if (!currmode) = FILESTREAM orelse 
			      not (!prompt) 
			      then (prompt:=true) else print ("- ")
		      
		  (* result enthaelt parseexpr *)
		  val result = ( Parser.Main Lexer.Token lexbuf )
				 
		      (* Fehlerbehandlung Parser/Lexer *)
		      handle Lexer.LexicalError (s,l1,l2) 
		      => let val name = Lexing.getLexeme lexbuf
		             val l = (!currline)
			     val c = (!currchar)
			 in (switch_buffer(); 
			     currline := (!currline) - 1;
			     (ErrorS (  "\n> Lexer Error : " ^ s 
				      ^ "\n  BAD SYMBOL : " ^ name 
				      ^ "\n  in Line : " ^ Int.toString(l)
				      ^ "  at Pos : " ^ Int.toString(l1-c) 
				      ^ "-" ^ Int.toString(l2-c) ^ "\n" )) )
			 end
 		      
		    | Parsing.ParseError ob => 
		      let val pos1 = Lexing.getLexemeStart lexbuf
			  val pos2 = Lexing.getLexemeEnd lexbuf
			  val name = Lexing.getLexeme lexbuf
			  val l = (!currline)
			  val c = (!currchar)
			      
		      in  (switch_buffer();
			   currline := (!currline) - 1;
			   (ErrorS  (  "\n> Parser Error"
				    ^ "\n  BAD SYMBOL : " ^ name 
				    ^ "\n  in Line : "^ Int.toString(l)
				    ^ "  at Pos : " ^ Int.toString(pos1-c)
				    ^ "-" ^ Int.toString(pos2-c) ^ "\n")) )
			
		      end 
		    | Error msg => 
		      let
			  val l = (!currline)
		      in (switch_buffer(); 
			  currline := (!currline) - 1;
			  ErrorS ("> " ^ msg ^ "\n  in Line : " ^ Int.toString(l) ^ "\n"))    (* Fehler *) 
		      end
		
	
		  val (goal,_) = getGoal(goalxs)
		  val poschar  = Lexing.getLexemeEnd lexbuf
		  val newgoalxs = ( ActionHandler result goalxs )
		      handle fileEnd => (print ("> Input-File successfully loaded \n");
					 switch_buffer(); goalxs ) 
 			   | Error msg => (print ("> Error : " ^ msg 
						  ^ "  in Line : " ^ Int.toString(!currline) ^ "\n");
					   switch_buffer();
					   ((goal, ErrorS "FEHLER")::goalxs) )     (* Fehler *)  
	    
	      in
		 ( currline := (!currline) + 1 ;
		   currchar := poschar ;
		   mainloop newgoalxs )
	      end
	  )  handle exitProgram => () 

    
    in
	mainloop (initentry::nil)
    end




end; (* end of local open *)
end (* of structure Main.sml *)

fun load (s) = Main.start (s::nil) 

fun start () = Main.start nil 

end


val _ = print ("Start NED with:\nNED.start()  or\nNED.load(NED_FILE)\n\n\n ")
